#ifndef _CGI_DEBUG_H_
#define _CGI_DEBUG_H_
/*
cgi_env() -
This function will print out the current environment variables on the
system.  This function will not send out the mime-types automatically.
*/
void cgi_env(void);

/*
list_print() -
This function will print out the key/value pairs in our linked list.
They will be formatted in a Definition List without any headers
being assigned.  The head pointer the user has must be passed to it.
*/
void list_print(LIST *head);


/*
list_clear() -	
The list_clear() function will clear the linked list and free up the memory.
*/
void list_clear(LIST *head);

/*
list_count() -
The list_count() function will return the number of items that
exist in the linked list.
*/
long list_count(LIST *head);

/*
list_delete_entry() -
This function will delete the specified entry in the list, if NULL is supplied
for the value then the first key found will be deleted, if NULL is supplied for
the key then the first matching value will be deleted, if both the key and 
value are supplied then the key and value must match in order for the node to
be deleted.  A zero value will be returned if the deletion was not sucessful,
and a non-zero value will be returned if it was successful.
*/
int list_delete_entry(LIST *head, char *key, char *value);

/*
list_delete_key_multi() -
This function will delete the list of specified entries based on the key passed
. This assumes that multiple entries with this one named key exist.  A zero 
will be returned if the deletion was not successful, and a non-zero value will 
be returned if it was successful at deleting any items, assuming any were found.
*/
int list_delete_key_multi(LIST *head, char *key);

/*
list_delete_val_multi() -
This function will delete the list of specified entries based on the value 
passed.  This assumes that multiple entries with this one name value exist.
A zero will be returned if the deletion was not successful, and a non-zero 
value will be returned if it was successuful at deleting any items, assuming
any were found.
*/
int list_delete_val_multi(LIST *head, char *value);


/* 
find_val() -
This function will search the linked list and return the value in the first 
node that it finds that matches the specified key. If none is found then NULL 
is returned.
*/
char *find_val(LIST *head, char *key);

/*
find_key() -
This function will search the linked list and return the key in the first node
that it finds that matches the specified value. If none is found then NULL is 
returned.
*/
char *find_key(LIST *head, char *value);

/*
find_val_multi() -
This function will return  the number of values found, If 0 are returned then 
it means that no values were found.  The actual values are being held in the
Vals variable, in order to use this variable the address of a **Var variable 
must be passed.  Space is being allocated for Vals, so it is upto the user to
free it	up when they are done.  You must pass the value to be searched for.
*/
long find_val_multi(LIST *head, char *key, char ***Vals);

/*
find_key_multi() -
This function will return the number of values found.  If 0 is returned then it means that no values were found.  The actual values are being held in the Vals
variable, in order to use this variable the address of a **Val variable must be
passed.  Space is being allocated for Vals, so it is upto the user to free it 
up when they are done.  The key to be searched for must be given.
*/
long find_key_multi(LIST *head, char *value, char ***Vals);
#endif
