
#ifndef _ASF_API_H_
#define _ASF_API_H_

/*---- Delear the following types as described in ASF Spec. ------*/
#ifndef MY_DATA_TYPE
#define MY_DATA_TYPE
typedef unsigned char   BYTE;
typedef unsigned short  WORD;   // 2 bytes
typedef unsigned int    DWORD;  // 4 bytes
typedef double          QWORD;  // 8 bytes
typedef long            LONG;   // 4 bytes
#endif

//#define Bitrate_320		384000
//#define Bitrate_640		768000
//#define Bitrate_160		128000
//#define ASF_MAX_BITRATE 512000        // 500Kbps
/**************************************************************************/
/*  ASF file (Broadcast)                                                  */
/*    +-----------------------------+                                     */
/*    |        Header Object        |                                     */
/*    +-----------------------------+                                     */
/*    |    File Property Object   |   <--+-- Objects in Header Object and */
/*    +---------------------------+      |   can be in any order.         */
/*    |   Stream Property Object  |   <--+                                */
/*    +---------------------------+      |                                */
/*    |  Header Extension Object  |   <--+                                */
/*    +---------------------------+      |                                */
/*    |     Codec List Object     |   <--+   <-- Optional                 */
/*    +-----------------------------+                                     */
/*    |                             |                                     */
/*    |         Data Object         |                                     */
/*    |                             |                                     */
/*    +-----------------------------+                                     */
/*    |    Simple Index Object      | <-- Not required for broadcast.     */
/*    +-----------------------------+                                     */
/**************************************************************************/


/*----------------------------------------------------------------*/
typedef struct S_ASF_FILE_PROPERTY {
  BYTE   objectID[16];                //   ASF_File_Properties_Object
  //QWORD  objectLen;                   //   This value must be larger than 104.
  DWORD  objectLen;                   //   This value must be larger than 104.
  DWORD  dummy1;
  BYTE   fileID[16];                  // @ Unique identifier for this file
  //QWORD  fileLen;                     //   Invalid if Broadcast flag is set to 1.
  DWORD  fileLen;                     //   Invalid if Broadcast flag is set to 1.
  DWORD  fileLen2;                     //   Invalid if Broadcast flag is set to 1.
  QWORD  createDate;                  // @ Maybe invalid if Broadcast flag is set to 1.
  //QWORD  dataPacketCount;             //   Invalid if Broadcast flag is set to 1.
  DWORD  dataPacketCount;             //   Must set to 1 to support Media Player v6.0
  DWORD  dummy2;
  //QWORD  playDuration;                //   Time needed to play the file in 100-nanosecond units,
  DWORD  playDuration; 
  DWORD  playDuration2;
                                      //   Invalid if Broadcast flag is set to 1.
  //QWORD  sendDuration;                //   Invalid if Broadcast flas is set to 1.
  DWORD  sendDuration;
  DWORD  sendDuration2;

  QWORD  preroll;                     //   Amount of time to buffer data before play the file, in millisecond.
  DWORD  broadcast          :1;       //   It should be set to 1 if Broadcast is set to 1.
  DWORD  seekable           :1;       //   Seekable. It should be set to 0.
  DWORD  reserved           :30;      //   Reserved and should be set to 0.
  DWORD  minDataPacketLen;            // @ The Min. data packet size in bytes.
                                      //   It shall be set to the same value as maxDataPacketLen.
  DWORD  maxDataPacketLen;            // @ The Max. data packet size in bytes.
  DWORD  maxBitrate;                  // @ The Max. instantaneous bitrate in bits per second for entire file.
} ASF_FILE_PROPERTY  __attribute__((packed));
//#define ASF_FILE_PROPERTY_LEN  sizeof(ASF_FILE_PROPERTY)  //==104
#define ASF_FILE_PROPERTY_LEN  104

static ASF_FILE_PROPERTY asfFileProperty = {
  { 0xA1, 0xDC, 0xAB, 0x8C, 0x47, 0xA9, 0xCF, 0x11, 0x8E, 0xE4, 0x00, 0xC0, 0x0C, 0x20, 0x53, 0x65 },
  ASF_FILE_PROPERTY_LEN,0,
  { 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
#if 0
  0x179,0,	//fileLen
#else
  0xffffffff,0,	//fileLen, For MP7.0 must be 0xffffffff
#endif
  0,		//createDate
#if 1
  0xffffffff,0,	//DataPacket 
  0xffffffff,0,	//playDuration
  0xffffff,0,	//sendDuration
#else
  0,0,	//DataPacket 
  0,0,	//playDuration
  0,0,	//sendDuration
#endif
  0,	//preroll
  1,	//broadcast
  0,	//seekable
  0,
  //0x02, //live Streaming Live
  0,	//MAX_DATA_PACKET_SIZE,
  0,	//MAX_DATA_PACKET_SIZE,
  0	//maxBitrate
};

/*----------------------------------------------------------------*/
typedef struct S_ASF_MP4S_HEADER {
  DWORD videoObjectStart;             //   0x00010000 (Video object start code)
  DWORD VideoObjectLayerStart;        //   0x20010000 (Video object layer start code)
//      randomAccessVol     :1;       //   0 (Random access vol)
//      ObjectType          :8;       //   0x01 (Video object type indication)
//      isObjectLayer       :1;       //   0 (Is object layer identifier)
//      aspectRatioInfo     :4;       //   1= square
//      volControl          :1;       //   0 (vol control parameters)
//      objectLayerShape    :2;       //   0
//      markerBit1          :1;       //   1       
//      vopTimeIncrement    :16;      // @ 1000 (Resolution of time line)
//      markerBit2          :1;       //   1
//      fixedVopRate        :1;       //   0
//      markerBit3          :1;       //   1
//      objectLayerWidth    :13;      // @ Frame width
//      markerBit4          :1;       //   1
//      ObjectLayerHeight   :13;      // @ Frame height
//WORD  markerBit5          :1;       //   1
//WORD  interlaced          :1;       //   0
//WORD  obmcDisable         :1;       //   1
//WORD  spriteEenable       :1;       //   0
//WORD  not8bit             :1;       //   0
//WORD  quantType           :1;       //   0
//WORD  estimationDisable   :1;       //   1
//WORD  resyncMarkerDisable :1;       // @ Can be 0 or 1
//WORD  dataPartitioned     :1;       //   0
//WORD  scalability         :1;       //   0
//WORD  padding:            :6;       //   Padding to next byte boundary, 0b011111 (A zeor stuffing bit followed by 1
  BYTE  video[10]; 
} ASF_MP4S_HEADER  __attribute__((packed));
#define ASF_MP4S_HEADER_LEN	18


typedef struct S_ASF_VIDEO_MEDIA1 {
  DWORD encodedImageWidth;            // @ Encoded image width in pixels
  DWORD encodedImageHeigh;            // @ Encoded image heigh in pixels
  BYTE  reserved1;                    //   Reserved and should be set to 2.
  //WORD  formatDatalen;                //   Size of format data in bytes.
  BYTE  formatDatalen;                //   Size of format data in bytes.
  BYTE  dummy;
} ASF_VIDEO_MEDIA1  __attribute__((packed));
#define ASF_VIDEO_MEDIA1_LEN  11


typedef struct S_ASF_VIDEO_MEDIA2 {
  //------------------------
  DWORD formatDataSize;               //   Size of format data in bytes.
  LONG  imageWidth;                   // @ Encoded image width in pixels
  LONG  imageHeigh;                   // @ Encoded image heigh in pixels
  WORD  reserved2;                    //   Reserved and should be set to 1
  WORD  bitsPerPixel;                 // @ Number of bits per pixel=24
  DWORD compressedID;                 //   Compression type
  DWORD imageSize;                    // @ Size of image in bytes
  LONG  hPixelPerMeter;               //   Target horizontal resolution inpixels per meter
  LONG  vPixelPerMeter;               //   Target vertical resolution inpixels per meter
  DWORD colorsUsedCount;              //   Color indexes in color table
  DWORD importantColorsCount;         //   Required number of color indexes
  //-------------------------
} ASF_VIDEO_MEDIA2  __attribute__((packed));
typedef struct S_ASF_VIDEO_MEDIA {
  ASF_VIDEO_MEDIA1 m1;		      //size=11	
  ASF_VIDEO_MEDIA2 m2;		      //size=40
  ASF_MP4S_HEADER mp4Header;          // @ Codec specific data type, size=18
  
} ASF_VIDEO_MEDIA  __attribute__((packed));
#define ASF_VIDEO_MEDIA2_LEN  40
#define ASF_VIDEO_MEDIA_LEN  (51+ASF_MP4S_HEADER_LEN) //69
#define ASF_VIDEO_FORMAT_LEN  (ASF_VIDEO_MEDIA_LEN-11) //58

#define ASF_ISO_V2_M4S2  0x3253344D
#define ASF_ISO_V1_MP4S  0x5334504D

/*----------------------------------------------------------------*/
typedef struct S_ASF_VIDEO_STREAM_PROPERTY {
  BYTE  objectID[16];                 //   ASF_Stream_Properties_Object
  //QWORD objectLen;                    //   This value must be larger than 78.
  DWORD objectLen;                    //   This value must be larger than 78.
  DWORD dummy;
  BYTE  streamType[16];               //   ASF_Video_Media
  BYTE  errorCorrectionType[16];      //   Video stream: ASF_No_Error_Corrction
  QWORD timeOffset;                   //   Presentation time offset of the steam in 100-nanosecond, typically 0.
  DWORD specificDataLen;              // @ Type specific data length in bytes
  DWORD errorCorrectionDataLen;       //   Error correction data length in bytes
  WORD  streamNum           :7;       //   Number of this stream (1~127, 0 is invalid)
  WORD  reserved1           :8;       //   Reserved and should be set to 0.
  WORD  EncryptedContent    :1;       //   Set if stream is encrypted.
  DWORD reserved2;                    //   Reserved and should be set to 0.
				      //  ==> 78 bytes
  //-------------------------
  ASF_VIDEO_MEDIA videoMedia;         // @ Type specific data, size=69
  //-------------------------
  // Error correction data            //   Error correction data
} ASF_VIDEO_STREAM_PROPERTY  __attribute__((packed));
#define ASF_VIDEO_STREAM_PROPERTY1_LEN  78	
#define ASF_VIDEO_STREAM_PROPERTY_LEN  (78+69)	//147
#define ASF_VIDEO_STREAM_NUM   1

static ASF_VIDEO_STREAM_PROPERTY asfVideoStreamProperty = {
  { 0x91, 0x07, 0xDC, 0xB7, 0xB7, 0xA9, 0xCF, 0x11, 0x8E, 0xE6, 0x00, 0xC0, 0x0C, 0x20, 0x53, 0x65 },
  ASF_VIDEO_STREAM_PROPERTY_LEN,0,
  { 0xC0, 0xEF, 0x19, 0xBC, 0x4D, 0x5B, 0xCF, 0x11, 0xA8, 0xFD, 0x00, 0x80, 0x5F, 0x5C, 0x44, 0x2B },
  { 0x00, 0x57, 0xFB, 0x20, 0x55, 0x5B, 0xCF, 0x11, 0xA8, 0xFD, 0x00, 0x80, 0x5F, 0x5C, 0x44, 0x2B },
  0,
  ASF_VIDEO_MEDIA_LEN,
  0,
  ASF_VIDEO_STREAM_NUM,
  0,
  0,
  0,
  //-------------------------
  {  {640,			//m1, size=11
     480,
     2,
     //ASF_VIDEO_FORMAT_LEN},
     58,0},	//ASF_VIDEO_FORMAT_LEN
     {ASF_VIDEO_FORMAT_LEN,	//m2, size=40
     640,
     480,
     1,
     24,
     ASF_ISO_V1_MP4S,
     921600,
     0,
     0,
     0,
     0},
     //----- codec specific data, size=18
     {
       0x00010000,
       0x20010000,
       {0x00, 0x84, 0x40, 0xFA, 0x28, 0x50, 0x20, 0xF0, 0xA3, 0x1F}    // 320x240
     }
  }
};
ASF_MP4S_HEADER mp4Hd_320={
       0x00010000,
       0x20010000,
       {0x00, 0x84, 0x40, 0xFA, 0x28, 0x50, 0x20, 0xF0, 0xA3, 0x1F}    // 320x240
};          // @ Codec specific data type, size=18
ASF_MP4S_HEADER mp4Hd_160={
       0x00010000,
       0x20010000,
       {0x00, 0x84, 0x40, 0xFA, 0x28, 0x28, 0x20, 0x78, 0xA3, 0x1F}    // 160x120

};          // @ Codec specific data type, size=18

ASF_MP4S_HEADER mp4Hd_640={
       0x00000100,
       0x20010000,
       {0x00, 0x84, 0x40, 0xFA, 0x28, 0xA0, 0x21, 0xE0, 0xA3, 0x1F}    // 640x480

};          // @ Codec specific data type, size=18
typedef struct S_ASF_AUDIO_MEDIA {
  WORD  codecID;                      //   Codec ID (format tag), G726= 0x0045
  WORD  channelNum;                   // @ Number of channels, 0x0001
  DWORD samplesPerChannel;            // @ Sampling rate in Hertz (cycles per second), 0x00001F40 (8000)
  DWORD bytesPerSecondWORD;           //   Average bytes per second, 0x00000FA0 (4000)
                                      //     G726_16000= 2000
                                      //     G726_24000= 3000
                                      //     G726_32000= 4000    
                                      //     G726_40000= 5000
                                      //     G726_64000= ????
  WORD  blockAlignment;               //   Block alignment in bytes of the audio codec, 0x0001
  WORD  bitsPerSample;                //   Bits per sample of mono data, 0x0004
  WORD  codecSpecificDataLen;         //   Codec specific data size, set to 0
//BYTE  codecSpecificData[];          //   Codec specific data
} ASF_AUDIO_MEDIA  __attribute__((packed));
//#define ASF_AUDIO_MEDIA_LEN  sizeof(ASF_AUDIO_MEDIA)
#define ASF_AUDIO_MEDIA_LEN  18

typedef struct S_ASF_SPREAD_AUDIO {
  BYTE  span;                         // @ Number of packets over which audio will be spread. 
  WORD  virtualPacketLen;             // @ Set to the size of the largest audio payload found in the audio stream
  WORD  virtualChunkLen;              // @ Set to the size of the largest audio payload found in the audio stream
  WORD  slientDataLen;                //   Set to 1
  BYTE  slientData[1];               // @@  Set to 0 ???
} ASF_SPREAD_AUDIO  __attribute__((packed));
//#define ASF_SPREAD_AUDIO_LEN  sizeof(ASF_SPREAD_AUDIO)
#define ASF_SPREAD_AUDIO_LEN  8

typedef struct S_ASF_AUDIO_STREAM_PROPERTY1 {
  BYTE  objectID[16];                 //   ASF_Stream_Properties_Object
  //QWORD objectLen;                    //   This value must be larger than 78.
  DWORD objectLen;                    //   This value must be larger than 78.
  DWORD dummy;
  BYTE  streamType[16];               //   ASF_Video_Media
  BYTE  errorCorrectionType[16];      //   Aideo stream: ASF_Audio_Spread
  QWORD timeOffset;                   //   Presentation time offset of the steam in 100-nanosecond, typically 0.
  DWORD specificDataLen;              // @ Type specific data length in bytes
  DWORD errorCorrectionDataLen;       //   Error correction data length in bytes
  WORD  streamNum           :7;       //   Number of this stream (1~127, 0 is invalid)
  WORD  reserved1           :8;       //   Reserved and should be set to 0.
  WORD  EncryptedContent    :1;       //   Set if stream is encrypted.
  DWORD reserved2;                    //   Reserved and should be set to 0.
} ASF_AUDIO_STREAM_PROPERTY1  __attribute__((packed));
#define ASF_AUDIO_STREAM_PROPERTY1_LEN  78	//78


typedef struct S_ASF_AUDIO_STREAM_PROPERTY2 {
  //------------------------------------------------
  // ----  ASF_AUDIO_MEDIA audioMedia;         // @ Type specific data
  //------------------------------------------------
  WORD  codecID;                      //   Codec ID (format tag), G726= 0x0045
  WORD  channelNum;                   // @ Number of channels, 0x0001
  DWORD samplesPerChannel;            // @ Sampling rate in Hertz (cycles per second), 0x00001F40 (8000)
  DWORD bytesPerSecondWORD;           // @ Average bytes per second, 0x00000FA0 (4000)
                                      //     G726_16000= 2000	7D0h
                                      //     G726_24000= 3000
                                      //     G726_32000= 4000    FA0h
                                      //     G726_40000= 5000
                                      //     G726_64000= ????
  WORD  blockAlignment;               //   Block alignment in bytes of the audio codec, 0x0001
  WORD  bitsPerSample;                //   Bits per sample of mono data, 0x0004
  WORD  codecSpecificDataLen;         //   Codec specific data size, set to 0
//BYTE  codecSpecificData[];          //   Codec specific data

  //------------------------------------------------
  //ASF_SPREAD_AUDIO audioSpread;       // @ Error correction data
  //------------------------------------------------
  BYTE  span;                         // @ Number of packets over which audio will be spread. 
} ASF_AUDIO_STREAM_PROPERTY2  __attribute__((packed));
#define ASF_AUDIO_STREAM_PROPERTY2_LEN  19	//18+1

typedef struct S_ASF_AUDIO_STREAM_PROPERTY3 {
  WORD  virtualPacketLen;             // @ Set to the size of the largest audio payload found in the audio stream, 00A0(160)
  WORD  virtualChunkLen;              // @ Set to the size of the largest audio payload found in the audio stream
  WORD  slientDataLen;                //   Set to 1
  BYTE  slientData[1];               // @  Set to 0 
} ASF_AUDIO_STREAM_PROPERTY3  __attribute__((packed));
#define ASF_AUDIO_STREAM_PROPERTY3_LEN  7	//8-1

//#define ASF_AUDIO_STREAM_PROPERTY_LEN  sizeof(ASF_AUDIO_STREAM_PROPERTY)
#define ASF_AUDIO_STREAM_PROPERTY_LEN  104	//78+18+8
#define ASF_AUDIO_STREAM_NUM   2

static ASF_AUDIO_STREAM_PROPERTY1 asfAudioStreamProperty1 = {
  { 0x91, 0x07, 0xDC, 0xB7, 0xB7, 0xA9, 0xCF, 0x11, 0x8E,0xE6, 0x00, 0xC0, 0x0C, 0x20, 0x53, 0x65 },
  ASF_AUDIO_STREAM_PROPERTY_LEN,0,
  { 0x40, 0x9E, 0x69, 0xF8, 0x4D, 0x5B, 0xCF, 0x11, 0xA8, 0xFD, 0x00, 0x80, 0x5F, 0x5C, 0x44, 0x2B },
  { 0x50, 0xCD, 0xC3, 0xBF, 0x8F, 0x61, 0xCF, 0x11, 0x8B, 0xB2, 0x00, 0xAA, 0x00, 0xB4, 0xE2, 0x20 },
  0,
  ASF_AUDIO_MEDIA_LEN,
  ASF_SPREAD_AUDIO_LEN,
  ASF_AUDIO_STREAM_NUM,
  0,
  0,
  0,
};
static ASF_AUDIO_STREAM_PROPERTY2 asfAudioStreamProperty2 = {
  //----- Type specific data ------------------------
   0x0045, 0x0001, 0x00001F40, 
   0x000007D0,0x0001, 0x0002, 0x0000 ,	//G726_16000
   //0x00000FA0,0x0001, 0x0004, 0x0000 ,	//G726_32000
  //----- Error correction data ---------------------
   0x01
};
static ASF_AUDIO_STREAM_PROPERTY3 asfAudioStreamProperty3 = {
 0x07D0, 0x07D0, 0x0001, {0x00} 
};
/*----------------------------------------------------------------*/
typedef struct S_ASF_HEADER_EXTENSION {
  BYTE  objectID[16];                 //   ASF_Header_Extension_Object
  //QWORD objectLen;                    //   This value should be 46.
  DWORD objectLen;                    //   This value should be 46.
  DWORD dummy;
  BYTE  reserved1[16];                //   ASF_Reserved_1
  WORD  reserved2;                    //   Reserved and should be set to 6.
  DWORD dataLen;                      //   0 (No additional data)
//BYTE  data;                         //   No additional data.
} ASF_HEADER_EXTENSION  __attribute__((packed));
#define ASF_HEADER_EXTENSION_LEN  46

static ASF_HEADER_EXTENSION asfHeaderExtension = {
  { 0xB5, 0x03, 0xBF, 0x5F, 0x2E, 0xA9, 0xCF, 0x11, 0x8E, 0xE3, 0x00, 0xC0, 0x0C, 0x20, 0x53, 0x65 },
  ASF_HEADER_EXTENSION_LEN,0,
  { 0x11, 0xD2, 0xD3, 0xAB, 0xBA, 0xA9, 0xCF, 0x11, 0x8E, 0xE6, 0x00, 0xC0, 0x0C, 0x20, 0x53, 0x65 },
  6,
  0
};

/*----------------------------------------------------------------*/
typedef struct ST_HD{
   unsigned short tp;	//0x4424
   unsigned short len;	//2008
   unsigned long  seq;  //start from 1
   unsigned short resv;
   unsigned short len2;	//2008
   
}ST_HD __attribute__((packed));
static ST_HD StreamHd={0x4824,377,0,0x0c00,377};
typedef struct ASF_HEADER {
  BYTE  objectID[16];                 //   ASF_Header_Object
  //QWORD objectLen;                    //
  DWORD objectLen;                    //
  DWORD dummy;
  DWORD objectNum;                    //   Number of header objects.
  BYTE  reserved1;                    //   Set the value to 1.
  BYTE  reserved2;                    //   Set the value to 2.
} ASF_HEADER  __attribute__((packed));

#define ASF_HEADER_LEN  30

#define ASF_ALL_HEADER_LEN_A  (ASF_HEADER_LEN		\
                             +ASF_FILE_PROPERTY_LEN	\
                             +ASF_AUDIO_STREAM_PROPERTY_LEN	\
                             +ASF_HEADER_EXTENSION_LEN)	//431-147=284
#define ASF_ALL_HEADER_LEN_AV  (ASF_HEADER_LEN		\
                             +ASF_FILE_PROPERTY_LEN	\
                             +ASF_VIDEO_STREAM_PROPERTY_LEN	\
                             +ASF_AUDIO_STREAM_PROPERTY_LEN	\
                             +ASF_HEADER_EXTENSION_LEN)	//327+104=431

#define ASF_ALL_HEADER_LEN_V  (ASF_HEADER_LEN		\
                             +ASF_FILE_PROPERTY_LEN	\
                             +ASF_VIDEO_STREAM_PROPERTY_LEN	\
                             +ASF_HEADER_EXTENSION_LEN)	//327

static ASF_HEADER asfHeader = {
  { 0x30, 0x26, 0xB2, 0x75, 0x8E, 0x66, 0xCF, 0x11, 0xA6, 0xD9, 0x00, 0xAA, 0x00, 0x62, 0xCE, 0x6C },
  ASF_ALL_HEADER_LEN_V,0,
  3,  //STREAM_OBJ_NO, =4 if AUDIO+VIDEO
  1,
  2
};
/**************************************************************************/
/*  ASF data object                                                       */
/*    +-------------------------------+                                   */
/*    |         Data Object           |                                   */
/*    +-------------------------------+ <-- Data packet starts from here. */
/*    |   Error Correction Data     | <-- Optional                        */
/*    +-----------------------------+                                     */
/*    |                             |                                     */
/*    | Payload parsing information |                                     */
/*    |                             |                                     */
/*    +------------+----------------+                                     */
/*    |            |   Payload 1    |                                     */
/*    |  Payload   +----------------+                                     */
/*    |  data      |   Payload 2    | <-- ?Payload data should not across */
/*    |            +----------------+     Lan packet.                     */
/*    |            |   ...          |                                     */
/*    |            +----------------+                                     */
/*    |            |   Payload n    |                                     */
/*    +------------+----------------+                                     */
/*    |      Padding Data (0xFB)    | <-- Optional                        */
/*    +-----------------------------+                                     */
/**************************************************************************/

/*----------------------------------------------------------------*/
typedef struct ASF_DATA {
  BYTE  objectID[16];                 //   ASF_DATA_Object
  //QWORD objectLen;                    //   Set the value to -1 if Broadcase flag is set to 1.
  DWORD objectLen;                    //   Set the value to ffff if Broadcase flag is set to 1.
  DWORD dummy;
  BYTE  fileID[16];                   // @ Same ad Field ID in File Property
  QWORD totalDataPackets;             //   Invalid if Broadcast flag is set to 1.
  WORD  reserved;                     //   Set the value to 0x0101
                                      // @ Data packets start from here.
} ASF_DATA  __attribute__((packed));
//#define ASF_DATA_LEN  sizeof(ASF_DATA)
#define ASF_DATA_LEN  50

static ASF_DATA asfData = {
  { 0x36, 0x26, 0xB2, 0x75, 0x8E, 0x66, 0xCF, 0x11, 0xA6, 0xD9, 0x00, 0xAA, 0x00, 0x62, 0xCE, 0x6C },
#if 1
  0xffffffff,0xfeffffff,
  //0xfeffffff,0,
#else
  0x32,0,	//Casero asf to change this 
#endif
  { 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  0,
  0x0101
};

/*----------------------------------------------------------------*/
typedef struct ASF_ERROR_CORRECTION {
  BYTE  dataLength          :4;       //    0002= 2-byte
  BYTE  opaque              :1;       //    0= No opaque data
  BYTE  lengthType          :2;       //    00= Size is defined in "dataLength" field
  BYTE  present             :1;       //    1= Error correction present

  BYTE  type                :4;       //    0000= Data is uncorrected. (No Error Correction Object in the Header object)
  BYTE  number              :4;       //    0000= Set the value to 0 if data is uncorrected (type= 0000).
  BYTE  Cycle;                        //    0= Set the value to 0 if data is uncorrected (type= 0000).
} ASF_ERROR_CORRECTION  __attribute__((packed));
//#define ASF_ERROR_CORRECTION_LEN  sizeof(ASF_ERROR_CORRECTION)
#define ASF_ERROR_CORRECTION_LEN  3

ASF_ERROR_CORRECTION asfErrorCorrection = {
  2,
  0,
  0,
  1,

  0,
  0,
  0
};

/*----------------------------------------------------------------*/
typedef struct ASF_PAYLOAD_INFORMATION {
  BYTE  multiplePayload     :1;       //!@ 1=
  BYTE  sequenceType        :2;       //    00= No sequence
  BYTE  paddingLengthType   :2;       //    10= Padding length type is WORD
  BYTE  packetLengthType    :2;       //    00= No packet length
  BYTE  errorCorrection     :1;       //    0= No error correction

  BYTE  replicatedType      :2;       //01=byte
  BYTE  offsetType          :2;       //11=dword
  BYTE  objectNumberType    :2;       //01=byte
  BYTE  streamNumberType    :2;       //   01= Stream number is coded using a BYTE

//      packetLength;                 //   Length of data packet
//      sequence;                     //   Reserved for future use
  WORD  paddingLength;                // @ Length of the padding at the end of a data packet
  DWORD sendTime;                     // @ Specified in milliseconds
  WORD  duration;                     // @ Specified in milliseconds
} ASF_PAYLOAD_INFORMATION  __attribute__((packed));
//#define ASF_PAYLOAD_INFORMATION_LEN  sizeof(ASF_PAYLOAD_INFORMATION)
#define ASF_PAYLOAD_INFORMATION_LEN  10
ASF_PAYLOAD_INFORMATION asfPayloadInfo = {
  1, //multiple payload,
  0,
  2,
  0,
  0,

  1,
  3,
  1,
  1,

  0,//?,
  0,//?,
  3,//duration 
};
#include "v_data.h"
#endif
