/*
 * linux/arch/armnommu/kernel/head-arm-dm270.S
 *
 *  Copyright (C) 2003 Cadenux.com <http://www.cadenux.com>
 *  Gregory Nutt <greg.nutt@cadenux.com>
 *
 * Derived from linux/arch/armnommu/kernel/head-arm-dsc25.S
 * Derived from head-arm-atmel.S, head-arm-dsc21.S, and Boot.asm
 *   copyright:
 *         (C) 2001 RidgeRun, Inc. (http://www.ridgerun.com)
 *   author: Steve Johnson <stevej@ridgerun.com>
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/linkage.h>
#include <linux/autoconf.h>
#include <asm/assembler.h>
#include <asm/hardware.h>
#include <asm/mach-types.h>
#include <asm/arch/memconfig.h>
#include <asm/arch/cache.h>

/* This is the define to put us into service mode. */

#define I_BIT        0x80
#define F_BIT        0x40
#define ARM_MODE_SVC 0x13

/* Pretend we know what our processor type code is */

#define ARM_PROCESSOR_ID 0x41007700

/* Size of the initial stack.   Thit init_task_union is the union of
 * a struct task_struct that is pre-initialized and a size arrary of
 * size 2048*sizeof(long) == 8192 (see include/linux/sched.h).
 * We want to write initial values to the stack for crash analysis,
 * but we don't want to clobber the initialized task_struct,
 * so we define: */

#define INIT_TASK_OFFSET  8192
#define INIT_TASK_RESERVE 1024
#define INITIAL_STACK_SIZE (INIT_TASK_OFFSET-INIT_TASK_RESERVE)

/* Register usage: */

#define ARG1        r0

#define TMP1        r0
#define TMP2        r1
#define TMP3        r2
#define TMP4        r3

#define SBSS        r4
#define EBSS        r5

#define COMPAT      r4
#define PROCID      r5
#define MACHID      r6
#define CFGALN      r7

#define DATA_SROM   TMP1
#define DATA_SRAM   TMP2
#define DATA_ERAM   TMP3
#define DATA_TMP    TMP4
#define DATA_FIRST  r3
#define DATA_LAST   r10

#define IRAM_SRC    TMP1
#define IRAM_DEST   TMP2
#define IRAM_END    TMP3
#define IRAM_SIZE   TMP4
#define IRAM_TMP    r4
#define IRAM_FIRST  r3
#define IRAM_LAST   r10

/* These are checkpoint values */

#define INITIAL_ENTRY     #'A'
#define ENTRY_VERIFY      #'B'
#define IRAM_INIT_OK      #'C'
#define BSS_CLEARED       #'D'
#define DATA_INITIALIZED  #'E'
#define STACK_INITIALIZED #'F'
#define CALL_START_KERNEL #'G'
#define END_CHECKPOINT    #'\n'

/* These macros are used to output debug status on the serial console.
 * These macros can be used only after a C stack has been initialized.
 *
 * These macros (incl. printch/hex8) use R0-R3 and lr.  As valid
 * stack pointer is not necessary to use them.
 *
 * We can use the UART this early in the boot sequence because
 * rrload has left it fully initialized for us.
 */

#ifdef CONFIG_DEBUG_LL
	.globl	printch
	.type	printch, #function

	.macro	checkpoint, ch
	mov	ARG1, \ch
	bl	printch
	.endm

	.globl	printhex8
	.type	printhex8, #function

	.macro	showvalue, addr, tmp
	ldr	\tmp, =addr
	mov	ARG1, \tmp
	bl	printhex8
	.endm
#else
	.macro	checkpoint, ch
	.endm
	.macro	showvalue, addr, tmp
	.endm
#endif /* CONFIG_DEBUG_LL */

/* We assume the bootloader has already initialized most of the h/w for
 * us and that only leaves us having to do some kernel specific things
 * below.
 */
        .global start
	.type	start, #function
        .global stext
	.type	stext, #function
        .global _start
	.type	_start, #function
        .global _stext
	.type	_stext, #function
        .global _entry
	.type	_entry, #function
        .global __stext
	.type	__stext, #function
        .global __entry
	.type	__entry, #function
start:
stext:
_start:
_stext:
_entry:
__stext:
__entry:

/* First, setup initial processor mode */

	mov	TMP1, #(ARM_MODE_SVC | I_BIT | F_BIT )
	msr	cpsr, TMP1

	checkpoint INITIAL_ENTRY /* Indicates that we have started */
	checkpoint ENTRY_VERIFY  /* Proves that printch works */

/* Do we need to copy the default .iram section to IRAM? */

#ifdef BSPCONF_USE_IRAM

	/* Yes, load addresses and counts */

	adr     IRAM_TMP, LC3
	ldmia   IRAM_TMP, {IRAM_SRC, IRAM_DEST, IRAM_END, IRAM_SIZE}

#if defined(BSPCONF_BOOT_FROM_FLASH)
	add	IRAM_SRC, IRAM_SRC, IRAM_SIZE
#endif

1:	ldmia	IRAM_SRC!, {IRAM_FIRST - IRAM_LAST}
	stmia	IRAM_DEST!, {IRAM_FIRST - IRAM_LAST}
	cmp	IRAM_DEST, IRAM_END
	blt	1b
#endif
	checkpoint IRAM_INIT_OK    /* Survived IRAM initialization */

/* Setup kernel stack (and get the BSS range) */

	adr	TMP1, LC0
	ldmia   TMP1, {SBSS, EBSS, sp}

/* Clear kernel BSS section */

	mov	TMP1, #0
1:	cmp	SBSS, EBSS                                  
	strcc	TMP1, [SBSS], #4
	bcc	1b

	checkpoint BSS_CLEARED   /* Indicates survived BSS clear */

/* Copy kernel's .data sections to new home in RAM. */

#ifdef BSPCONF_BOOT_FROM_FLASH
	
	adr	DATA_TMP, LC2
	ldmia	DATA_TMP, {DATA_SROM, DATA_SRAM, DATA_ERAM}

1:	ldmia	DATA_SROM!, {DATA_FIRST - DATA_LAST}
	stmia	DATA_SRAM!, {DATA_FIRST - DATA_LAST}
	cmp	DATA_SRAM, DATA_ERAM
	blt	1b

#endif
	checkpoint DATA_INITIALIZED   /* Indicates survived DATA init */

/* Get more memory references */
	
	adr	TMP1, LC1
#ifdef CONFIG_ALIGNMENT_TRAP
	ldmia   TMP1, {COMPAT, PROCID, MACHID, CFGALN}
#else
	ldmia   TMP1, {COMPAT, PROCID, MACHID}
#endif

/* Initialize Kernel Stack Contents */

	mov	TMP2, sp
	sub	TMP2, TMP2, #INITIAL_STACK_SIZE
	ldr	TMP1, L_STACK_MAGIC
	str	TMP1, [TMP2], #4
	ldr	TMP1, L_STACK_UNTOUCHED_MAGIC
1:      cmp	TMP2, sp
        strcc	TMP1, [TMP2], #4
        bcc	1b

	checkpoint STACK_INITIALIZED   /* Indicates survived stack init */
	
/* Setup parameters for start_kernel... */
/* Suppress messages about compatibility... */

	mov	TMP1, #0
	str	TMP1, [COMPAT]

/* Pretend we know what our processor code is */

	ldr	TMP1, =ARM_PROCESSOR_ID
	str	TMP1, [PROCID]

	mov	TMP1, #MACH_TYPE_DM270
	str	TMP1, [MACHID]

#ifdef CONFIG_ALIGNMENT_TRAP
# error CONFIG_ALIGNMENT_TRAP not supported
#endif

/* Enable the DM270's instruction cache */

#ifdef CONFIG_CPU_WITH_CACHE
	dm270_clear_cache TMP1, TMP2, TMP3
	dm270_enable_cache TMP1, TMP2
#else
	dm270_disable_cache TMP1, TMP2
#endif

/* Jump to kernel entry */

	checkpoint CALL_START_KERNEL   /* Indicates start_kernel called */
	checkpoint END_CHECKPOINT      /* (for cosmetic reasons) */

	mov	fp, #0
	b	start_kernel

/* Variables */

LC0:	.long	SYMBOL_NAME(__bss_start)
	.long	SYMBOL_NAME(_end)
	.long	SYMBOL_NAME(init_task_union)+INIT_TASK_OFFSET

LC1:	.long	SYMBOL_NAME(compat)
	.long	SYMBOL_NAME(processor_id)
	.long	SYMBOL_NAME(__machine_arch_type)
#ifdef CONFIG_ALIGNMENT_TRAP
	.long	SYMBOL_NAME(cr_alignment)
#endif
 
#ifdef BSPCONF_BOOT_FROM_FLASH
LC2:	.long	__RONLY_end  @ Where .data defaults are stored in Flash.
	.long	__data_start @ Where .data needs to reside in SDRAM.
	.long	__data_end
#endif /* BSPCONF_BOOT_FROM_FLASH */

#ifdef BSPCONF_USE_IRAM

#if defined(BSPCONF_BOOT_FROM_FLASH)
LC3:	.long	__RONLY_end  @ Where .data is stored in ELF file, we will add sizeof later.
#else
LC3:	.long	_edata  @ Start of where .iram defaults are stored in ELF file.
#endif
	.long	__iram_start @ start of where .iram needs to reside in IRAM.
	.long	__iram_end
	.long	__DATASEG_size
#endif

	/*  These values should stay in sync with linux/kernel.h */

L_STACK_UNTOUCHED_MAGIC: .long 0xfeef1ef0
L_STACK_MAGIC:           .long 0xdeadbeef
	.end

