/*
 * linux/arch/armnommu/mach-dm270/irq.c:
 *         dm270 IRQ mask, unmask & ack routines.
 *
 *    Copyright (C) 2003 Cadenux.com <http://www.cadenux.com>
 *    Gregory Nutt <greg.nutt@cadenux.com>
 *
 * Derived from linux/arch/armnommu/mach-dsc25/irq.c:
 * Derived from linux/arch/armnommu/mach-dsc21/irq.c:
 *    copyright:
 *         (C) 2001 RidgeRun, Inc. (http://www.ridgerun.com)
 *    author: Gordon McNutt <gmcnutt@ridgerun.com>
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <linux/types.h>
#include <asm/arch/irq.h>

void dm270_mask_irq(unsigned int irq)
{
  unsigned long regaddr;

  /* Then, mask the interrupt by clearing the
   * corresponding bit in the IRQ enable register.
   */

  if (irq < 16)
    {
      regaddr = DM270_IRQ0_ENABLE;
    }
  else if (irq < 32)
    {
      regaddr = DM270_IRQ1_ENABLE;
      irq    -= 16;
    }
  else
    {
      regaddr = DM270_IRQ2_ENABLE;
      irq    -= 32;
    }
  outw((inw(regaddr) & ~(1 << irq)), regaddr);
}	

void dm270_unmask_irq(unsigned int irq)
{
  unsigned long regaddr;

  /* Then, unmask the interrupt by setting the
   * corresponding bit in the IRQ enable register.
   */

  if (irq < 16)
    {
      regaddr = DM270_IRQ0_ENABLE;
    }
  else if (irq < 32)
    {
      regaddr = DM270_IRQ1_ENABLE;
      irq    -= 16;
    }
  else
    {
      regaddr = DM270_IRQ2_ENABLE;
      irq    -= 32;
    }
  outw((inw(regaddr) | (1 << irq)), regaddr);
}

void dm270_mask_ack_irq(unsigned int irq)
{
  unsigned long regaddr;

  /* Mask the interrupt. */

  dm270_mask_irq(irq);

  /* Then, acknowledge the interrupt by setting
   * the corresponding bit in the IRQ status register.
   */

  if (irq < 16)
    {
      regaddr = DM270_IRQ0_STATUS;
    }
  else if (irq < 32)
    {
      regaddr = DM270_IRQ1_STATUS;
      irq    -= 16;
    }
  else
    {
      regaddr = DM270_IRQ2_STATUS;
      irq    -= 32;
    }
  //outw((inw(regaddr) | (1 << irq)), regaddr);
  outw((1 << irq), regaddr);
}
