/**************************************************************************
 *
 * linux/arch/armnommu/mach-dm270/setup.c
 *
 * BRIEF MODULE DESCRIPTION
 *   This has additional initialiation needed by the DM270.
 *
 * Derived from linux/arch/armnommu/mach-dsc25/setup.c
 * Derived from linux/arch/armnommu/mach-dsc21/setup.c
 *    Copyright (C) 2001 RidgeRun, Inc.
 *    Author: RidgeRun, Inc.
 *          stevej@ridgerun.com
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED O
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **************************************************************************/

/**************************************************************************
 * Compilation Switches
 **************************************************************************/

#define SETUP_CONSOLE_WRITE 0
#define SETUP_EXTERNAL_BUS  1
#define SETUP_GIO           1
#define SETUP_UART          0

/**************************************************************************
 * Included Files
 **************************************************************************/
#include <linux/kernel.h>
#include <linux/config.h>
#include <linux/console.h>
#include <linux/gio.h>
#include <asm/io.h>
#include <asm/arch/hardware.h>
#include <asm/arch/memconfig.h>

#if defined(SETUP_EXTERNAL_BUS) && (SETUP_EXTERNAL_BUS != 0)
#include <asm/arch/extmem.h>
#endif

#include <linux/delay.h>

/**************************************************************************
 * Private Functions
 **************************************************************************/
#if SETUP_CONSOLE_WRITE

extern void register_console(struct console *);
static const char welcome[] = "\nWelcome to Cadenux DM270 uClinux-2.4\n";

void dm270_console_write(struct console *con, const char *buf, unsigned len)
{
  int i;

  for (i = 0; i < len; i++)
    {
      if (buf[i] == '\n')
	{
	  while(!(inw(DM270_UART0_SR) & 0x0400));
	  outb('\r', DM270_UART0_DTRR);
	}
      while(!(inw(DM270_UART0_SR) & 0x0400));
      outb(buf[i], DM270_UART0_DTRR);		
    }
}

static struct console dm270_console =
{
  name: "dm270con",
  write: dm270_console_write
};

#else
# define welcome       NULL
# define dm270_console NULL
# define register_console(con)
# define dm270_console_write(con,buf,len)
#endif

#if SETUP_EXTERNAL_BUS
static void dm270_external_bus_init(void)
{
  /* Setup Flash bus control and timing (connected to EM0) */

  // outw(DM270_EXBC_CS0CTRL1_DATA, DM270_EXBC_CS0CTRL1);
  // outw(DM270_EXBC_CS0CTRL2_DATA, DM270_EXBC_CS0CTRL2);

  /* Set the initial value of every EXBC register (see extmem.h) */

  outw(DM270_EXBC_CS1CTRL1A_DATA, DM270_EXBC_CS1CTRL1A);
  outw(DM270_EXBC_CS1CTRL1B_DATA, DM270_EXBC_CS1CTRL1B);
  outw(DM270_EXBC_CS1CTRL2_DATA,  DM270_EXBC_CS1CTRL2);
  outw(DM270_EXBC_CS2CTRL1_DATA,  DM270_EXBC_CS2CTRL1);
  outw(DM270_EXBC_CS3CTRL1_DATA,  DM270_EXBC_CS3CTRL1);
  outw(DM270_EXBC_CS2CTRL2_DATA,  DM270_EXBC_CS2CTRL2);
  outw(DM270_EXBC_CS3CTRL2_DATA,  DM270_EXBC_CS3CTRL2);
  outw(DM270_EXBC_CS4CTRL1_DATA,  DM270_EXBC_CS4CTRL1);
  outw(DM270_EXBC_CS4CTRL2_DATA,  DM270_EXBC_CS4CTRL2);
  outw(DM270_EXBC_BUSCTRL_DATA,   DM270_EXBC_BUSCTRL);
  outw(DM270_EXBC_BUSRLS_DATA,    DM270_EXBC_BUSRLS);
  outw(DM270_EXBC_DPSTR0_DATA,    DM270_EXBC_DPSTR0);
  outw(DM270_EXBC_DPSTR1_DATA,    DM270_EXBC_DPSTR1);
  outw(DM270_EXBC_DPSTR2_DATA,    DM270_EXBC_DPSTR2);
  outw(DM270_EXBC_DPSTR3_DATA,    DM270_EXBC_DPSTR3);
  outw(DM270_EXBC_DPSTR4_DATA,    DM270_EXBC_DPSTR4);
  outw(DM270_EXBC_DPSTR5_DATA,    DM270_EXBC_DPSTR5);
  outw(DM270_EXBC_TEST_DATA,      DM270_EXBC_TEST);
}
#else
# define dm270_external_bus_init()
#endif

#if SETUP_GIO
static void dm270_gio_init(void)
{
  unsigned short irqport = 0x0000; /* GIO0 -GIO15 */
  unsigned short irqedge = 0x0000; /* GIO0 -GIO15 */
  unsigned short dir0    = 0xffff; /* GIO0 -GIO15 */
  unsigned short dir1    = 0xffff; /* GIO16-GIO31 */
  unsigned short dir2    = 0x0003; /* GIO32-GIO33 */
  unsigned short inv0    = 0x0000; /* GIO0 -GIO15 */
  unsigned short inv1    = 0x0000; /* GIO16-GIO31 */
  unsigned short inv2    = 0x0000; /* GIO32-GIO33 */
  unsigned short chat0   = 0x0000; /* GIO0 -GIO15 */
  unsigned short chat1   = 0x0000; /* GIO16-GIO31 */
  unsigned short chat2   = 0x0000; /* GIO32-GIO33 */
  unsigned short nchat   = 0x0008;
#if 0	       /* Change fsel0 default setting to specfic function. */
  unsigned short fsel0   = 0x0000; /* GIO8 -GIO23 */
  unsigned short fsel1   = 0x0000; /* GIO24-GIO33 */
#else
  unsigned short fsel0   = 0x8000; /* GIO23: RXD1 */
  unsigned short fsel1   = 0x0001; /* GIO24: TXD1 */
#endif

				/* Change dir0 default setting to output. */
  dir0                  &= ~(GIO_8_BIT);   /* Output - CCD_SCL */
  dir0                  &= ~(GIO_10_BIT);  /* Output - CCD_RST */
  dir0                  &= ~(GIO_12_BIT);  /* Output - LCD_CS# */
  dir0                  &= ~(GIO_13_BIT);  /* Output - LCD_WR# */
  dir0                  &= ~(GIO_14_BIT);  /* Output - LCD_DATA*/
  dir0                  &= ~(GIO_15_BIT);  /* Output - LCD_BL  */
				/* Change dir1 default setting to output. */
  dir1                  &= ~(GIO_16_BIT);  /* Output - RTC_SCL */
  dir1                  &= ~(GIO_19_BIT);  /* Output - AUD_SDIN */
  dir1                  &= ~(GIO_20_BIT);  /* Output - AUD_CS# */
  dir1                  &= ~(GIO_21_BIT);  /* Output - AUD_SCLK */
  dir1                  &= ~(GIO_24_BIT);  /* Output - CPU_TXD */
  dir1                  &= ~(GIO_25_BIT);  /* Output - CPU_RST# */
  dir1                  &= ~(GIO_26_BIT);  /* Output - PCMCIA_RST# */
  dir1                  &= ~(GIO_27_BIT);  /* Output - PHY_RST# */
  dir1                  &= ~(GIO_28_BIT);  /* Output - LED_PWRG */
  dir1                  &= ~(GIO_29_BIT);  /* Output - LED_PWRO */
  dir1                  &= ~(GIO_30_BIT);  /* Output - LED_WGG */

  /* Initialize bit 2 of GIO IF for PushButton use.(Low Active) */
  irqport               |= GIO_2_BIT; /* GIO 2 is interrupt */

  /* Initialize bit 4 of GIO IF for NIC use.(High Active) */
  irqport               |= GIO_4_BIT; /* GIO 4 is interrupt */
  inv0                  |= GIO_4_BIT; /* Inverted input */

  /* Initialize bit 5 of GIO IF for Wireless use.(Low Active) */
  irqport               |= GIO_5_BIT; /* GIO 5 is interrupt */

  /* Initialize bit 6 of GIO IF for SPDIF use.(High Active) */
  irqport               |= GIO_6_BIT; /* GIO 6 is interrupt */
  inv0                  |= GIO_6_BIT; /* Inverted input */

  /* Initialize bit 7 of GIO IF for CIR_ACK use.(High Active) */
  irqport               |= GIO_7_BIT; /* GIO 7 is interrupt */
  inv0                  |= GIO_7_BIT; /* Inverted input */


  outw(0,       DM270_GIO_IRQPORT);
  outw(irqport, DM270_GIO_IRQPORT);
  outw(irqedge, DM270_GIO_IRQEDGE);

  outw(dir0,    DM270_GIO_DIR0);
  outw(dir1,    DM270_GIO_DIR1);
  outw(dir2,    DM270_GIO_DIR2);

  outw(inv0,    DM270_GIO_INV0);
  outw(inv1,    DM270_GIO_INV1);
  outw(inv2,    DM270_GIO_INV2);
  outw(chat0,   DM270_GIO_CHAT0);
  outw(chat1,   DM270_GIO_CHAT1);
  outw(chat2,   DM270_GIO_CHAT2);
  outw(nchat,   DM270_GIO_NCHAT);
  outw(fsel0,   DM270_GIO_FSEL0);
  outw(fsel1,   DM270_GIO_FSEL1);

}
#else
# define dm270_gio_init()
#endif

#if SETUP_UART
static void dm270_uart_init(void)
{
  /* Disable ints */

  outw(0, DM270_UART0_MSR);
  outw(0, DM270_UART1_MSR);

  /* Clear fifos */

  outw(0x8000, DM270_UART0_RFCR);
  outw(0x8000, DM270_UART0_TFCR);
  outw(0x8000, DM270_UART1_RFCR);
  outw(0x8000, DM270_UART1_TFCR);

  /* No breaks */

  outw(0, DM270_UART0_LCR);
  outw(0, DM270_UART1_LCR);

  /* Now, setup UART0 for use as a printk console while we boot. */

  /* 115200 baud */

  outw(0x0013, DM270_UART0_BRSR);

  /* 8N1 & enable rcv interrupt */

  outw(0x8000, DM270_UART0_MSR);

  /* 1 byte rcv fifo trigger */

  outw(0, DM270_UART0_RFCR);

  /* 16 byte xmt fifo trigger */

  outw(0x0300, DM270_UART0_TFCR);
}
#else
# define dm270_uart_init()
#endif


/**************************************************************************
 * Public Functions
 **************************************************************************/

void setup_dm270(void)
{
  dm270_external_bus_init();
  dm270_gio_init();
  dm270_uart_init();
  dm270_console_write(&dm270_console, welcome, sizeof(welcome));
  register_console(&dm270_console);
}


/**************************************************************************
 * Public Functions
 **************************************************************************/
#ifdef CONFIG_ARCH_DM270

/* Sam Chang  2003/08/11						    */
/* Refer from linux/arch/armnommu/kernel/setup-dsc21.c GIO by Vincent	    */ 
/*--------------------------------------------------------------------------*/
/* GIO Register Overlay.                                                    */
/* On the DSC21, there are 32 GIO pins.  These pins can be used to support  */
/* other peripherals, not currently supported by the DSC21. It is important */
/* to provide an interface to adjust the values of this register, as        */
/* interrupt direction and levels can change dynamically.                   */
/*                                                                          */
/* We will define an overlay and expose functions to control data values in */
/* the GIO register.                                                        */
/* Each register element is 16 bits wide.  The semantics come directly from */
/* the DSC21 Register document.                                             */
/*--------------------------------------------------------------------------*/
typedef struct GIORegister {
   //
   // Input/Output direction of each port for GIO 15 - 0
   // 0 - output 1 - input
   //
   unsigned short dir0;

   //
   // Input/Output direction of each port for GIO 31 - 16
   // 0 - output 1 - input
   //
   unsigned short dir1;

   //
   // Input/Output direction of each port for GIO 32 - 33
   // 0 - output 1 - input
   //
   unsigned short dir2;
   
   //
   // Port inverse.  This is used to inverse the polarity of the signal for
   // a specific GIO port. GIO 15 - 0
   // 0 - not inverted, 1 - inverted
   //
   unsigned short inv0;

   //
   // Port inverse.  This is used to inverse the polarity of the signal for
   // a specific GIO port. GIO 31 - 16
   //
   unsigned short inv1;

   //
   // Port inverse.  This is used to inverse the polarity of the signal for
   // a specific GIO port. GIO 32 - 33 
   //
   unsigned short inv2;

   //
   // Port Bit Register.  When 1 is written into a bit that is set to an output
   // port, the output of the cooresponding output port is set to "H".  When 
   // it is read, the current port status is read.
   // GIO 15 - 0
   //
   unsigned short bitset0;

   //
   // Port Bit Register.  When 1 is written into a bit that is set to an output
   // port, the output of the cooresponding output port is set to "H".  When 
   // it is read, the current port status is read.
   // GIO 31 - 16
   //
   unsigned short bitset1;

   //
   // Port Bit Register.  When 1 is written into a bit that is set to an output
   // port, the output of the cooresponding output port is set to "H".  When 
   // it is read, the current port status is read.
   // GIO 32 - 33 
   //
   unsigned short bitset2;

   //
   // Port Clear Register.  When 1 is written into a bit that is set to an 
   // output port, the cooresponding output port is cleared to "L".  When it 
   // is read, the current port status is read.
   // GIO 15 - 0
   //
   unsigned short bitclr0;

   //
   // Port Clear Register.  When 1 is written into a bit that is set to an 
   // output port, the cooresponding output port is cleared to "L".  When it 
   // is read, the current port status is read.
   // GIO 31 - 16
   //
   unsigned short bitclr1;

   //
   // Port Clear Register.  When 1 is written into a bit that is set to an 
   // output port, the cooresponding output port is cleared to "L".  When it 
   // is read, the current port status is read.
   // GIO 32 - 33
   //
   unsigned short bitclr2;

   //
   // Interrupt Register.  This register indicates if ports GIO 0 - 7 are used
   // as external interrupt ports. Writting a 1 to any of these ports means 
   // the cooresponding port can be used as an external interrupt pin, 
   // provided that the port has direction input.
   //
   unsigned short irqport;

   //
   // Interrupt Edge Register.
   // 0 : Single-sided
   //		GIOxxINV = 0 : falling edge
   //		GIOxxINV = 1 : rising edge
   // 1 : Dual-sided
   //
   unsigned short irqedge;
   
   //
   // Chatter Setting Register.  When enabled, input data is set only after
   // input signal is stable for an interval longer than set in NCHAT 
   // 0 : enable
   // 1 : disable
   // GIO 15 - 0
   //
   unsigned short chat0;

   //
   // Chatter Setting Register.  When enabled, input data is set only after
   // input signal is stable for an interval longer than set in NCHAT 
   // 0 : enable
   // 1 : disable
   // GIO 16 - 31
   //
   unsigned short chat1;

   //
   // Chatter Setting Register.  When enabled, input data is set only after
   // input signal is stable for an interval longer than set in NCHAT 
   // 0 : enable
   // 1 : disable
   // GIO 32 - 33
   //
   unsigned short chat2;

   //
   // Chatter Value Register. The input GIO data is imported when stable at
   // a logic level longer that width of ( nchat+1 ) x ARM Clocks.  
   //
   unsigned short nchat;

   //
   // Function Select Register. When 1 is written into a bit that is set to 
   // other function except norman GIO function 
   // GIO 8 - 23
   //
   unsigned short fsel0;

   //
   // Function Select Register. When 1 is written into a bit that is set to 
   // other function except norman GIO function 
   // GIO 24 - 33
   //
   unsigned short fsel1;

} GIORegister_t;

/*--------------------------------------------------------------------------*/
/* Overlay data struct to GIO memory.                                       */
/* This will define an accessor pointer to the actual GIO register          */
/*--------------------------------------------------------------------------*/
static GIORegister_t *pGIORegs = (GIORegister_t *)GIO_REGISTER_BASE;

/*--------------------------------------------------------------------------*/
/* GIO   Overlay.                                                           */
/* These functions represent each actions on the GIO registers.             */
/* As documented by the DM270 Register Manuals.                             */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* GIOGetDir                                                                */
/* This function is used to get the value of either the upper or lower 16   */
/* port direction.                                                          */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetDir(GIORegSelect word)
{
	unsigned short returnVal = -1;

	if ( word == bit15_0 ) {
		returnVal = pGIORegs->dir0;
	} else if ( word == bit31_16 ) {
		returnVal = pGIORegs->dir1;
	} else if ( word == bit33_32 ) {
		returnVal = pGIORegs->dir2;
	} else {
		printk("GIO: GIOGetDir - word selector invalid => %d\n", word);
	}
	return returnVal;
}  // End of GIOGetDir

/*--------------------------------------------------------------------------*/
/* GIOSetDir                                                                */
/* This function is used to set the value of either the upper or lower 16   */
/* port direction.                                                          */
/*--------------------------------------------------------------------------*/
void GIOSetDir(GIORegSelect word, unsigned short value)
{
	if ( word == bit15_0 ) {
		pGIORegs->dir0 = value;
	} else if ( word == bit31_16 ) {
		pGIORegs->dir1 = value;
	} else if ( word == bit33_32 ) {
		pGIORegs->dir2 = value;
	} 
}  // End of GIOSetDir

/*--------------------------------------------------------------------------*/
/* GIOGetInv                                                                */
/* This function is used to get the value of the invert port bits, for      */
/* either the upper or lower registers.                                     */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetInv(GIORegSelect word)
{
	unsigned short returnVal = -1;

	if ( word == bit15_0 ) {
		returnVal = pGIORegs->inv0;
	} else if ( word == bit31_16 ) {
		returnVal = pGIORegs->inv1;
	} else if ( word == bit33_32 ) {
		returnVal = pGIORegs->inv2;
	} else {
		printk("GIO: GIOGetInv - word selector invalid => %d\n", word);
	}
	return returnVal;
}  // End of GIOGetInv

/*--------------------------------------------------------------------------*/
/* GIOSetInv                                                                */
/* This function is used to set the value of the invert port bits, for      */
/* either the upper or lower.                                               */
/*--------------------------------------------------------------------------*/
void GIOSetInv(GIORegSelect word, unsigned short value)
{
	if ( word == bit15_0 ) {
		pGIORegs->inv0 = value;
	} else if ( word == bit31_16 ) {
		pGIORegs->inv1 = value;
	} else if ( word == bit33_32 ) {
		pGIORegs->inv2 = value;
	}
}  // End of GIOSetInv

/*--------------------------------------------------------------------------*/
/* GIOGetPortBitSet                                                         */
/* This function is used to get the value of the port bit register set,     */
/* either upper or lower.                                                   */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetPortBitSet(GIORegSelect word)
{
	unsigned short returnVal = -1;

	if ( word == bit15_0 ) {
		returnVal = pGIORegs->bitset0;
	} else if ( word == bit31_16 ) {
		returnVal = pGIORegs->bitset1;
	} else if ( word == bit33_32 ) {
		returnVal = pGIORegs->bitset2;
	} else {
		printk("GIO: GIOGetPortBitSet - word selector invalid => %d\n", word);
	}
	return returnVal;
}  // End of GIOGetPortBitSet

/*--------------------------------------------------------------------------*/
/* GIOSetPortBitSet                                                         */
/* This function is used to set the value of the port bit register set,     */
/* either upper or lower.                                                   */
/*--------------------------------------------------------------------------*/
void GIOSetPortBitSet(GIORegSelect word, unsigned short value)
{
	if ( word == bit15_0 ) {
		pGIORegs->bitset0 = value;
	} else if ( word == bit31_16 ) {
		pGIORegs->bitset1 = value;
	} else if ( word == bit33_32 ) {
		pGIORegs->bitset2 = value;
	}
}  // End of GIOSetPortBitSet

/*--------------------------------------------------------------------------*/
/* GIOGetPortBitClear                                                       */
/* This function is used to get the value of the port bit clear register    */
/* set, either the upper or lower.                                          */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetPortBitClear(GIORegSelect word)
{
	unsigned short returnVal = -1;

	if ( word == bit15_0 ) {
		returnVal = pGIORegs->bitclr0;
	} else if ( word == bit31_16 ) {
		returnVal = pGIORegs->bitclr1;
	} else if ( word == bit33_32 ) {
		returnVal = pGIORegs->bitclr2;
	} else {
		printk("GIO: GIOGetPortBitClear - word selector invalid => %d\n", word);
	}
	return returnVal;
}  // End of GIOGetPortBitClear

/*--------------------------------------------------------------------------*/
/* GIOSetPortBitClear                                                       */
/* This function is used to set the value of the port bit clear register    */
/* set.                                                                     */
/*--------------------------------------------------------------------------*/
void GIOSetPortBitClear(GIORegSelect word, unsigned short value)
{
	if ( word == bit15_0 ) {
		pGIORegs->bitclr0 = value;
	} else if ( word == bit31_16 ) {
		pGIORegs->bitclr1 = value;
	} else if ( word == bit33_32 ) {
		pGIORegs->bitclr2 = value;
	}
}  // GIOSetPortBitClear

/*--------------------------------------------------------------------------*/
/* GIOGetPortExternalIRQ                                                    */
/* This function returns the port IRQ register.                             */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetPortExternalIRQ(void)
{
	return pGIORegs->irqport;
}  // End of GIOGetPortIRQ

/*--------------------------------------------------------------------------*/
/* GIOSetPortExternalIRQ                                                    */
/* This function is used to set the GIO port IRQ.                           */
/*--------------------------------------------------------------------------*/
void GIOSetPortExternalIRQ(unsigned short irqval)
{
	pGIORegs->irqport = irqval;
}  // End of GIOSetPortExternalIRQ

/*--------------------------------------------------------------------------*/
/* GIOGetEdgeIRQ                                                    	    */
/* This function returns GIO IRQ Edge Setting register.                     */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetEdgeIRQ(void)
{
	return pGIORegs->irqedge;
}

/*--------------------------------------------------------------------------*/
/* GIOSetEdgeIRQ                                                            */
/* This function is used to set the GIO 0-15 Interrupt Edge                 */
/*--------------------------------------------------------------------------*/
void GIOSetEdgeIRQ(unsigned short edgeval)
{
	pGIORegs->irqedge = edgeval;
}

/*--------------------------------------------------------------------------*/
/* GIOGetChatter                                                    	    */
/* This function returns GIO Chatter Setting register.                      */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetChatter(GIORegSelect word)
{
	unsigned short returnVal = -1;

	if ( word == bit15_0 ) {
		returnVal = pGIORegs->chat0;
	} else if ( word == bit31_16 ) {
		returnVal = pGIORegs->chat1;
	} else if ( word == bit33_32 ) {
		returnVal = pGIORegs->chat2;
	} else {
		printk("GIO: GIOGetChatter - word selector invalid => %d\n", word);
	}
	return returnVal;
}  // End of GIOGetChatter

/*--------------------------------------------------------------------------*/
/* GIOSetChatter                                                    	    */
/* This function is used to set GIO Chatter Setting register. 		    */
/*--------------------------------------------------------------------------*/
void GIOSetChatter(GIORegSelect word, unsigned short value)
{
	if ( word == bit15_0 ) {
		pGIORegs->chat0 = value;
	} else if ( word == bit31_16 ) {
		pGIORegs->chat1 = value;
	} else if ( word == bit33_32 ) {
		pGIORegs->chat2 = value;
	}
}  // GIOSetChatter

/*--------------------------------------------------------------------------*/
/* GIOGetNCHAT                                                    	    */
/* This function returns GIO Chatter Value register.                        */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetNCHAT(void)
{
	return pGIORegs->nchat;
}

/*--------------------------------------------------------------------------*/
/* GIOSetNCHAT                                                    	    */
/* This function is used to set GIO Chatter Value register. 		    */
/*--------------------------------------------------------------------------*/
void GIOSetNCHAT(unsigned short chatval)
{
	pGIORegs->nchat = chatval;
}

/*--------------------------------------------------------------------------*/
/* GIOGetFunctionSelect0                                               	    */
/* This function returns GIO8 - GIO23 Function Select register.             */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetFunctionSelect0(void)
{
	return pGIORegs->fsel0;
}

/*--------------------------------------------------------------------------*/
/* GIOSetFunctionSelect0                                              	    */
/* This function is used to set GIO8 - GIO23 Function Select register. 	    */
/*--------------------------------------------------------------------------*/
void GIOSetFunctionSelect0(unsigned short fsel0val)
{
	pGIORegs->fsel0 = fsel0val;
}

/*--------------------------------------------------------------------------*/
/* GIOGetFunctionSelect1                                               	    */
/* This function returns GIO24 - GIO33 Function Select register.             */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetFunctionSelect1(void)
{
	return pGIORegs->fsel1;
}

/*--------------------------------------------------------------------------*/
/* GIOSetFunctionSelect1                                              	    */
/* This function is used to set GIO24 - GIO33 Function Select register.     */
/*--------------------------------------------------------------------------*/
void GIOSetFunctionSelect1(unsigned short fsel1val)
{
	pGIORegs->fsel1 = fsel1val;
}
/*--------------------------------------------------------------------------*/
/* End-Of-GIO_Overlay.                                                      */
/*--------------------------------------------------------------------------*/



/*--------------------------------------------------------------------------*/
/* Interrupt Controller (Similar to the GIO settings)                       */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* End-Of-Interrupt Controller                                              */
/*--------------------------------------------------------------------------*/



/*--------------------------------------------------------------------------*/
/* External BUS Controller: Bus Mode: Memory map/IO map (0004:0800 ~ 0FFF)  */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* End-Of-External Bus Controller                                           */
/*--------------------------------------------------------------------------*/
#endif

/* ######## H/W Specific Stuff (Start) ######## */
#define MACADDR_BASE	(BSPCONF_FLASH_BASE+WMV_MAC_FLASH_OFFSET)
#define MACADDR_LEN  6		/* the same as if_ether.h (ETH_ALEN) */
typedef struct MACAddrRegister {
   unsigned char macaddr[MACADDR_LEN];
} __attribute__ ((packed)) MACAddrRegister_t;

int WMV_MACAddrValid(unsigned char *addr)
{
	int i;
	for (i=0; i<MACADDR_LEN; i++) { /* Check all <00> */
		if (addr[i] != 0x00) {
			break;
		}
	}
	if (i == MACADDR_LEN) {
		return 0;	/* not valid */
	}
	for (i=0; i<MACADDR_LEN; i++) { /* Check all <FF> */
		if (addr[i] != 0xFF) {
			break;
		}
	}
	if (i == MACADDR_LEN) {
		return 0;	/* not valid */
	}
	return 1;
}  // End of NC7XXMACAddrValid
int WMV_GetMACAddr(unsigned char *addr)
{
	MACAddrRegister_t *pMACReg = (MACAddrRegister_t *)MACADDR_BASE;
	unsigned char data[MACADDR_LEN];

			/* ---- read from the flash ---- */
	memset(&data[0], '\0', MACADDR_LEN);
	memcpy(&data[0], pMACReg->macaddr, MACADDR_LEN);
			/* -- all <00>/<FF> ??, return default value. -- */
			/* -- but do NOT write back to the FLASH      -- */
	if (!WMV_MACAddrValid(&data[0])) {
		data[0] = 0x00;
		data[1] = 0xC0;
		data[2] = 0x02;
		data[3] = 0x00;
		data[4] = 0x00;
		data[5] = 0x00;	/* default = 00:C0:02:00:00:00 */
	} /* end-if: if the data is invalid, use default */
	memcpy(addr, &data[0], MACADDR_LEN);
	return 0;
}  // End of NC7XXGetMACAddr
void WMV_SetMACAddr(unsigned char *value)
{
}  // End of NC7XXSetMACAddr

/*--------------------------------------------------------------------------*/
/* End-Of-Mac Address for LAN and/or WLAN                                   */
/*--------------------------------------------------------------------------*/
/* ######## H/W Specific Stuff (End  ) ######## */
