/**************************************************************************
 *
 * linux/include/asm-armnommu/arch-dm270/cache.h
 *
 * Copyright (C) 2003 Cadenux, LLC (http://www.cadenux.com)
 * author:  Gregory Nutt <greg.nutt@cadenux.com>
 *
 **************************************************************************/

#ifndef __ASM_ARCH_CACHE_H
#define __ASM_ARCH_CACHE_H

/**************************************************************************
 * Included Files
 **************************************************************************/

#include <asm/arch/hardware.h>

/**************************************************************************
 * Definitions
 **************************************************************************/

/* BITS	FIELD-NAME	DESCRIPTION
 * 8	REQ4W		Enable 4 burst Instruction read from SDRAM.
 *			1: 4 word burst access request
 *			0: Single word access request
 * 1	ICACLR		Cache Clear bit. When write `1' to this bit,
 *			ICAENB MUST be `0'.
 *			1: ICACHE Clear start
 *			0: ICACHE Clear done
 * 0	ICAENB		Cache enable bit.
 *			1: ICACHE is enable
 *			0: ICACHE is disable (bypass)
 *
 * When CACLR bit is set to 1, clearing of cache starts. When this bit is
 * read during clearing operation, 1 is returned, and back to 0 after
 * clearing completed. ICAENB must be 0 when writing 0 to ICACLR.
 *
 * After reset, ensure that cache-clearing operation is conducted by
 * writing 1 to ICACLR for initialization of TAG RAM.
 *
 * When the program located at external memory is updated, cache should be
 * cleared by writing 1 to ICACLR.
 *
 * REQ4W enable burst transfer (4 burst) from SDRAM to ICACHE, when the
 * program is located in SDRAM. (When 32-bit SDRAM is used) REQ4W can be
 * set, only when ICAENB is 1. (REQ4W and ICAENB can be set to 1 at same
 * time.)
 */

#define DM270_ICAMODE_ICAENB 0x0001 /* 4 word burst access request */
#define DM270_ICAMODE_ICACLR 0x0002 /* Cache clear bit */
#define DM270_ICAMODE_REQ4W  0x0100 /* Cache enable bit */

#define ENABLE_SETTING  DM270_ICAMODE_ICAENB
#define DISABLE_SETTING 0

/**************************************************************************
 * Assembly Language Macros
 **************************************************************************/

	.macro	dm270_clear_cache, reg1, reg2, reg3
	ldr	\reg2, =DM270_ICACHE_ICAMODE	/* reg2 = ICAMODE addr */
	ldrh	\reg3, [\reg2]			/* reg3 = current enables */
	mov	\reg1, #DM270_ICAMODE_ICACLR	/* reg1 = clear bit */
	strh	\reg1, [\reg2]			/* set clear bit */
1:
 	ldrh	\reg1, [\reg2]			/* Re read ICAMODE reg */
 	tst	\reg1, #DM270_ICAMODE_ICACLR	/* Still clearing? */
	bne	1b				/* Yes, loop til done */
	strh	\reg3, [\reg2]			/* Restore enable setting */
	.endm

	.macro	dm270_enable_cache, reg1, reg2
	ldr	\reg1, =ENABLE_SETTING		/* reg1 = bits to enable */
	ldr	\reg2, =DM270_ICACHE_ICAMODE	/* reg2 = ICAMODE addr */
	strh	\reg1, [\reg2]			/* enable icache */
	.endm
	
	.macro	dm270_disable_cache, reg1, reg2
	mov	\reg1, #DISABLE_SETTING		/* reg1 = bits to enable */
	ldr	\reg2, =DM270_ICACHE_ICAMODE	/* reg2 = ICAMODE addr */
	strh	\reg1, [\reg2]			/* enable icache */
	.endm

#endif /* __ASM_ARCH_CACHE_H */
