/**************************************************************************
 * linux/include/asm-armnommu/arch-dm270/ccd_ioctl.h
 *
 * CCD Control Driver Header File
 *    
 * Derived from TI's DM270 CCDC.h
 *       
 * This header is to be used by application programs needing to control the
 * CCD Controller register on the DM270.
 *            
 *   Copyright (C) 2003 Cadenux, LLC. All rights reserved.
 *   Todd Fischer <todd.fischer@cadenux.com>
 *
 * Derived from:
 *
 *   linux/include/asm-armnommu/arch-dsc21/ccd_ioctl.h
 *            
 *   Copyright (C) 2003 Cadenux, LLC. All rights reserved.
 *   Gregory Nutt <greg.nutt@cadenux.com>
 *
 *   Author:  Marcus Smith
 *   Date: 05/14/2001
 *             
 **************************************************************************/

#ifndef __ASM_ARCH_CCD_IOCTL_H
#define __ASM_ARCH_CCD_IOCTL_H
 
/**************************************************************************
 * Included Files
 **************************************************************************/

#include <asm/ioctl.h>

/**************************************************************************
 * Public Definitions
 **************************************************************************/

/**************************************************************************
 * Public Types
 **************************************************************************/
 

struct CCDSetRawDataMode
{
    unsigned short nX;                     // [IN] right position of the window
    unsigned short nY;                     // [IN] top position of the window
    unsigned short nWidth;                 // [IN] window width
    unsigned short nHeight;                // [IN] window height
    unsigned char  nHevenPttn;             // [IN] horizontal down-sampling pattern for even line
    unsigned char  nHoddPttn;              // [IN] horizontal down-sampling pattern for odd line
    unsigned char  nVPttn;                 // [IN] vertical down-sampling pattern
    unsigned char  bInterlace;             // [IN] non-zero:interlace mode, 0: non-interlace mode
    unsigned char  b2LineAlternate;        // [IN] non-zero:2-line alternated, 0:1-line
};


struct CCDConfigData {
    unsigned short dataInputMode;           // [IN] Data input mode: use CCD_DATA_INPUT_... values
    unsigned short ccdDataWidth;            // [IN] Input pixel width in units of bits, valid data widths are 8..14
    unsigned short ccdModeInterlace;        // [IN] CCD reading method, non-zero: Interlaced mode , 0: Progressive mode
    unsigned short dataPolarityInverse;     // [IN] Data polarity, non-zero: Inverse , 0: Non-inverse
    unsigned short useWen;                  // [IN] WEN signal, non-zero: Use WEN signal, 0: Don't use WEN signal
    unsigned short fieldIdPolarityNegative; // [IN] Field ID polarity, non-zero: Negative, 0: Positve
    unsigned short hdPolarityNegative;      // [IN] HD sync polarity, non-zero: Negative, 0: Positive
    unsigned short vdPolarityNegative;      // [IN] VD polarity, non-zero: Negative, 0: Positive
    unsigned short fieldIdDirOutput;        // [IN] Field ID direction, non-zero: Output, 0: Input
    unsigned short hdvdDirOutput;           // [IN] HD/VD direction, non-zero: Output, 0: Input
    unsigned short rec656EnableInterface;   // [IN] REC656 interface, non-zero: enable, 0: disable
    unsigned short rec656ErrorCorrection;   // [IN] REC656 error correction, non-zero: enable, 0: disable
    unsigned short vsyncLatch;              // [IN] non-zero: not latched on VSYNC, 0: latched on VSYNC
    unsigned short chromaOutMsbInverted;    // [IN] MSB of chroma output signal, non-zero: MSB inverted, 0: normal
    unsigned short chromaInMsbInverted;     // [IN] MSB of chroma input signal stored, non-zero: MSB inverted, 0: normal
    unsigned short byteSwap;                // [IN] non-zero: swap bytes, 0: normal
    unsigned short ycbcrYLocation;          // [IN] Y signal location (YCbCr 8bit data only), non-zero: odd pixel, 0: even pixel
    unsigned short externalTriggerSense;    // [IN] External trigger sense: non-zero: falling edge, zero: rising edge
    unsigned short sdramInitHwSignal;       // [IN] Signal that initializes SDRAM address, non-zero: FID input, 0: DWEN bit
    unsigned short wenDataCapture;          // [IN] Data caputure using WEN), non-zero: data stored on WEN or frame definition
                                            //                                    zero: data stored on WEN and frame definition
    unsigned short fidDetection;            // [IN] FID Detection: use CCD_FID_MODE_... values
    unsigned short ccir656DataWidth;        // [IN] Data width (when in CCIR656 mode), non-zero: 10 bits, zero: 8 bits
    unsigned short yinCinSwap;              // [IN] Y input and C input swapping, non-zero: swap C and Y, 0: normal
    unsigned short youtCoutControl;         // [IN] Y output and C output control, non-zero: input passthrough, 0: video encoder output
    unsigned short youtCoutSwap;            // [IN] Y output and C output swapping, non-zero: swap C and Y, 0: normal
    unsigned short yinCinPort;              // [IN] Y input and C input port: non-zero: Y and C output port, 0: Y and C input port
    unsigned short youtCoutHwDirection;     // [IN] YOUT and COUT direction: non-zero: input, 0: output
};

struct CCDCaptureModeData {
    unsigned short hdWidth;                 // [IN] HD width, 1..4096, units: pixel clock cycles
    unsigned short vdWidth;                 // [IN] VD Width, 1..4096, units: lines
    unsigned short pixelsPerLine;           // [IN] Pixels per line, 1..65536
    unsigned short linesPerFrame;           // [IN] Lines per frames, 1..32768
    unsigned short startPixelH;             // [IN] Start pixel horizontal, 0..4095
    unsigned short numPixelH;               // [IN] Number of horizontal pixels, 1..4096
    unsigned short startLineVEvenField;     // [IN] Start line vertical, Even Field, 0..4095
    unsigned short startLineVOddField;      // [IN] Start line vertical, Odd  Field, 0..4095
    unsigned short numPixelV;               // [IN] Number of vertical lines, 1..4096
    unsigned short cullPatternHEvenLine;    // [IN] Horiz cull pattern mask bitmap - even line, bit set: retail, bit clear: cull
    unsigned short cullPatternHOddLine;     // [IN] Horiz cull pattern mask bitmap - odd line, bit set: retail, bit clear: cull
    unsigned short cullPatternV;            // [IN] Vert cull pattern mask bitmap - odd line, bit set: retail, bit clear: cull
    unsigned short FieldIdSignalPolarity;   // [IN] Field identification signal polarity - non-zero: inverse, 0: normal
             short Field1LineOffset;        // [IN] Line offset value for field 1: +1 to +4 lines 
             short Field1EvenLineOffset;    // [IN] Line offset value for field 1 even line: -4 to +4 lines, 0=-4 lines
             short Field1OddLineOffset;     // [IN] Line offset value for field 1 odd line: -4 to +4 lines, 0=-4 lines
             short Field0EvenLineOffset;    // [IN] Line offset value for field 0 even line: -4 to +4 lines, 0=-4 lines
             short Field0OddLineOffset;     // [IN] Line offset value for field 0 odd line: -4 to +4 lines, 0=-4 lines
    unsigned short sdramLineOffset;         // [IN] SDRAM line offset, in units of bytes, must be multiple of 32 bytes
    unsigned short sdramStorageMode;        // [IN] SDRAM storage mode, non zero: 8 bit/pixel, 0: 16 bit/pixel
    unsigned short vdint0Interval;          // [IN] VD0 interrupt timing, 0..4095 lines from start of VD
    unsigned short vdint1Interval;          // [IN] VD1 interrupt timing, 0..4095 lines from start of VD
    unsigned short shutterControlEnable;    // [IN] Shutter Control, non-zero: Enable, 0: Disable
};

struct CCDProcessingData {
    unsigned short hlpfEnable;              // [IN] Horizontal low pass filter, non-zero: Enable, 0: Disable
    unsigned short clampEnable;             // [IN] Clamp enable, non-zero: Enable, 0: Disable
    unsigned short clampPixel;              // [IN] Number of pixels for clamp detection, non-zero: 16 pixels, zero: 8 pixels
    unsigned short clampLine;               // [IN] Number of lines used for clamp detection, non-zero: 16 lines, 0: 8 lines
    unsigned short clampStart;              // [IN] Clamp detection start position in units of pixels from start of HD, 0..4095
    unsigned short dcsubValue;              // [IN] DC value subtracted when clampEnable is disabled, 0..16383
    unsigned short colPatFld0EvenEven;      // [IN] Color Pattern, Field 0, Even pixel, Even line: use CCD_COLOR_PATTERN_... values
    unsigned short colPatFld0OddEven;       // [IN] Color Pattern, Field 0, Odd  pixel, Even line: use CCD_COLOR_PATTERN_... values
    unsigned short colPatFld0EvenOdd;       // [IN] Color Pattern, Field 0, Even pixel, Odd  line: use CCD_COLOR_PATTERN_... values
    unsigned short colPatFld0OddOdd;        // [IN] Color Pattern, Field 0, Odd  pixel, Odd  line: use CCD_COLOR_PATTERN_... values
    unsigned short colPatFld1EvenEven;      // [IN] Color Pattern, Field 1, Even pixel, Even line: use CCD_COLOR_PATTERN_... values
    unsigned short colPatFld1OddEven;       // [IN] Color Pattern, Field 1, Odd  pixel, Even line: use CCD_COLOR_PATTERN_... values
    unsigned short colPatFld1EvenOdd;       // [IN] Color Pattern, Field 1, Even pixel, Odd  line: use CCD_COLOR_PATTERN_... values
    unsigned short colPatFld1OddOdd;        // [IN] Color Pattern, Field 1, Odd  pixel, Odd  line: use CCD_COLOR_PATTERN_... values
    short offsetR;                          // [IN] R offset, -128..127
    short offsetGr;                         // [IN] Gr offset, -128..127
    short offsetGb;                         // [IN] Gb offset, -128..127
    short offsetB;                          // [IN] B offset, -128..127
    unsigned short medianFilterEnable;      // [IN] Median filter enable, non-zero: Enable, 0: Disable
    unsigned short medianFilterThreshold;   // [IN] Median filter threshold, 0..8191
    unsigned short gain;                    // [IN] Gain, 0..(7.9921875), Format U11Q8
    unsigned short offset;                  // [IN] Offset, 0..1023
    unsigned short outputThreshold;         // [IN] Output threshold, 0..0x3FFF
    unsigned short outputClip;              // [IN] Output clip, 0..0x3FFF
    unsigned short gammaInputWidth;         // [IN] Input data width to A-law is selected: use CCD_GAMMA_WIDTH_... values
    unsigned short gammaH3AEnable;          // [IN] A-law table for H3A, non-zero: Enable, 0: Disable
    unsigned short gammaCCDEnable;          // [IN] A-law table for CCD, non-zero: Enable, 0: Disable
};

struct CCDCapture
{
        unsigned int numVdInterrupts;       // [IN] number of VD interrupts to wait for before returning
};

struct CCDEnable
{
    unsigned char dataWriteEnable;          // [IN] Data write enable, non-zero: enable, 0: disable
    unsigned char vdHdEnable;               // [IN] Enable VD/HD (activites internal timing generator if VD/HD are output,
                                            // otherwise enables internal timing generator to sync with external signal),
                                            // non-zero: enable, 0: disable
};

enum modeType
{
  eDraft,
  eFine
};

struct CCDAfConfigData
{
    unsigned short fvAccMode;               // [IN] Focus Value Accumulation Mode: use CCD_AF_MODE_...
    unsigned char  greenPixel[2][2];        // [IN] Green Pixel Extraction: use CCD_AF_GREEN_...
                                            //      [0][0]: an upper left pixel  in 2x2 block,
                                            //      [0][1]: an upper right pixel in 2x2 block,
                                            //      [1][0]: a  lower left pixel  in 2x2 block,
                                            //      [1][1]: a  lower rigth pixel in 2x2 block
    unsigned short winHeight;               // [IN] Paxel Height, range from 2 to 64 - REVISIT: driver divides by 2
    unsigned short winWidth;                // [IN] Paxel Width,  range from 1 to 256- REVISIT: driver divides by 2
    unsigned short numWinH;                 // [IN] Paxel Count for H direction, range from 1 to 9
    unsigned short numWinV;                 // [IN] Paxel Count for V direction, range from 1 to 4
    unsigned short winStartH;               // [IN] Paxel Start Position H direction, range from 1 to 4095, 
                                            //      must be equal to or greater than (IIR Filter Start Position + 1)
    unsigned short winStartV;               // [IN] Paxel Start Position V direction, range from 0 to 4095
    unsigned short iirStartH;               // [IN] IIR Start Position, range from 0 to 4095
    unsigned short winLineIncrement;        // [IN] Line increment in a paxel, range from 1 to 16
    unsigned int   sdramAddress;            // [IN] Absolute SDRAM Address to store the results of AF engine. 
                                            //      Must be 32byte aligned
    unsigned short iircoeff_0[11];          // [IN] IIR Filter Coeffecients for table 1. 
                                            //      Refer register manual for format of these values
    unsigned short iircoeff_1[11];          // [IN] IIR Filter Coeffecients for table 2. 
                                            //      Refer register manual for format of these values
};

struct CCDAewbConfigData
{
    unsigned short saturationLimit;         // [IN] Saturation limit 0..255
    unsigned short winHeight;               // [IN] Window Height, range from 4 to 64, Should be an even number
    unsigned short winWidth;                // [IN] Window Width, range from 2 to 256, Should be an even number
    unsigned short numWinH;                 // [IN] Window Count for H direction, range from 1 to 12
    unsigned short numWinV;                 // [IN] Window Count for V direction, range from 1 to 8
    unsigned short winStartH;               // [IN] Window Start Position H direction, range from 0 to 4094, 
                                            //      Should be an even number
    unsigned short winStartV;               // [IN] Window Start Position V direction, range from 0 to 4094, 
                                            //      Should be an even number
    unsigned short winIncH;                 // [IN] Sampling point increments in window H direction, range from 2 to 32, 
                                            //      Should be an even number
    unsigned short winIncV;                 // [IN] Sampling point increments in window V direction, range from 2 to 32, 
                                            //      Should be an even number
    unsigned int sdramAddress;              // [IN] Absolute SDRAM Address to store the results of AE/AWB engine. 
                                            //      Must be 32 byte aligned
};

struct CCDAfData
{
    unsigned short vdIntID;                 // [IN] Interrupt to wait on - use: CCD_VD_INTR_ID_...
    unsigned int numVdInterrupts;           // [IN] number of VD interrupts to wait for before returning
                                            //      normally 1
    unsigned int sdramAddress;              // [OUT] Location of AutoFocus calculation results
};

struct CCDAewbData
{
    unsigned short block;                   // [IN] non-zero if application should be blocked until 
                                            //      interrupt is received
    unsigned short vdIntID;                 // [IN] Interrupt to wait on (not used if block is zero): 
                                            //      use CCD_VD_INTR_ID_...
    unsigned int sdramAddress;              // [OUT] Location of AutoExposure / AutoWhiteBalance calculation results
};


/**************************************************************************
 * Definitions
 **************************************************************************/
 
/* List of ioctl commands. */
 
#define CCD_MAGIC_NUMBER 'D'

#define CCD_ENABLE                 _IOW( CCD_MAGIC_NUMBER, 0x0, sizeof(struct CCDEnable))          /* dm270 */
#define CCD_SET_LINE_PER_FRAME     _IOW( CCD_MAGIC_NUMBER, 0x1, sizeof(unsigned short))
#define CCD_SET_RAW_DATA_MODE      _IOW( CCD_MAGIC_NUMBER, 0x2, sizeof(struct CCDSetRawDataMode))
#define CCD_GET_FRAME              _IOR( CCD_MAGIC_NUMBER, 0x3, sizeof(struct CCDCapture))         /* dm270 */
#define CCD_SET_SDRAM_ADDR         _IOW( CCD_MAGIC_NUMBER, 0x4, sizeof(unsigned long))             /* dm270 */
#define CCD_SET_MODE               _IOW( CCD_MAGIC_NUMBER, 0x5, sizeof(modeType))
#define CCD_CONFIGURE_CONTROLLER   _IOW( CCD_MAGIC_NUMBER, 0x6, sizeof(struct CCDConfigData))      /* dm270 */
#define CCD_SET_CAPTURE_MODE       _IOW( CCD_MAGIC_NUMBER, 0x7, sizeof(struct CCDCaptureModeData)) /* dm270 */
#define CCD_IMAGE_PROCESSING       _IOW( CCD_MAGIC_NUMBER, 0x8, sizeof(struct CCDProcessingData))  /* dm270 */
#define CCD_SET_AF_CONFIGURATION   _IOW( CCD_MAGIC_NUMBER, 0x9, sizeof(struct CCDAfConfigData))    /* dm270 */
#define CCD_SET_AEWB_CONFIGURATION _IOW( CCD_MAGIC_NUMBER, 0xA, sizeof(struct CCDAewbConfigData))  /* dm270 */
#define CCD_CALCULATE_AF           _IOWR(CCD_MAGIC_NUMBER, 0xB, sizeof(struct CCDAfData))          /* dm270 */
#define CCD_ENABLE_AEWB            _IOW( CCD_MAGIC_NUMBER, 0xC, sizeof(unsigned short))            /* dm270 */
#define CCD_RETRIEVE_AEWB          _IOWR(CCD_MAGIC_NUMBER, 0xD, sizeof(struct CCDAewbData))        /* dm270 */

#define CCD_IOC_MAXNR 0xD


/* CCD Color Patterns - colPatFld... in struct CCDProcessingData */

#define CCD_COLOR_PATTERN_RED_YELLOW   0
#define CCD_COLOR_PATTERN_GREEN_CYAN   1
#define CCD_COLOR_PATTERN_GB_G         2
#define CCD_COLOR_PATTERN_BLUE_MAGENTA 3

/* Data Input Modes - dataInputMode in struct CCDConfigData */

#define CCD_DATA_INPUT_RAW             0
#define CCD_DATA_INPUT_YUV16           1
#define CCD_DATA_INPUT_YUV8            2

/* FID Detection Modes - fidDetection in struct CCDConfigData */

#define CCD_FID_MODE_VD_HEAD           0
#define CCD_FID_MODE_DIRECT            1
#define CCD_FID_MODE_RISING_FALLING_VD 2
#define CCD_FID_MODE_IN_PHASE          3

/* Gamma Input Widths - gammaInputWidth in struct CCDProcessingData */

#define CCD_GAMMA_WIDTH_13_4           0
#define CCD_GAMMA_WIDTH_12_3           1
#define CCD_GAMMA_WIDTH_11_2           2
#define CCD_GAMMA_WIDTH_10_1           3
#define CCD_GAMMA_WIDTH_9_0            4

/* Focus Value Accumulation Mode - fvAccMode in struct CCDAfConfigData */

#define CCD_AF_MODE_SUM                0
#define CCD_AF_MODE_PEAK               1

/* Green Pixel Extraction: - greenPixel in struct CCDAfConfigData */

#define CCD_AF_GREEN_NON               0
#define CCD_AF_GREEN                   1

/* vd Interrupt Identification:  vdIntID in struct CCDAewbData */

#define CCD_VD_INTR_ID_0               0
#define CCD_VD_INTR_ID_1               1

/* Major numbers, (from Documentation/devices.txt local/experimental list)  */
 
#define CCD_CONTROL_MAJOR     122

#endif /* __ASM_ARCH_CCD_IOCTL_H */
