/*
 * asm/arch-armnommu/arch-dm270/dma.h:
 *
 *   Copyright (C) 2003 Cadenux, LLC (http://www.cadenux.com)
 *   author:  Gregory Nutt <greg.nutt@cadenux.com>
 *
 *
 * Derived from asm/arch-armnommu/arch-dsc25/dma.h
 * Derived from asm/arch-armnommu/arch-dsc21/dma.h
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __ASM_ARCH_DMA_H
#define __ASM_ARCH_DMA_H

#include "asm/arch/hardware.h"
#include "asm/arch/memconfig.h"

/* Type dmamode_t is declared in include/asm-armnommu along with values.
 * The DM270 only supports DMA between External memory, the serial
 * interface, USB, and and MMC/SD.  The following additional DMA mode
 * values are declared here extend the possible meanings for dmamode_t
 * as used by the function set_dma_mode().
 */
                               /* DMA is between SDRAM and... */
#define DMA_MODE_EXTMEM   0x08 /* the external memory interface */
#define DMA_MODE_SERIAL   0x10 /* the serial interface */
#define DMA_MODE_USB      0x20 /* USB */
#define DMA_MODE_MMCSD    0x40 /* the MMC/SD controller */

#define DMA_ENDPOINT_MASK 0x78 /* Isolates the above settings */

/* MAX_DMA_ADDRESS -- max address we can program into the DMA address
 * registers? Not sure. Various drivers use this value as a test before
 * setting up a DMA. Also, the macros in linux/bootmem.h use this value
 * as a goal for many of the bootmem allocations.
 */

/* Configured memory is specified during configuration. */

#include "linux/autoconf.h"
#define MAX_DMA_ADDRESS (BSPCONF_DRAM_BASE + BSPCONF_DRAM_SIZE - 1)

/* MAX_DMA_CHANNELS -- # of DMA control register sets we possess.
 * The DM270 SDRAM controller supports two DMA channels which are
 * used to support DMAs involving SDRAM abd EMIF/Serial IF/USB/MMC/SD.
 * The DMA channel is selected via the DM270's REFCTL register;
 * DMA channel SDRAM access priority can also be controlled using
 * SDPRTY1-9.
 */

#define MAX_DMA_CHANNELS 2

#endif /* __ASM_ARCH_DMA_H */
