/**************************************************************************
 * linux/include/asm-nommu/arch-dm270/extmem.h
 * External Memory Register Definitions for the DM270.
 *
 *   Copyright (C) 2003 Cadenux, LLC (http://www.cadenux.com)
 *   author:  Gregory Nutt <greg.nutt@cadenux.com> 
 *
 **************************************************************************/

/* This file contains register settings that control the operation
 * of the DM270's external memory controller.  External memory
 * register definitions are contained in hardware.h.
 */

#ifndef __ASM_ARCH_EXTMEM_H
#define __ASM_ARCH_EXTMEM_H

/**************************************************************************
 * Included Files
 **************************************************************************/

#include <linux/config.h>          /* Kernel configuration */
#include <asm/arch/memconfig.h>    /* BSP configuration */
#include <asm/arch/hardware.h>     /* DM270 Register definitions */
#include <asm/arch/irqs.h>         /* DM270 IRQ definitions */

/**************************************************************************
 * Definitions
 **************************************************************************/

/**************************************************************************
 * External Bus Initial Settings
 **************************************************************************/

#if defined(BSPCONF_DM270_INGENIENT)
# define DM270_EXBC_CS0CTRL1_DATA    0x889a
# define DM270_EXBC_CS0CTRL2_DATA    0x1110
# define DM270_EXBC_CS1CTRL1A_DATA   0x1415
# define DM270_EXBC_CS1CTRL1B_DATA   0x1109
# define DM270_EXBC_CS1CTRL2_DATA    0x1220
# define DM270_EXBC_CS2CTRL1_DATA    0x3207
# define DM270_EXBC_CS2CTRL2_DATA    0x1129
# define DM270_EXBC_CS3CTRL1_DATA    0xBBFF
# define DM270_EXBC_CS3CTRL2_DATA    0x7220
# define DM270_EXBC_CS4CTRL1_DATA    0x225b
# define DM270_EXBC_CS4CTRL2_DATA    0x4110
#else
# define DM270_EXBC_CS0CTRL1_DATA    0x5877 /* OE=6w  WE=9w  CE=8w  CYC=8w */
# define DM270_EXBC_CS0CTRL2_DATA    0x3110 /* IDLE=3 OE=1   WE=1   CE=0 */
# define DM270_EXBC_CS1CTRL1A_DATA   0x1415 /* CE=21w CYC=22w */
# define DM270_EXBC_CS1CTRL1B_DATA   0x1109 /* OE=18w WE=10w */
# define DM270_EXBC_CS1CTRL2_DATA    0x1220 /* IDLE=1 OE=2   WE=2   CE=0 */
# define DM270_EXBC_CS2CTRL1_DATA    0x110a /* OE=2w  WE=2w  rsvd   CYC=11w */
# define DM270_EXBC_CS3CTRL1_DATA    0x1129 /* OE=2w  WE=2w  CE=3w  CYC=10w */
# define DM270_EXBC_CS2CTRL2_DATA    0x8899 /* IDLE=0 OE=8   WE=9   RE=12 */
# define DM270_EXBC_CS3CTRL2_DATA    0x1110 /* WD=8b  IDLE=1 OE=1   CE=0 */
# define DM270_EXBC_CS4CTRL1_DATA    0x979C /* OE=10w  WE=8w  CE=10w CYC=11w */
# define DM270_EXBC_CS4CTRL2_DATA    0x5220 /* LO BUS=16bit IDLE=3 OE=3 CE=0 */
#endif

#define DM270_EXBC_BUSCTRL_DATA     0x0003 /* Enable EMWE3 */
#define DM270_EXBC_BUSRLS_DATA      0x0000 /* normal not-open not-open */
#define DM270_EXBC_CFCTRL1_DATA     0x0000 /* CF card selected */

#if defined(BSPCONF_DM270_INGENIENT)
# define DM270_EXBC_CFCTRL2_DATA    0x0000 /* CF dynamic sizing high output */
#else
# define DM270_EXBC_CFCTRL2_DATA    0x0000 /* CF 16bit low output */
#endif

#define DM270_EXBC_SMCTRL_DATA      0x0001 /* SCE low level */
#define DM270_EXBC_BUSINTEN_DATA    0x0000 /* CF low-high disabled */
#define DM270_EXBC_BUSSTS_DATA      0x0007 /* IOIS16+CFWAIT+CFRDY shown */
#define DM270_EXBC_BUSWAITMD_DATA   0x0000 /* CF RDY/-BSY connected */
#define DM270_EXBC_ECC1CP_DATA      0x00ff /* SM ECC CP */
#define DM270_EXBC_ECC1LP_DATA      0xffff /* SM ECC LP */
#define DM270_EXBC_ECC2CP_DATA      0x00ff /* SM ECC CP */
#define DM270_EXBC_ECC2LP_DATA      0xffff /* SM ECC LP */
#define DM270_EXBC_ECCCLR_DATA      0x0000 /* SM ECC clear */
#define DM270_EXBC_PAGESZ_DATA      0x0001 /* SM page size = 256+8byte/page */
#define DM270_EXBC_IMGDSPDEST_DATA  0x0000 /* IMG source=CS0 dest=CS0 */
#define DM270_EXBC_PRIORCTL_DATA    0x001b /* Priorities */
#define DM270_EXBC_SOURCEADD1_DATA  0x0000 /* DMA addr incr + upper address */
#define DM270_EXBC_SOURCEADD2_DATA  0x0000 /* DMA lower address */
#define DM270_EXBC_DESTADD1_DATA    0x0000 /* DMA addr incr + upper address */
#define DM270_EXBC_DESTADD2_DATA    0x0000 /* DMA lower address */
#define DM270_EXBC_DMASIZE_DATA     0x0000 /* DMA size of transfer */
#define DM270_EXBC_DMADEVSEL_DATA   0x0000 /* DMA src=CS0 dest=CS0 */
#define DM270_EXBC_DMACTL_DATA      0x0000 /* DMA no switching, not started */
#define DM270_EXBC_IMGDSPADD1_DATA  0x0000 /* IMG upper address */
#define DM270_EXBC_IMGDSPADD2_DATA  0x0000 /* IMG lower addres */

#if defined(BSPCONF_DM270_INGENIENT)
# define DM270_EXBC_DPSTR0_DATA     0x0009 /* CS0 (FLASH) 10:0000-090:0000 */
# define DM270_EXBC_DPSTR1_DATA     0x0049 /* SDRAM       90:0000-049:0000 */
# define DM270_EXBC_DPSTR2_DATA     0x0059 /* CS1 (CF)   490:0000-590:0000 */
# define DM270_EXBC_DPSTR3_DATA     0x0069 /* CS2 (SM)   590:0000-690:0000 */
# define DM270_EXBC_DPSTR4_DATA     0x0079 /* CS3 (IDE)  690:0000-790:0000 */
# define DM270_EXBC_DPSTR5_DATA     0x0081 /* CS4 ether  790:0000-810:0000 */
#else
# define DM270_EXBC_DPSTR0_DATA     0x0021 /* CS0 (FLASH) 10:0000-210:0000 */
# define DM270_EXBC_DPSTR1_DATA     0x0061 /* SDRAM      210:0000-610:0000 */
# define DM270_EXBC_DPSTR2_DATA     0x0066 /* CS1 (CF)   610:0000-660:0000 */
# define DM270_EXBC_DPSTR3_DATA     0x006a /* CS2 (SM)   660:0000-6a0:0000 */
# define DM270_EXBC_DPSTR4_DATA     0x006b /* CS3        6a0:0000-6b0:0000 */
# define DM270_EXBC_DPSTR5_DATA     0x006c /* CS4 ether  6b0:0000-6c0:0000 */
#endif

#define DM270_EXBC_TEST_DATA        0x0000 /* Normal mode */

/* Given the above initialization, the external memory regions will be
 * positioned as follows:
 */

#define DM270_CS0_START_ADDR        0x00100000
#define DM270_CS1_START_ADDR        (DM270_EXBC_DPSTR1_DATA << 20)
#define DM270_CS2_START_ADDR        (DM270_EXBC_DPSTR2_DATA << 20)
#define DM270_CS3_START_ADDR        (DM270_EXBC_DPSTR3_DATA << 20)
#define DM270_CS4_START_ADDR        (DM270_EXBC_DPSTR4_DATA << 20)

/**************************************************************************
* Compact flash settings used by the CF IDE driver (CS1) 
 **************************************************************************/

/* CFC CFRDY interrupt */

#define DM270_CFC_CFRDY_IRQ         DM270_IRQ_MTC1 /* MTC 1 */

/* CFC addressing */

#define DM270_CFC_BASE_ADDR         DM270_CS1_START_ADDR
#define DM270_CFC_IOSPACE_ADDR      (DM270_CFC_BASE_ADDR + 0x00400000)

/* CFCTRL1 bit definitions */

#define DM270_CFCTRL1_CFSEL         0x0001 /* 0=CFC, 1=SSFDC */

/* CFCTRL2 bit definitions */

#define DM270_CFCTRL2_BUSSZ         0x0010 /* 0=16bit CF, 1=dynamic width */
#define DM270_CFCTRL2_CFMOD         0x0001 /* 0=access to attribute memory,
					    * 1=access to common memory */

/* CFCTRL2 convenience values */

#define DM270_CFC_ATTRIBUTE_MEM     0x0000
#define DM270_CFC_COMMON_MEM        DM270_CFCTRL2_CFMOD

/* SMCTRL bit definitions */

#define DM270_SMCTRL_SCEH           0x0001 /* 0=SCE low, 1=SCE high */

/* BUSINTEN bit definitions */

#define DM270_BUSINTEN_CBCND        0x0002 /* 0=lo-to-hi, 1=hi-to-lo */
#define DM270_BUSINTEN_CBIEN        0x0001 /* 0=disable, 1=enable */

/* BUSSTS bit definitions */

#define DM270_BUSSTS_EMWT           0x0008 /* State of EM_WAI terminal */
#define DM270_BUSSTS_IO16           0x0008 /* State of IO16 terminal */
#define DM270_BUSSTS_CFWT           0x0008 /* State of CFWAIT terminal */
#define DM270_BUSSTS_CFRDY          0x0008 /* State of CFRDY terminal */

/* BUSWAITMD bit definitions */

#define DM270_BUSWAITMD_CFRSL       0x0002 /* 0=RDY/-BSY of CF card,
					    * 1=RDY/-BSY of SmartMedia card */
#define DM270_BUSWAITMD_CFRW        0x0001 /* 0=Access to card has begun;
					    * 1=Access not begun (w/CFRDY low)
					    */

#endif /* __ASM_ARCH_EXTMEM_H */
