/*
 * include/asm-armnommu/arch-dm270/irqs.h:
 *
 *   Copyright (C) 2003 Cadenux, LLC (http://www.cadenux.com)
 *   author:  Gregory Nutt <greg.nutt@cadenux.com>
 *
 * Derived from include/asm-armnommu/arch-dsc25/irqs.h:
 * Adapted from include/asm-armnommu/arch-dsc21/irqs.h
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#ifndef __ASM_ARCH_IRQS_H__
#define __ASM_ARCH_IRQS_H__

#include <linux/autoconf.h>

/* irq #s as ordered in DM270_IRQ0_STATUS and DM270_IRQ1_STATUS */

#define DM270_IRQ_TIMER0   0 /* Timer 0 */
#define DM270_IRQ_TIMER1   1 /* Timer 1 */
#define DM270_IRQ_TIMER2   2 /* Timer 2 */
#define DM270_IRQ_TIMER3   3 /* Timer 3 */
#define DM270_IRQ_CCDVD0   4 /* Video I/F VD0 */
#define DM270_IRQ_CCDVD1   5 /* Video I/F VD1 */
#define DM270_IRQ_CCDVD2   6 /* Video I/F VD2 */
#define DM270_IRQ_OSD      7 /* OSD */
#define DM270_IRQ_SERIAL0  8 /* Serial 0 */
#define DM270_IRQ_SERIAL1  9 /* Serial 1 */
#define DM270_IRQ_EXTHOST 10 /* External Host */
#define DM270_IRQ_DSP     11 /* DSP HINT */
#if 0				/* exchange UART0/1 for H/W change */
#define DM270_IRQ_UART0   12 /* UART 0 */
#define DM270_IRQ_UART1   13 /* UART 1 */
#else
#define DM270_IRQ_UART1   12 /* UART 0->1 */
#define DM270_IRQ_UART0   13 /* UART 1->0 */
#endif
#define DM270_IRQ_USB     14 /* USB */
#define DM270_IRQ_MTC0    15 /* MTC 0 */
#define DM270_IRQ_MTC1    16 /* MTC 1 */
#define DM270_IRQ_MMCSD0  17 /* SD/MMC I/F 0 or MS */
#define DM270_IRQ_MMCSD1  18 /* SD/MMC I/F 1 or MG */
#define DM270_IRQ_EXB0    19 /* GIO 0 */
#define DM270_IRQ_EXB1    20 /* GIO 1 */
#define DM270_IRQ_EXB2    21 /* GIO 2 */
#define DM270_IRQ_EXB3    22 /* GIO 3 */
#define DM270_IRQ_EXB4    23 /* GIO 4 */
#define DM270_IRQ_EXB5    24 /* GIO 5 */
#define DM270_IRQ_EXB6    25 /* GIO 6 */
#define DM270_IRQ_EXB7    26 /* GIO 7 */
#define DM270_IRQ_EXB8    27 /* GIO 8 */
#define DM270_IRQ_EXB9    28 /* GIO 9 */
#define DM270_IRQ_EXB10   29 /* GIO 10 */
#define DM270_IRQ_EXB11   30 /* GIO 11 */
#define DM270_IRQ_EXB12   31 /* GIO 12 */
#define DM270_IRQ_EXB13   32 /* GIO 13 */
#define DM270_IRQ_EXB14   33 /* GIO 14 */
#define DM270_IRQ_EXB15   34 /* GIO 15 */
#define DM270_IRQ_PREVIEW 35 /* Preview Engine */
#define DM270_IRQ_WDT     36 /* Watchdog Timer */
#define DM270_IRQ_I2C     37 /* I2C */
#define DM270_IRQ_CLKC    38 /* CLKC */
#define DM270_IRQ_RSVD    39 /* Reserved */
#define NR_IRQS           40

/* alias */

#define UCLINUX_IRQ_TIMER DM270_IRQ_TIMER0
#define IRQ_TIMER         DM270_IRQ_TIMER0

/* flags for request_irq() */

#define IRQ_FLG_LOCK      (0x0001)  /* handler is not replaceable */
#define IRQ_FLG_REPLACE   (0x0002)  /* replace existing handler */
#define IRQ_FLG_FAST      (0x0004)
#define IRQ_FLG_SLOW      (0x0008)
#define FIQ_FORCE         (0x4000)
#define IRQ_FLG_STD       (0x8000)  /* internally used */

#endif /* __ASM_ARCH_IRQS_H__ */
