/* asm/arch/memconfig.h */

#ifndef __ASM_ARCH_MEMCONFIG_H_
#define __ASM_ARCH_MEMCONFIG_H_

/*************************************************
 *                   FLASH
 *   0010:0000 +----------------+ (0000:0000)
 *             |     Loader     |     (  32K)
 *   0010:8000 +----------------+ (0000:8000)
 *             |  MAC Address   |     (   8K)
 *   0010:A000 +----------------+ (0000:A000)
 *             |  Config Data   |     (   8K)
 *   0010:C000 +----------------+ (0000:C000)
 *             |    Reserved    |     (  16K)
 *   0011:0000 +----------------+ (0001:0000)
 *             | Linux Kernel   |     ( 896K)
 *             |                |
 *   001F:0000 +----------------+ (000F:0000)
 *             | File System    |     (3136K)
 *             |                |
 *             |                |
 *   0050:0000 +----------------+ (0040:0000)
 *
 *                   SDRAM
 *   0090:0000 +----------------+ (0000:0000)
 *	       | (Video Buffer) |
 *             |     Loader     |
 *   0100:0000 +----------------+ (0070:0000)
 *             | File System    |
 *   0131:0000 +----------------+ (00A1:0000)
 *             | Linux Kernel   |
 *             |                |
 *             |                |
 *   0290:0000 +----------------+ (0200:0000)
 *************************************************/



/* Specify Platform */

#define BSPCONF_DM270_CUSTOM 1
#define BSPCONF_DM270_CUSTOM_MP4900_1 1
#define BSPCONF_DM270_INGENIENT 1
/* FLASH Resources */
#define BSPCONF_FLASH_BASE 0x00100000
#define BSPCONF_FLASH_SIZE 0x00400000
#define FLASH_MEM_BASE BSPCONF_FLASH_BASE
#define FLASH_SIZE BSPCONF_FLASH_SIZE

/* FLASH Memory Map */
#define WMV_MAC_FLASH_OFFSET 0x00008000
#define WMV_MAC_FLASH_SIZE 0x00002000
#define WMV_CONFIG_FLASH_OFFSET 0x0000A000
#define WMV_CONFIG_FLASH_SIZE 0x00002000
#define WMV_KERNEL_FLASH_OFFSET 0x00010000
#define WMV_FS_FLASH_OFFSET 0x000F0000

/* Compression settings */
#define BSPCONF_KERNEL_COMPRESSED 0
#define BSPCONF_FS_COMPRESSED 0

#if 0	/* Reserve 4MB for OSD */
#define BSPCONF_SDRAM_BASE 0x00900000
#define BSPCONF_SDRAM_SIZE 0x02000000
#else
#define BSPCONF_SDRAM_BASE 0x01310000
#define BSPCONF_SDRAM_SIZE 0x015F0000
#endif

/* SDRAM Memory Map */
//#define BSPCONF_BTLDR_SDRAM_START 0x00900000
//#define BSPCONF_BTLDR_SDRAM_END 0x00920000
//#define BSPCONF_FS_SDRAM_START 0x01000000
//#define BSPCONF_KERNEL_SDRAM_START 0x01310000

/* IRAM Memory Map */
#undef BSPCONF_USE_IRAM
#define BSPCONF_IRAM_BASE 0x00000004

/* Linux Kernel */

#undef BSPCONF_BOOT_FROM_FLASH
#define BSPCONF_BOOT_FROM_SDRAM 1
#define BSPCONF_KERNEL_IMAGE_START 0x01310000
#define BSPCONF_KERNEL_STACK 0x00007800

/* Linux Root File System */

#define BSPCONF_FLASH_FS 0
// uncomment this if we run fs in flash
#define BSPCONF_FILESYS_START 0x01000000
#define BSPCONF_FILESYS_SIZE  0x00310000

/* Linux Memory Pool */

#define BSPCONF_DRAM_BASE 0x01310000	/* DRAM_BASE + DRAM_SIZE (=SDRAM_END) */
#define BSPCONF_DRAM_SIZE 0x015F0000	/* = SDRAM_BASE + SDRAM_SIZE */

#endif /* __ASM_ARCH_MEMCONFIG_H_ */
