/*
 * linux/include/asm-armnommu/arch-dm270/memory.h
 *
 *   Copyright (C) 2003 Cadenux, LLC (http://www.cadenux.com)
 *   author:  Gregory Nutt <greg.nutt@cadenux.com>
 *
 * Derived from linux/include/asm-armnommu/arch-dsc25/memory.h
 *
 *   Copyright (C) 2002 Cadenux, LLC (http://www.cadenux.com)
 *   author:  Gregory Nutt <greg.nutt@cadenux.com>
 *
 * Adapted from linux/include/asm-armnommu/arch-dsc21/memory.h
 */

#ifndef __ASM_ARCH_MEMORY_H
#define __ASM_ARCH_MEMORY_H

/* Configured DRAM memory is specified during configuration and is found
 * in asm/arch/memconfig.h as BSPCONF_DRAM_BASE and BSPCONF_DRAM_SIZE.
 */

#include "linux/autoconf.h"
#include "asm/arch/memconfig.h"
#define END_MEM		(BSPCONF_DRAM_BASE + BSPCONF_DRAM_SIZE)

/* These are used in the fault handling code (even for nommu) and various other
 * macros. The current value comes from Steve Johnson's 2.0.38 uClinux port for
 * the dsc21.
 */

#define TASK_SIZE	(0x01a00000UL)
#define TASK_SIZE_26	TASK_SIZE

/* The alignment requirement of the memory management system
 * this is ~(1 << (PAGE_OFFSET+(MAX_ORDER-1)))-1.
 */

#define PHYS_OFFSET_MASK 0x001fffff

/* The start of physical memory.  This is the first SDRAM address.
 * This value is used in the arch-specific bootup code like setup_arch &
 * bootmem_init.
 *
 * NOTE the alignment requirement of the memory management system
 * this is given by PHYS_OFFSET_MASK.  There are two approaches for
 * meeting this alignment requirement.  (1) place PHYS_OFFSET below
 * the kernel/start of SDRAM.  This could be made to work becuase
 * the region between the start of usable RAM (BSPCONF_DRAM_BASE) and
 * PHYS_OFFSET will never be released to the memory manager.
 * This would be:
 * 
 * #define PHYS_OFFSET (BSPCONF_DRAM_BASE & ~PHYS_OFFSET_MASK)
 *
 * However, the memory management system will attempt to locate the
 * mem_map tables at PAGE_OFFSET.  So these second, less conservative
 * solutions are probably safer:
 */

#if defined(BSPCONF_BOOT_FROM_SDRAM)

  /* In this case, BSPCONF_DRAM_BASE will be include memory for
   * the kernel .text, .rodata, and .data.  Since the mem_map tables
   * will begin at PAGE_OFFSET, we will have to start the memory pool
   * after the kernel.  The value _end_kernel is defined in
   * arch/armnommu/vmlinux-armv.lds.in lie in SDRAM after the memory
   * used by the kernel.  NOTE:  The following alignment could cause
   * a substantial loss of memory due to the alignment requirements!
   */

  extern unsigned long _end_kernel;
# define PHYS_OFFSET \
  ((((unsigned long)&_end_kernel) + PHYS_OFFSET_MASK) & ~PHYS_OFFSET_MASK)
# define UNALIGNED_PHYS_OFFSET ((unsigned long)&_end_kernel)
#else

  /* Otherwise, the kernel .text and .rodata are in rom/flash and only
   * .data lies in SDRAM.  So the mem_map tables can begin near the
   * beginning of physical SDRAM.  The value _end_kernel still provides
   * SDRAM address after the memory used by the kernel .data section.
   * There can still be a significant loss of memory due to the alignment
   * requirement.
   */

  extern unsigned long _end_kernel;
# define PHYS_OFFSET \
  ((((unsigned long)&_end_kernel) + PHYS_OFFSET_MASK) & ~PHYS_OFFSET_MASK)
# define UNALIGNED_PHYS_OFFSET ((unsigned long)&_end_kernel)

#endif

/* PAGE_OFFSET -- the first address of the first page of memory. For archs with
 * no MMU this corresponds to the first free page in physical memory (aligned
 * on a page boundary).
 */

#define PAGE_OFFSET PHYS_OFFSET

#endif
