/**************************************************************************
 * linux/include/asm-armnommu/arch-dm270/pe_ioctl.h
 *
 * Preview Engine Control Driver Header File
 *    
 * Derived from TI's DM270 prev270.h
 *       
 * This header is to be used by application programs needing to control the
 * Preview Engine hardware registers on the DM270.
 *            
 *   Copyright (C) 2003 Cadenux, LLC. All rights reserved.
 *   Todd Fischer <todd.fischer@cadenux.com>
 *
 **************************************************************************/

#ifndef __ASM_ARCH_PE_IOCTL_H
#define __ASM_ARCH_PE_IOCTL_H
 
/**************************************************************************
 * Included Files
 **************************************************************************/

#include <asm/ioctl.h>

/**************************************************************************
 * Public Definitions
 **************************************************************************/

/**************************************************************************
 * Public Types
 **************************************************************************/
 
struct PEConfigData
{
        unsigned short ccdcInputWidth8bits;          // Width of data from CCDC
                                                     //    zero: 10 bits        
                                                     //    non-zero: 8 bits
        unsigned short alawEnable;                   // Inverse A-law mode
                                                     //    zero: disable
                                                     //    non-zero: enable
        unsigned short oneShotEnable;                // One shot mode
                                                     //    zero: disable
                                                     //    non-zero: enable
        unsigned short resizeOnlyEnable;             // Resize only mode
                                                     //    zero: disable
                                                     //    non-zero: enable
        unsigned short sdramInputEnable;             // SDRAM input mode
                                                     //    zero: disable
                                                     //    non-zero: enable
        unsigned short burstAlignModeEnable;         // Burst align mode
                                                     //    zero: disable
                                                     //    non-zero: enable
        unsigned short writeEnableValid;             // CCDC WEN signal valid
                                                     //    zero: invalid
                                                     //    non-zero: valid
        void          *readAddress;                  // Read address, absolute SDRAM address, must be 32 byte aligned
        void          *writeAddress;                 // Write address, absolute SDRAM address, must be 32 byte aligned
        unsigned int   hstart;                       // If sdramInputEnable == zero
                                                     //    horizontal start position of taking data to SDRAM
                                                     //    unit: number of pixels from CCDC HD
                                                     // If sdramInputEnable == non-zero
                                                     //    horizontal size of data in SDRAM
                                                     //    unit: pixels
        unsigned int   vstart;                       // Vertical start position of capturing data to SDRAM
                                                     //    unit: number of lines from CCDC VD
        unsigned int   inputPixelsH;                 // Input data width, unit: pixels
        unsigned int   inputLinesV;                  // Input data height, unit: lines
        unsigned int   outputPixelsH;                // Output data width, unit: pixels
        unsigned int   outputLinesV;                 // Output data height, unit: lines
};

struct PEModeData
{
        unsigned char  hinterpolationMode;          // Horizontal interpolation mode: use PE_INTERPOLATION...
        unsigned char  vinterpolationMode;          // Vertical interpolation mode: use PE_INTERPOLATION...
        unsigned short horizontalNoiseFilterEnable;  // Horizontal noise filter
                                                     //    zero: disable
                                                     //    non-zero: enable
        unsigned char  horizontalNoiseFilterLevel;   // Horizontal noise filter level: use PE_NOISE_FILTER_...
        unsigned short verticalNoiseFilterEnable;    // Vertical noise filter
                                                     //    zero: disable
                                                     //    non-zero: enable
        unsigned short smoothEnable;                 // Smoothing
                                                     //    zero: disable
                                                     //    non-zero: enable
        unsigned char  smoothLevel;                  // Smoothing level
                                                     //    0: Less
                                                     //    1: More
        unsigned char  ccdcMode;                     // CCDC Mode
                                                     //    0: RGB
                                                     //    1: Complementary color
        unsigned char  greenBlendMode;               // Green blend mode (valid when ccdcMode = 1)
                                                     // In complementary CCD mode, adjust blending ratio between
                                                     // original green and a calculated green from the other colors.
                                                     // Goutput = rGorg + (1-r)(Ye+Cy-Mg)
                                                     //    0: r=1 
                                                     //    1: r=3/4 
                                                     //    2: r=1/2 
                                                     //    3: r=1/4 
                                                     //    4: r=0 
        unsigned char  colorPatternEvenEven;         // Color Pattern, Even Pixel, Even Line: use PE_COLOR_PATTERN_...

        unsigned char  colorPatternOddEven;          // Color Pattern, Odd  Pixel, Even Line: use PE_COLOR_PATTERN_...
        unsigned char  colorPatternEvenOdd;          // Color Pattern, Even Pixel, Odd  Line: use PE_COLOR_PATTERN_...
        unsigned char  colorPatternOddOdd;           // Color Pattern, Odd  Pixel, Odd  Line: use PE_COLOR_PATTERN_...
};

struct PEWbData
{
        unsigned int   dgain;                        // Digital gain.              Format U10Q8
        unsigned int   gainGr;                       // White balance gain for Gr. Format U8Q6
        unsigned int   gainR;                        // White balance gain for R.  Format U8Q6
        unsigned int   gainGb;                       // White balance gain for Gb. Format U8Q6
        unsigned int   gainB;                        // White balance gain for B.  Format U8Q6
};

struct PEBlkData
{
        char           offsetR;                      // Black compensation R offset, Format 2's complement
        char           offsetG;                      // Black compensation G offset, Format 2's complement
        char           offsetB;                      // Black compensation B offset, Format 2's complement
};

struct PERgb2RgbData
{
        unsigned int   matrix[3][3];                 // Format S8Q6
        unsigned int   offset[3];                    // Format 2's complement integer
};

struct PERgb2YuvData
{
        unsigned int   matrix[3][3];                 // Format S8Q6        
        unsigned int   offset[3];                    // See register YOFST, COFST        
};

struct PEGammaData
{
        unsigned char  tableSelect;                  // Table select
                                                     //    0: table 0
                                                     //    1: table 1
        unsigned short enableR;                      // Use selected table for R component
                                                     //    non-zero: use gamma
                                                     //    zero: do not use, bypass gamma
        unsigned short enableG;                      // Use selected table for G component
                                                     //    non-zero: use gamma
                                                     //    zero: do not use, bypass gamma
        unsigned short enableB;                      // Use selected table for B component
                                                     //    non-zero: use gamma
                                                     //    zero: do not use, bypass gamma
};

struct PEContrast
{
        unsigned char  gainY;                        // Contrast Adjustment. Y signal gain, Format U8Q4
        unsigned char  offsetY;                      // Brightness Adjustment. Y signal offset, 0..255
};

struct PEColorSuppression
{
        unsigned short enable;                       // Chroma suppression
                                                     //    non-zero: enable
                                                     //    zero: disable
        unsigned char  threshold;                    // Chroma suppression threshold, 0..255
        unsigned char  gain;                         // Chroma suppression gain, Format U8Q8
};

struct PEYCbCrLevel
{
        unsigned char  minY;                         // Minimum Y level, 0..255
        unsigned char  maxY;                         // Minimum Y level, 0..255
        unsigned char  minC;                         // Minimum Cb, Cr level, 0..255
        unsigned char  maxC;                         // Maximum Cb, Cr level, 0..255
};

/**************************************************************************
 * Definitions
 **************************************************************************/
 
/* List of ioctl commands. */
 
#define PE_MAGIC_NUMBER 'P'

#define PE_ENABLE                 _IOW( PE_MAGIC_NUMBER, 0x00, sizeof(int))
#define PE_CONFIGURE_CONTROLLER   _IOW( PE_MAGIC_NUMBER, 0x01, sizeof(struct PEConfigData))
#define PE_SET_READ_ADDR          _IOW( PE_MAGIC_NUMBER, 0x02, sizeof(unsigned long))
#define PE_SET_WRITE_ADDR         _IOW( PE_MAGIC_NUMBER, 0x03, sizeof(unsigned long))
#define PE_SET_MODE               _IOW( PE_MAGIC_NUMBER, 0x04, sizeof(struct PEModeData))
#define PE_SET_WHITE_BALANCE      _IOW( PE_MAGIC_NUMBER, 0x05, sizeof(struct PEWbData))
#define PE_SET_BLACK_COMPENSATION _IOW( PE_MAGIC_NUMBER, 0x06, sizeof(struct PEBlkData))
#define PE_SET_RGB_TO_RGB_MATRIX  _IOW( PE_MAGIC_NUMBER, 0x07, sizeof(struct PERgb2RgbData))
#define PE_SET_RGB_TO_YUV_MATRIX  _IOW( PE_MAGIC_NUMBER, 0x08, sizeof(struct PERgb2YuvData))
#define PE_SET_GAMMA              _IOW( PE_MAGIC_NUMBER, 0x09, sizeof(struct PEGammaData))
#define PE_SET_CONTRAST           _IOW( PE_MAGIC_NUMBER, 0x0A, sizeof(struct PEContrast))
#define PE_SET_COLOR_SUPPRESSION  _IOW( PE_MAGIC_NUMBER, 0x0B, sizeof(struct PEColorSuppression))
#define PE_SET_YCBCR_LEVEL        _IOW( PE_MAGIC_NUMBER, 0x0C, sizeof(struct PEYCbCrLevel))
#define PE_WAIT                   _IOW( PE_MAGIC_NUMBER, 0x0D, 0)

#define PE_IOC_MAXNR 0x0D

/* PE Color Patterns - colorPattern...  in struct PEModeData */

#define PE_COLOR_PATTERN_GB       0
#define PE_COLOR_PATTERN_B        1
#define PE_COLOR_PATTERN_R        2
#define PE_COLOR_PATTERN_GR       3

/* PE Interpolation tap filters - ...interpolationMode in struct PEModeData */

#define PE_INTERPOLATION_5_TAP_FILTER 0
#define PE_INTERPOLATION_2_TAP_FILTER 1

/* PE Noise filters - horizontalNoiseFilterLevel in struct PEModeData */

#define PE_NOISE_FILTER_VERY_WEAK      0
#define PE_NOISE_FILTER_WEAK           1
#define PE_NOISE_FILTER_STRONG         2
#define PE_NOISE_FILTER_VERY_STRONG    3

/* Major numbers, (from Documentation/devices.txt local/experimental list)  */
 
#define PE_CONTROL_MAJOR     121

#endif /* __ASM_ARCH_PE_IOCTL_H */
