/* linux/include/asm-armnommu/arch-dm270/spi_ioctl.h
 * This file represents the SPI interface. 
 *
 *   Copyright (C) 2003 Cadenux, LLC (http://www.cadenux.com)
 *   Author:  Gregory Nutt <greg.nutt@cadenux.com>
 *
 * Modified from linux/include/asm-armnommu/arch-dsc21/spi_ioctl.h:
 *
 *   Copyright (C) 2003 Cadenux, LLC (http://www.cadenux.com)
 *   Author:  Gregory Nutt <greg.nutt@cadenux.com>
 *
 * Modified from linux/include/asm-armnommu/arch-dsplinux/spi_ioctl.h:
 *
 *   Copyright (C) 2001 RidgeRun, Inc. (http://www.ridgerun.com)
 *
 */

#ifndef __ASM_ARCH_SPI_IOCTL_H
#define __ASM_ARCH_SPI_IOCTL_H

/* Included Files */
 
#include <asm/ioctl.h>
 
/* SPI Definitions */
 
#define SPI_LEVEL_TRIGGER  0
#define SPI_EDGE_TRIGGER   1

#define SPI_NEG_ENABLE     0
#define SPI_POS_ENABLE     1

#define SPI_FALLING_EDGE   0
#define SPI_RISING_EDGE    1

#define SPI_MIN_CLOCK      0     /* SPI Clock settings must be powers of 2 */
#define SPI_CLOCK_DIV1     0x0
#define SPI_CLOCK_DIV2     0x1
#define SPI_CLOCK_DIV4     0x2
#define SPI_CLOCK_DIV8     0x3
#define SPI_CLOCK_DIV16    0x4
#define SPI_CLOCK_DIV32    0x5
#define SPI_CLOCK_DIV64    0x6
#define SPI_MAX_CLOCK      6

#define SPI_MIN_WORDSIZE   1
#define SPI_MAX_WORDSIZE   32

#define SPI_SET_MASTER 0
#define SPI_SET_SLAVE 1

#define SPI_DATA_LSB 0
#define SPI_DATA_MSB 1

#define SPI_MAX_BUFSIZE    2048

/* ioctl Command Definitions */

#define SPI_MAGIC_NUMBER   's'

#define SPI_TRIGGER_FORMAT _IO(SPI_MAGIC_NUMBER, 0)
#define SPI_ENABLE_LEVEL   _IO(SPI_MAGIC_NUMBER, 1)
#define SPI_CLOCK_EDGE     _IO(SPI_MAGIC_NUMBER, 2)
#define SPI_CLOCK_PRESCALE _IO(SPI_MAGIC_NUMBER, 3)
#define SPI_WORDSIZE       _IO(SPI_MAGIC_NUMBER, 4)
#define SPI_BIT_RATE       _IO(SPI_MAGIC_NUMBER, 5)
#define SPI_MASTER_SLAVE   _IO(SPI_MAGIC_NUMBER, 6)
#define SPI_DATA_ORDER     _IO(SPI_MAGIC_NUMBER, 7)

#define SPI_IOC_MAXNR      7
 
/* Major numbers, (from Documentation/devices.txt local/experimental list)  */
 
#define SPI_MAJOR          118

#endif /* __ASM_ARCH_SPI_IOCTL_H */
