/* linux/include/asm-arm/arch-dm270/time.h
 *
 *   Copyright (C) 2003 Cadenux, LLC (http://www.cadenux.com)
 *   author:  Gregory Nutt <greg.nutt@cadenux.com>
 *
 * Derived from linux/include/asm-arm/arch-dsc25/time.h
 * Derived from linux/include/asm-arm/arch-dsc21/time.h
 * Derived from linux/include/asm-arm/arch-sa1100/time.h, via
 *
 *   Copyright (C) 1998 Deborah Wallach.
 *   Twiddles  (C) 1999 	Hugo Fiennes <hugo@empeg.com>
 * 
 * 2000/03/29 (C) Nicolas Pitre <nico@cam.org>
 *   Rewritten: big cleanup, much simpler, better HZ acuracy.
 *
 * 2001/02/20 (C) RidgeRun, Inc (http://www.ridgerun.com)
 *   Leveraged for the dsc21
 *
 * 2002/10/17 Leveraged for the dsc25
 * 2002/01/19 Leveraged for the dm270
 */

#ifndef __ASM_ARCH_TIME_H__
#define __ASM_ARCH_TIME_H__

#include <asm/uaccess.h>
#include <asm/io.h>
#include <asm/hardware.h>
#include <asm/arch/timex.h>

extern struct irqaction timer_irq;

extern unsigned long dm270_gettimeoffset(void);
extern void dm270_timer_interrupt(int irq, void *dev_id, struct pt_regs *regs);

static void __inline__ setup_timer (void)
{
  /* Disable */

  outw(0, DM270_TIMER0_MODE);
  outw(0, DM270_TIMER1_MODE);
  outw(0, DM270_TIMER2_MODE);
  outw(0, DM270_TIMER3_MODE);

  /* System clock formula:
   *         freq = clock / (div * scale)
   * Where:
   *         clock = 27MHz
   *         div   = 27,000
   *         scale = 10
   *
   * Which gives us 100Hz, as desired.
   */

  /* Set timer prescale value */

  outw((CLOCK_PRESCALE-1), DM270_TIMER0_SCAL);
	
  /* Set timer divisor */

  outw((CLOCK_DIVISOR-1), DM270_TIMER0_DIV);
	
  /* Set mode periodic */

  outw(2, DM270_TIMER0_MODE);

  gettimeoffset     = dm270_gettimeoffset;	
  timer_irq.handler = dm270_timer_interrupt;

  setup_arm_irq(IRQ_TIMER, &timer_irq);
}

#endif /* __ASM_ARCH_TIME_H__ */
