/**************************************************************************
 * linux/asm-armnommu/arch-dm270/timer_ioctl.h
 *
 * This file represents the DM270 timer peripheral.
 *
 * Copyright (C) 2003 Cadenux, LLC (http://www.cadenux.com)
 * author:  Gregory Nutt <greg.nutt@cadenux.com>
 *
 * Derived from linux/asm-armnommu/arch-dsplinux/timer_ioctl.h
 *
 * Copyright (C) 2001, RidgeRun, Inc.
 *
 **************************************************************************/

#ifndef __ASM_ARCH_TIMER_IOCTL_H
#define __ASM_ARCH_TIMER_IOCTL_H

/**************************************************************************
 * Included Files
 **************************************************************************/

#include <asm/ioctl.h>
 
/**************************************************************************
 * Public Types
 **************************************************************************/

typedef enum
{
  eStop,
  eOneShot,
  eFreeRun
} timer_mode_t;

typedef enum
{
  eARMClock,
  e27MHzClock
} timer_clock_select_t;

typedef struct
{
  timer_mode_t mode;
  timer_clock_select_t select;
  __u16 divisor;
  __u16 prescaler;  /* Max value of 0x3ff(10 bits) */
} timer_setup_t;

/**************************************************************************
 * Definitions
 **************************************************************************/

/* List of ioctl commands. */
 
#define TIMER_MAGIC_NUMBER 'T'
 
/* Blocking call to wait for timer interrupt to fire. */

#define TIMER_WAIT _IO(TIMER_MAGIC_NUMBER, 0)

/* Start one shot timer. */

#define TIMER_TRIGGER_ONE_SHOT _IO(TIMER_MAGIC_NUMBER, 1)

/* Return current value of timer. */

#define TIMER_GET_VALUE _IOR(TIMER_MAGIC_NUMBER, 2, short int)

/* If you want total control of your timer, use this IOCTL to set it.
 *
 *  timer set takes this parameter (timer_setup_t):
 *    mode (timer_mode_t): Stop, OneShot (run once), or
 *                         FreeRun (run continually)
 *    timer clock selection (timer_clock_select_t): ARM clock or 27MHz clock.
 *    prescaler division ratio (u16).
 *    timer division ratio (u16).
 */

#define TIMER_SET_ALL _IOW(TIMER_MAGIC_NUMBER, 3, timer_setup_t)

/* If you want a simple interface, send in the number of microseconds
 *  you want to occur before the timer interrupt fires.
 */

#define TIMER_SET_USEC _IOW(TIMER_MAGIC_NUMBER, 4, __u32)

#define TIMER_IOC_MAXNR 4
 
/* Major numbers, (from Documentation/devices.txt local/experimental list)  */
 
#define TIMER_MAJOR     124
 
#endif /* __ASM_ARCH_TIMER_IOCTL_H */
