/* linux/include/asm-armnommu/arch-dm270/timex.h
 *         System clock frequency.
 *
 *   Copyright (C) 2003 Cadenux, LLC (http://www.cadenux.com)
 *   author:  Gregory Nutt <greg.nutt@cadenux.com>
 *
 * Derived from linux/include/asm-armnommu/arch-dsc25/timex.h
 *
 *   Copyright (C) 2002 Cadenux, LLC (http://www.cadenux.com)
 *   author:  Gregory Nutt <greg.nutt@cadenux.com>
 *
 * Derived from linux/include/asm-armnommu/arch-dsc21/timex.h
 *   copyright:
 *         (C) 2001 RidgeRun, Inc. (http://www.ridgerun.com)
 *   author: Gordon McNutt <gmcnutt@ridgerun.com>
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __ASM_ARCH_TIMEX_H__
#define __ASM_ARCH_TIMEX_H__

/* The system clock is driven off timer 0.  The timer 0 interrupt frequency
 * is controlled by three values: CLOCK_SOURCE_FREQUENCY, PRESCALE, and DIV
 * according to:
 *
 * TIMER_INTERRUPT_FREQUENCY =
 *   CLOCK_SOURCE_FREQUENCY / (PRESCALE + 1) / (DIV + 1)
 */

/* The source frequency is the 27MHz MXI clock */

#define CLOCK_SOURCE_FREQUENCY 27000000

/* (PRESCALE+1) == 10 yield LSB=10/27000000=370ns */

#define CLOCK_PRESCALE         10

/* CLOCK_TICK_RATE -- frequency fed to the timer. Used to compute LATCH.
 * LATCH is the number of clock ticks per timer interrupt. (See linux/timex.h,
 * armnommu/mach-dm270/time.c).  We don't actually use LATCH, so this is
 * just to cover all bases.
 */

#define CLOCK_TICK_RATE	       (CLOCK_SOURCE_FREQUENCY / CLOCK_PRESCALE)

/* And, finally... The following assumes that HZ is set 100 (asm/params.h).
 * This yields the final divisor+1 of:
 */

#define CLOCK_DIVISOR          (CLOCK_TICK_RATE / 100)

#endif /* __ASM_ARCH_TIMEX_H__ */
