#ifndef _SETUP_DM270_GIO_
#define _SETUP_DM270_GIO_

typedef enum {
	bit33_32,
	bit31_16,
	bit15_0
} GIORegSelect;

/*--------------------------------------------------------------------------*/
/* GIO Functions:                                                           */
/*--------------------------------------------------------------------------*/
#define GIO_BIT_0	0	/* No use:       INT0 		*/
#define GIO_BIT_1	1	/* No use:       INT1 		*/
#define GIO_BIT_2	2	/* Input:        PB_RST# 	*/
#define GIO_BIT_3	3	/* Input:        PE_USE  	*/
#define GIO_BIT_4	4	/* Bi-direction: PHY_INT 	*/
#define GIO_BIT_5	5	/* Bi-direction: PCMCIA_INT     */
#define GIO_BIT_6	6	/* Bi-direction: RTC_INT 	*/
#define GIO_BIT_7	7	/* Input:        Reserved 	*/
#define GIO_BIT_8	8	/* Output: 	 CCD_SCL 	*/
#define GIO_BIT_9	9	/* Bi-direction: CCD_SDA	*/
#define GIO_BIT_10	10	/* Output:	 CCD_RST 	*/
#define GIO_BIT_11	11	/* Input: 	 Reserved	*/
#define GIO_BIT_12	12	/* Output:	 LCD_CS# 	*/
#define GIO_BIT_13	13	/* Output:       LCD_WR#	*/
#define GIO_BIT_14	14	/* Output:	 LCD_DATA 	*/
#define GIO_BIT_15	15	/* Output:	 LCD_BL 	*/
#define GIO_BIT_16	0	/* Output: 	 RTC_SCL	*/
#define GIO_BIT_17	1	/* Bi-direction: RTC_SDA 	*/
#define GIO_BIT_18	2	/* Input:    	 Reserved	*/
#define GIO_BIT_19	3	/* Output:	 AUD_SDIN 	*/
#define GIO_BIT_20	4	/* Output:	 AUD_SC# 	*/
#define GIO_BIT_21	5	/* Output:	 AUD_SCLK 	*/
#define GIO_BIT_22	6	/* Input:        Reserved 	*/
#define GIO_BIT_23	7	/* Input:	 CPU_RXD 	*/
#define GIO_BIT_24	8	/* Output:	 CPU_TXD 	*/
#define GIO_BIT_25	9	/* Output:       CPU_RST# 	*/
#define GIO_BIT_26	10	/* Output:       PCMCIA_RST# 	*/
#define GIO_BIT_27	11	/* Output:       PHY_RST# 	*/
#define GIO_BIT_28	12	/* Output: 	 LED_PWRG 	*/
#define GIO_BIT_29	13	/* Output:       LED_ACTG 	*/
#define GIO_BIT_30	14	/* Output:       LED_WGG 	*/
//#define GIO_BIT_31	15	/* Bi-direction: VDO_SCL 	*/
#define GIO_BIT_32	0	/* Input:        Reserved 	*/
#define GIO_BIT_33	1	/* Input:        Reserved 	*/

/*--------------------------------------------------------------------------*/
/* GIOGetDir                                                                */
/* This function is used to get the value of bit47_32 (port 0), bit31_16    */
/* (port 1) or bit15_0 (port 2) port direction.                             */
/* Input: port ( bit47_32, bit31_16 or bit15_0 )	 		    */
/* Return: value of port direction					    */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetDir(GIORegSelect word);

/*--------------------------------------------------------------------------*/
/* GIOSetDir                                                                */
/* This function is used to set the value of port direction.                */
/* Input: port and value of port direction				    */
/*--------------------------------------------------------------------------*/
void GIOSetDir(GIORegSelect word, unsigned short value);

/*--------------------------------------------------------------------------*/
/* GIOGetInv                                                                */
/* This function is used to get the value of the invert port bits.          */
/* Input: port ( bit47_32, bit31_16 or bit15_0 )	 		    */
/* Return: value of the invert port bits				    */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetInv(GIORegSelect word);

/*--------------------------------------------------------------------------*/
/* GIOSetInv                                                                */
/* This function is used to set the value of the invert port bits           */
/* Input: port and value of the invert port bits			    */
/*--------------------------------------------------------------------------*/
void GIOSetInv(GIORegSelect word, unsigned short value);

/*--------------------------------------------------------------------------*/
/* GIOGetPortBitSet                                                         */
/* This function is used to get the value of the port bit register set.     */
/* Input: port ( bit47_32, bit31_16 or bit15_0 )	 		    */
/* Return: value of port direction					    */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetPortBitSet(GIORegSelect word);

/*--------------------------------------------------------------------------*/
/* GIOSetPortBitSet                                                         */
/* This function is used to set the value of the port bit register set.     */
/* Input: port and value of port direction				    */
/*--------------------------------------------------------------------------*/
void GIOSetPortBitSet(GIORegSelect word, unsigned short value);

/*--------------------------------------------------------------------------*/
/* GIOGetPortBitClear                                                       */
/* This function is used to get the value of the port bit clear register    */
/* set.				                                            */
/* Input: port ( bit47_32, bit31_16 or bit15_0 )	 		    */
/* Return: value of port direction					    */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetPortBitClear(GIORegSelect word);

/*--------------------------------------------------------------------------*/
/* GIOSetPortBitClear                                                       */
/* This function is used to set the value of the port bit clear register    */
/* set.                                                                     */
/* Input: port and value of port direction				    */
/*--------------------------------------------------------------------------*/
void GIOSetPortBitClear(GIORegSelect word, unsigned short value);

/*--------------------------------------------------------------------------*/
/* GIOGetPortExternalIRQ (GIO0-GIO15)                                       */
/* This function returns the port IRQ register.                             */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetPortExternalIRQ(void);

/*--------------------------------------------------------------------------*/
/* GIOSetPortExternalIRQ                                                    */
/* This function is used to set the GIO 0-15 IRQ.                           */
/* Input: value of port irq 	 					    */
/*--------------------------------------------------------------------------*/
void GIOSetPortExternalIRQ(unsigned short irqval);

/*--------------------------------------------------------------------------*/
/* GIOGetEdgeIRQ                                                    	    */
/* This function returns GIO IRQ Edge Setting register.                     */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetEdgeIRQ(void);

/*--------------------------------------------------------------------------*/
/* GIOSetEdgeIRQ                                                            */
/* This function is used to set the GIO 0-15 Interrupt Edge                 */
/* Input: value of port irq edge					    */
/* Note: 0 -> single-sided						    */
/*		GIOxxINV = 0 -> falling edge        			    */	/*		GIOxxINV = 1 -> rising edge                                 */
/* 	 1 -> double-sided						    */
/*--------------------------------------------------------------------------*/
void GIOSetEdgeIRQ(unsigned short edgeval);

/*--------------------------------------------------------------------------*/
/* GIOGetChatter                                                    	    */
/* This function returns GIO Chatter Setting register.                      */
/* Input: port ( bit47_32, bit31_16 or bit15_0 )	 		    */
/* Return: value of port direction					    */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetChatter(GIORegSelect word);

/*--------------------------------------------------------------------------*/
/* GIOSetChatter                                                    	    */
/* This function is used to set GIO Chatter Setting register. When enabled, */
/* input is set only after input signal is stable for longer then NCHAT     */
/* Input: port and value of chatter 					    */
/*--------------------------------------------------------------------------*/
void GIOSetChatter(GIORegSelect word, unsigned short value);

/*--------------------------------------------------------------------------*/
/* GIOGetNCHAT                                                    	    */
/* This function returns GIO Chatter Value register.                        */
/* Return: value of internal chatter 					    */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetNCHAT(void);

/*--------------------------------------------------------------------------*/
/* GIOSetNCHAT                                                    	    */
/* This function is used to set GIO Chatter Value register. 		    */
/* Input: value of internal chatter 					    */
/*--------------------------------------------------------------------------*/
void GIOSetNCHAT(unsigned short chatval);

/*--------------------------------------------------------------------------*/
/* GIOGetFunctionSelect0                                               	    */
/* This function returns GIO8 - GIO23 Function Select register.             */
/* Return: value of function select 					    */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetFunctionSelect0(void);

/*--------------------------------------------------------------------------*/
/* GIOSetFunctionSelect0                                              	    */
/* This function is used to set GIO8 - GIO23 Function Select register. 	    */
/* Input: value of function select					    */
/*--------------------------------------------------------------------------*/
void GIOSetFunctionSelect0(unsigned short fsel0val);

/*--------------------------------------------------------------------------*/
/* GIOGetFunctionSelect1                                               	    */
/* This function returns GIO24 - GIO33 Function Select register.            */
/* Return: value of function select 					    */
/*--------------------------------------------------------------------------*/
unsigned short GIOGetFunctionSelect1(void);

/*--------------------------------------------------------------------------*/
/* GIOSetFunctionSelect1                                              	    */
/* This function is used to set GIO24 - GIO33 Function Select register.	    */
/* Input: value of function select					    */
/*--------------------------------------------------------------------------*/
void GIOSetFunctionSelect1(unsigned short fsel1val);

#if 0
/*--------------------------------------------------------------------------*/
/* INT Functions:                                                           */
/*--------------------------------------------------------------------------*/
#define INT_BIT_0	0	 /* External Interrupt 0 */
#define INT_BIT_1	1	 /* External Interrupt 1 */
#define INT_BIT_2	2	 /* External Interrupt 2 */
#define INT_BIT_3	3	 /* External Interrupt 3 */
#define INT_BIT_4	4	 /* External Interrupt 4 */
#define INT_BIT_5	5	 /* External Interrupt 5 */
#define INT_BIT_6	6	 /* External Interrupt 6 */
#define INT_BIT_7	7	 /* External Interrupt 7 */
#define INT_BIT_8	8	 /* External Interrupt 8 */
#define INT_BIT_9	9	 /* External Interrupt 9 */
#define INT_BIT_10	10	 /* External Interrupt 10 */
#define INT_BIT_11	11	 /* External Interrupt 11 */
#define INT_BIT_12	12	 /* External Interrupt 12 */
#define INT_BIT_13	13	 /* External Interrupt 13 */
#define INT_BIT_14	14	 /* External Interrupt 14 */
#define INT_BIT_15	15	 /* External Interrupt 15 */

unsigned short INTGetREG0(void);
unsigned short INTGetREG1(void);
void INTSetREG0(unsigned short value);
void INTSetREG1(unsigned short value);
unsigned short INTGetIQR0(void);
unsigned short INTGetIQR1(void);
void INTSetIQR0(unsigned short value);
void INTSetIQR1(unsigned short value);
/*--------------------------------------------------------------------------*/
/* BUS Functions:                                                           */
/*--------------------------------------------------------------------------*/
#define BUS_BIT_0	0	/* CFC mapping mode: Memory map/IO map */

unsigned short BUSGetBusMD(void);
void BUSSetBusMD(unsigned short value);
#endif


/* ######## Media Adapter H/W Specific Stuff (Start) ######## */
/*--------------------------------------------------------------------------*/
/* MACADDR Functions:                                                       */
/*--------------------------------------------------------------------------*/
int  WMV_GetMACAddr(unsigned char *addr);
void WMV_SetMACAddr(unsigned char *value);
/* ######## Media Adapter H/W Specific Stuff (End  ) ######## */

#endif /* SETUP_DM270_GIO_ */
