/* linux/kernel/drop_caches.c
 * 
 * Tries to free memory by releasing the caches and buffers.
 *
 * This syscall emulates the calling of free_more_memory, but
 * enters the try_to_free_pages subrutine to be able to set
 * the number of page frames to release.
 *
 * The sycall takes one argument: int release_pages
 * the number tells the syscall how much memory to try to free
 *
 * Each page is 4k in size, so a call to drop_caches(128) will
 * try to release 128x4k=512k
 *
 * It's up to the calling program to determine how much 
 * memory/page_frames it wants to try to release.
 *
 * A normal call using release_pages=SWAP_CLUSTER_MAX [32]
 * has been timed to take under 10ms to complete on a dm270
 * with a 94500 kHz ARM clock. 
 *
 * Copyright (C) 2003 Cadenux, LLC (http://www.cadenux.com)
 * Author:  Marco Caamano <marco.caamano@cadenux.com>
 *
 *  * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */


 
#include <linux/drop_caches.h>
 
#include <linux/config.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/init.h>
#include <linux/delay.h>

#include <linux/slab.h>
#include <linux/shm.h>
#include <linux/mman.h>
#include <linux/locks.h>
#include <linux/pagemap.h>
#include <linux/swap.h>
#include <linux/smp_lock.h>
#include <linux/blkdev.h>
#include <linux/file.h>
#include <linux/swapctl.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/iobuf.h>

extern int shrink_caches(zone_t * classzone, int priority, unsigned int gfp_mask, int nr_pages);

asmlinkage int sys_drop_caches(int release_pages){
	zone_t * zone = contig_page_data.node_zonelists[GFP_NOFS & GFP_ZONEMASK].zones[0];
        int priority = 6;       /* DEF_PRIORITY default is 6 */
        int nr_pages = 128;     /* SWAP_CLUSTER_MAX default is 32*/
        unsigned int gfp_mask;
        
        if (release_pages>0){	/* small sanity check */
        	nr_pages = release_pages;
        }
        
        gfp_mask = pf_gfp_mask(GFP_USER);
        do {
        	nr_pages = shrink_caches(zone, priority, gfp_mask, nr_pages);
        	if (nr_pages <= 0)
        		break;
        } while (--priority);

	return 0;
}


