/*
 * File: boot.c
 *
 * rrload moves itself from flash to RAM
 *
 * Copyright (C) 2002 RidgeRun, Inc.
 * Author: RidgeRun, Inc  <skranz@ridgerun.com>
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS  PROVIDED  ``AS  IS''  AND   ANY  EXPRESS  OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT,  INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Please report all bugs/problems to the author or <support@dsplinux.net>
 *
 * key: RRGPLCR (do not remove)
 *
 */

#include "io.h"
#include "memconfig.h"

#define GIO_BITSET1	0x0003058E
#define GIO_BITCLR1	0x00030594

extern void main(void);
extern int btldr_start;
extern int btldr_end;

/******************************
 Routine:
 Description:
   rrload moves itself from flash to RAM and
   then jumps to its main() now located in RAM.
   The ram location used by the transfer algorithm
   is determined by the linker script used to
   build rrload.
******************************/
void bootstrap(void)
{
  typedef void(*ENTRY)(void); // Signature of bootloader's "main()".
  ENTRY entry_addr;
  unsigned int num_bytes;
  unsigned int start;
  unsigned int end;
  int i;
  
  // NOTE: | I suspect that this block can be completely
  //       | removed and *all* supported plateforms can
  //       | get by with the faster algorithm currently
  //       | used for C5471 above. At the moment I'm porting
  //       | rrload to C5471 and don't want to run the risk
  //       | of breaking those other platforms that we know
  //       | already work. In the future when we have time
  //       | to test that the new algorithm really works on 
  //       | them we'll switch over and retire the enclosed
  //       | logic below. skranz, Sep 2001.
  unsigned short *fs_addr;
  unsigned short *rs_addr;
  unsigned int num_16bit_words;
  
  /* led test */
//  *(volatile unsigned short *)GIO_BITSET1 = 0x7000;
  *(volatile unsigned short *)GIO_BITCLR1 = 0x7000;
  for(i = 0; i < 1000000; i++);
//  *(volatile unsigned short *)GIO_BITCLR1 = 0x7000;
  *(volatile unsigned short *)GIO_BITSET1 = 0x7000;

  start = (unsigned int)(&btldr_start);
  end   = (unsigned int)(&btldr_end);
  num_bytes = (end - start) + 1;
  // Next, | Move the bootloader image from flash
  //       | to RAM.
  fs_addr = (unsigned short *) (BSPCONF_FLASH_BASE);
  rs_addr = (unsigned short *) (&btldr_start);
  num_16bit_words = (num_bytes + 1) >> 1; // round up.
  for (i=0; i<num_16bit_words; i++) {
    *rs_addr = *fs_addr; // 16bit words at a time.
    rs_addr++;
    fs_addr++;
  }
  
  // Now jump to "main()" of the now RAM based bootloader.
  entry_addr = &main;
  (*entry_addr)(); 
}
