/*
 * File: btldr_ui.c
 *
 * An implementation of a menu based UI which takes user input and forms it
 * into commands to the bootloader Programmer's Interface (pi).
 *
 * Copyright (C) 2002 RidgeRun, Inc.
 * Author: RidgeRun, Inc  <skranz@ridgerun.com>
 *  - Support for DSC25 added, 9-6-02, Gregory Nutt
 *  - Support for DM270 added, 2-19-03, Gregory Nutt
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS  PROVIDED  ``AS  IS''  AND   ANY  EXPRESS  OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT,  INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Please report all bugs/problems to the author or <support@dsplinux.net>
 *
 * key: RRGPLCR (do not remove)
 *
 */

#include "types.h"
#include "io.h"
#include "memconfig.h"
#include "btldr_pi.h"
#include "util.h"
#include "flash.h"
#include "prototype.h"
#include "lcd.h"

#define VERSION "5.6"
#define MTCODE 0

extern int ForUpGrade; // defined by linker script.

#if MTCODE
/*
 * ram_check ()
 * 
 */

void ram_check (void)
{

#define GIO_BITSET1	0x0003058E
#define GIO_BITCLR1	0x00030594
	
	unsigned long *rs_addr;
	unsigned long tmp;
	rs_addr = (unsigned long*)0x00A00000;
	
	for(;rs_addr<(unsigned long *)0x02900000;rs_addr++)
	{
		/* copy a check word into the address */
			*rs_addr = 0x55aacc33;	
	
		/* then read the data in that address */
			tmp=*rs_addr;
			
		/* compare the data to see if the data is correct */
		if( rs_addr ==(unsigned long *) 0x00A00000)
	          	util_printf("memory check at %2X OK\n",rs_addr);

		// brutal and ugly method to do the comparsion.....
		if( tmp  == 0x55aacc33  )
		{
			if((rs_addr==(unsigned long *)0x0A000000)||(rs_addr==(unsigned long*)0x0B00000||rs_addr==(unsigned long*)0x0C00000||rs_addr==(unsigned long*)0x0D00000
				||rs_addr==(unsigned long*)0x0E00000||rs_addr==(unsigned long*)0x0F00000)||rs_addr==(unsigned long*)0x1000000||rs_addr==(unsigned long*)0x1100000
				||rs_addr==(unsigned long*)0x1200000||rs_addr==(unsigned long*)0x1300000||rs_addr==(unsigned long*)0x1400000||rs_addr==(unsigned long*)0x1500000
				||rs_addr==(unsigned long*)0x1600000||rs_addr==(unsigned long*)0x1700000||rs_addr==(unsigned long*)0x1800000||rs_addr==(unsigned long*)0x1900000
				||rs_addr==(unsigned long*)0x1A00000||rs_addr==(unsigned long*)0x1B00000||rs_addr==(unsigned long*)0x1C00000||rs_addr==(unsigned long*)0x1D00000
				||rs_addr==(unsigned long*)0x1F00000||rs_addr==(unsigned long*)0x2000000||rs_addr==(unsigned long*)0x2100000||rs_addr==(unsigned long*)0x2200000
				||rs_addr==(unsigned long*)0x2300000||rs_addr==(unsigned long*)0x2400000||rs_addr==(unsigned long*)0x2500000||rs_addr==(unsigned long*)0x2600000
				||rs_addr==(unsigned long*)0x2700000||rs_addr==(unsigned long*)0x2800000)
	         		util_printf(".");
		}
		else
		{
          		util_printf("\nmemory check at %x Fail\n",rs_addr);
			//LED on
  			//*(volatile unsigned short *)GIO_BITSET1 = 0x7000;
  			*(volatile unsigned short *)GIO_BITCLEAR1 = 0x7000;
		}
	}
          		util_printf("\nmemory check done\n");
}
#endif

static int chksum_verify(void)	// verify unprotected area checksum
{
	u8 *FlashStart,*RamStart;
	u32 Length = (u32)(FLASH_SIZE-BSPCONF_KERNEL_FLASH_OFFSET)-4;
	u32 checksum=0;
	int i=0;
	unsigned long *p;


	FlashStart =(u8 *)(FLASH_BASE+BSPCONF_KERNEL_FLASH_OFFSET); 

	RamStart = (u8 *)0x01000000;		// SDRAM position	

#if 0	// Read flash directly instead of copying to SDRAM
	u_memcpy((u8*)RamStart,(u8*)FlashStart,Length);

	/* checksum the flash before we jump to it. */
	for(p=(unsigned long *)RamStart;i<Length;p++,i+=4)
		checksum+=*p;
#else
	for(p=(unsigned long *)FlashStart;i<Length;p++,i+=4)
		checksum+=*p;
#endif

	util_printf("unprotected area checksum = %X\n",checksum);
	if(checksum)
	{
	    util_printf("checksum error, enter download mode\n");
	    return -1;
	}
	
	return 0;
}

/******************************
 Routine:
 Description:
 ******************************/
int main(int argc, char *argv[])
{
  int dl = 0, val;
  u8 mac[6];

  btldr_init();
  lcd_clearall();

  unlock_flash(); // We'll keep flash unlocked the whole
                  // time the bootloader is running until
                  // it is time to transfer control to the 
                  // kernel at which time we first lock
                  // flash. Hopefully this will prevent
                  // corrupted flash that might otherwise
                  // result in a run-away program randomly
                  // jumping into the flash logic of the
                  // bootloader image still laying around
                  // in ram from its prior run.

  dmfe_init();

  dl = GpIOBitGet0(PUSH_BUTTON);
  util_printf("push button %d\n",dl);

  dmfe_GetMACAddr(mac);
//  util_printf("MAC Read from Flash:");
//  for (val=0; val<5; val++)
//     util_printf("%x:", mac[val]);
//  util_printf("%x\n", mac[val]);
  val=dmfe_validateMacAddress(mac);
  if (!val) { 
     /* Set to default */
     mac[0] = 0x00;
     mac[1] = 0xC0;
     mac[2] = 0x02;
     mac[3] = 0xFF;
     mac[4] = 0xFF;
     mac[5] = 0xFF;
     util_printf("Invalid MAC set to default: 00c002ffffff\n");
  }

  /* Set Node Address */
  dmfe_SetMACAddr(mac);

  if (! val)
     to_download(0);

  if (dl == 0 || chksum_verify() == -1)
     to_download(1);
#if MTCODE
  /* do ram test here */ 
  util_printf("Starting memory check\n");
  ram_check();
#endif

  util_printf("boot kernel\n");
  cmd_boot_kernel(TRUE,NULL,0);
  return 0;
}

