/*************************************************************
 * dm270-registers.h
 *************************************************************/

#ifndef __RRLOAD_DM270_REGISTER_H
#define __RRLOAD_DM270_REGISTER_H

#include "memconfig.h"

/* Memory mapped registers */

#define TIMER0_REGISTER_BASE    0x00030000 /* Timer0 */
#define TIMER1_REGISTER_BASE	0x00030080 /* Timer1 */
#define TIMER2_REGISTER_BASE	0x00030100 /* Timer2 */
#define TIMER3_REGISTER_BASE	0x00030180 /* Timer3 */
#define SERIAL0_REGISTER_BASE	0x00030200 /* Serial0 */
#define SERIAL1_REGISTER_BASE	0x00030280 /* Serial1 */
#define UART0_REGISTER_BASE	0x00030300 /* UART0 */
#define UART1_REGISTER_BASE	0x00030380 /* UART1 */
#define WDT_REGISTER_BASE	0x00030400 /* Watch dog timer */
#define MMCSD_REGISTER_BASE     0x00030480 /* MMC/SD */
#define INTCTRL_REGISTER_BASE	0x00030500 /* INTC */
#define GIO_REGISTER_BASE	0x00030580 /* GIO */
#define DSPC_REGISTER_BASE      0x00030600 /* DSP Controller */
#define OSD_REGISTER_BASE	0x00030680 /* OSD */
#define CCDC_REGISTER_BASE	0x00030700 /* CCD controller */
#define PVEN_REGISTER_BASE	0x00030780 /* Preview engine */
#define NTSCPAL_REGISTER_BASE	0x00030800 /* NTSC/PAL video encoder */
#define CLOCKC_REGISTER_BASE	0x00030880 /* Clock controller */
#define BUSC_REGISTER_BASE	0x00030900 /* Bus controller */
#define SDRAMC_REGISTER_BASE	0x00030980 /* SDRAM controller */
#define EMEMC_REGISTER_BASE	0x00030a00 /* External Memory Interface */
#define USB0_REGISTER_BASE      0x00030a80 /* USB 0 */
#define USB1_REGISTER_BASE      0x00030b00 /* USB 1 */
#define AF_REGISTER_BASE        0x00030b80 /* 3A(AF/AE/AWB) */
#define MSTICK1_REGISTER_BASE   0x00030c00 /* Memory Stick 1 */
#define MSTICK2_REGISTER_BASE   0x00030c80 /* Memory Stick 2 */
#define ATM_REGISTER_BASE       0x00030d00 /* ATM */
#define I2C_REGISTER_BASE       0x00030d80 /* I2C */
#define ICACHE_REGISTER_BASE    0x00030e00 /* Instruction cache */

/* Clock Controller */

#define CLOCKC_PLLA             0x00030880 /* PLLA config */
#define CLOCKC_PLLB             0x00030882 /* PLLB config */
#define CLOCKC_CLKC             0x00030884 /* CLK reverse, source select */
#define CLOCKC_SEL              0x00030886 /* PLL selection */
#define CLOCKC_DIV              0x00030888 /* DIV ratency config */
#define CLOCKC_BYP              0x0003088a /* PLL Bypass config */
#define CLOCKC_MMCLK            0x0003088c /* MMC CLK */
#define CLOCKC_CTST             0x0003088e /* CTST CLK */
#define CLOCKC_MOD0             0x00030890 /* CLK Enable/Disable */
#define CLOCKC_MOD1             0x00030892 /* CLK Enable/Disable */
#define CLOCKC_MOD2             0x00030894 /* CLK Enable/Disable */
#define CLOCKC_LPCTL0           0x00030896 /* SLEEP */
#define CLOCKC_LPCTL1           0x00030898 /* Power Down */
#define CLOCKC_OSEL             0x0003089a /* General CLK src selection */
#define CLOCKC_O0DIV            0x0003089c /* General CLK0 DIV config */
#define CLOCKC_O1DIV            0x0003089e /* General CLK1 DIV config */
#define CLOCKC_O2DIV            0x000308a0 /* General CLK2 DIV config */
#define CLOCKC_PWM0C            0x000308a2 /* PWM0 CLK config */
#define CLOCKC_PWM0H            0x000308a4 /* PWM0 H-Level duration config */
#define CLOCKC_PWM1C            0x000308a6 /* PWM1 CLK config */
#define CLOCKC_PWM1H            0x000308a8 /* PWM1 H-Level duration config */

/* Initial Clock Controller Settings */

#if 1
// TESTING
//# define PLLA_DATA  0x00e3 /* PLLA: 101.25 MHz (27MHz * 15 / 4) */
# define PLLA_DATA  0x0073 /* PLLA: 101.25 MHz (27MHz * 15 / 4) */
# define PLLB_DATA  0x00d1 /* PLLB: 189 MHz (27 MHz * 14 / 2) */
# define CLKC_DATA  0x07e0 /* set UART[0-1]/TIMER[0-3] clock to PLLIN */
# define SEL_DATA   0x1011 /* PLLA: SDRAM, PLLB: ACL, DSP, ARM */
# define DIV_DATA   0x0011
#else
# define PLLA_DATA  0x0091 /* PLLA config: M=10 N=2 */
# define PLLB_DATA  0x0091 /* PLLB config: M=10 N=2 */
# define CLKC_DATA  0x41e1 /* CLK reverse, CLK src select: Timer source MXI */
# define SEL_DATA   0x0000 /* PLL selection: AAAA selected */
# define DIV_DATA   0x1111 /* DIV ratency config: SDR=1/2 ARM=1/2 */
#endif

#define MMCLK_DATA  0x0002 /* MMC CLK: ARM/3 = 22.5MHz */
#define BYP_DATA    0x0000 /* PLL Bypass config: NO BYPASS */
#define MSCLK_DATA  0x0000 /* MS CLK */
#define MOD0_DATA   0x07ff /* CLK Enable/Disable: Enable ALL */
#define MOD1_DATA   0x01ff /* CLK Enable/Disable: Enable ALL */
#define MOD2_DATA   0x3fff /* CLK Enable/Disable: Enable ALL */
#define LPCTL0_DATA 0x0000 /* SLEEP: Normal */
#define LPCTL1_DATA 0x0000 /* Power Down: default */
#define OSEL_DATA   0x0000 /* General CLK src selection: default */
#define O0DIV_DATA  0x0000 /* General CLK0 DIV config: default */
#define O1DIV_DATA  0x0000 /* General CLK1 DIV config: default */
#define O2DIV_DATA  0x0000 /* General CLK2 DIV config: default */
#define PWM0C_DATA  0x0000 /* PWM0 CLK config: default */
#define PWM0H_DATA  0x0000 /* PWM0 H-Level duration config: default */
#define PWM1C_DATA  0x0000 /* PWM1 CLK config: default */
#define PWM1H_DATA  0x0000 /* PWM1 H-Level duration config: default */
#define MOD0_DATA_1 0x00e7 /* CLK Enable/Disable: Enable ALL */
#define MOD0_DATA_2 0x06ff /* CLK Enable/Disable: ALL Ena exp for CEHIF */

/* SDRAM Controller */

#define SDRAMC_BUF0             0x00030980 /* D0L */
#define SDRAMC_BUF1             0x00030982 /* D0H */
#define SDRAMC_BUF2             0x00030984 /* D1L */
#define SDRAMC_BUF3             0x00030986 /* D1H */
#define SDRAMC_BUF4             0x00030988 /* D2L */
#define SDRAMC_BUF5             0x0003098a /* D2H */
#define SDRAMC_BUF6             0x0003098c /* D3L */
#define SDRAMC_BUF7             0x0003098e /* D3H */
#define SDRAMC_BUF8             0x00030990 /* D4L */
#define SDRAMC_BUF9             0x00030992 /* D4H */
#define SDRAMC_BUFA             0x00030994 /* D5L */
#define SDRAMC_BUFB             0x00030996 /* D5H */
#define SDRAMC_BUFC             0x00030998 /* D6L */
#define SDRAMC_BUFD             0x0003099a /* D6H */
#define SDRAMC_BUFE             0x0003099c /* D7L */
#define SDRAMC_BUFF             0x0003099e /* D7H */
#define SDRAMC_AD0              0x000309a0 /* Address autoincr, SDR address1 */
#define SDRAMC_AD1              0x000309a2 /* SDR address2 */
#define SDRAMC_BUFCTL           0x000309a4 /* SDR and BUFF control */
#define SDRAMC_MODE             0x000309a6 /* SDR mode */
#define SDRAMC_REFCTL           0x000309a8 /* DMA select, Refresh control */
#define SDRAMC_SDPRTY1          0x000309aa /* Priority1 */
#define SDRAMC_SDPRTY2          0x000309ac /* Priority2 */
#define SDRAMC_SDPRTY3          0x000309ae /* Priority3 */
#define SDRAMC_SDPRTY4          0x000309b0 /* Priority4 */
#define SDRAMC_SDPRTY5          0x000309b2 /* Priority5 */
#define SDRAMC_SDPRTY6          0x000309b4 /* Priority6 */
#define SDRAMC_SDPRTY7          0x000309b6 /* Priority7 */
#define SDRAMC_SDPRTY8          0x000309b8 /* Priority8 */
#define SDRAMC_SDPRTY9          0x000309ba /* Priority9 */
#define SDRAMC_SDPRTY10         0x000309bc /* Priority10 */
#define SDRAMC_PRTYON           0x000309be /* PRTY Enable */
#define SDRAMC_SDRCTEST         0x000309bE

/* SDRAM Controller Initial Settings */

#if 1

#define SDMODE_DATA             0x3200
#define SDREF_DATA              0x0140
#define SDCNT_DATA1             0x3202
#define SDCNT_DATA2             0x3204
#define SDCNT_DATA3             0x3201

#elif (BSPCONF_SDRAM_TYPE == SDRAM_TYPE_K4S561632D)

/* assumes there are two SDRAM chips for a total of 64 Mbytes */

#define SDMODE_DATA             0x1300
#define SDREF_DATA              0x0140
#define SDCNT_DATA1             (SDMODE_DATA | 0x0002)
#define SDCNT_DATA2             (SDMODE_DATA | 0x0004)
#define SDCNT_DATA3             (SDMODE_DATA | 0x0001)

#elif (BSPCONF_SDRAM_TYPE == SDRAM_TYPE_HY57V653220)

/*                              32x1      32x1h  16x1   16x16 */
#define SDMODE_DATA             0x1800 /* 0x9C00 0x5900 0x1900 */
#define SDREF_DATA              0x0540 /* 0x0140 0x0140 0x0140 */
#define SDCNT_DATA1             0x1002 /* 0x9C02 0x5902 0x1902 */
#define SDCNT_DATA2             0x1004 /* 0x9C04 0x5904 0x1904 */
#define SDCNT_DATA3             0x1001 /* 0x9C01 0x5901 0x1901 */

#else

#error Unknown BSPCONF_SDRAM_TYPE

#endif

#define SDRAMC_REFCTL_DATA      0x0000
#define SDRAMC_SDPRTY1_DATA     0x0001
#define SDRAMC_SDPRTY2_DATA     0x0002
#define SDRAMC_SDPRTY3_DATA     0x0004
#define SDRAMC_SDPRTY4_DATA     0x0008
#define SDRAMC_SDPRTY5_DATA     0x0010
#define SDRAMC_SDPRTY6_DATA     0x0020
#define SDRAMC_SDPRTY7_DATA     0x0040
#define SDRAMC_SDPRTY8_DATA     0x0080
#define SDRAMC_SDPRTY9_DATA     0x0100
#define SDRAMC_SDPRTY10_DATA    0x0200

/* DSP Controller */

#define DSP_HPIB_CTL            0x00030600 /* HPIB Control Register */
#define DSP_HPIB_STAT           0x00030602 /* HPIB Status Register */

/* DSP Controller Initial Settings */

#define DSP_RESET_DATA          0x0689
#define DSP_NORMAL_DATA         0x0789
#define DSP_INT0_DATA           0x0709

/* External Bus */

#define EXBC_CS0CTRL1           0x00030a00 
#define EXBC_CS0CTRL2           0x00030a02
#define EXBC_CS1CTRL1A          0x00030a04
#define EXBC_CS1CTRL1B          0x00030a06
#define EXBC_CS1CTRL2           0x00030a08
#define EXBC_CS2CTRL1           0x00030a0a
#define EXBC_CS3CTRL1           0x00030a0e
#define EXBC_CS2CTRL2           0x00030a0c
#define EXBC_CS3CTRL2           0x00030a10
#define EXBC_CS4CTRL1           0x00030a12
#define EXBC_CS4CTRL2           0x00030a14
#define EXBC_BUSCTRL            0x00030a16
#define EXBC_BUSRLS             0x00030a18
#define EXBC_CFCTRL1            0x00030a1a
#define EXBC_CFCTRL2            0x00030a1c
#define EXBC_SMCTRL             0x00030a1e
#define EXBC_BUSINTEN           0x00030a20
#define EXBC_BUSSTS             0x00030a24
#define EXBC_BUSWAITMD          0x00030a26
#define EXBC_ECC1CP             0x00030a28
#define EXBC_ECC1LP             0x00030a2a
#define EXBC_ECC2CP             0x00030a2c
#define EXBC_ECC2LP             0x00030a2e
#define EXBC_ECCCLR             0x00030a30
#define EXBC_PAGESZ             0x00030a32
#define EXBC_IMGDSPDEST         0x00030a34
#define EXBC_PRIORCTL           0x00030a36
#define EXBC_SOURCEADD1         0x00030a38
#define EXBC_SOURCEADD2         0x00030a3a
#define EXBC_DESTADD1           0x00030a3c
#define EXBC_DESTADD2           0x00030a3e
#define EXBC_DMASIZE            0x00030a40
#define EXBC_DMADEVSEL          0x00030a42
#define EXBC_DMACTL             0x00030a44
#define EXBC_IMGDSPADD1         0x00030a46
#define EXBC_IMGDSPADD2         0x00030a48
#define EXBC_DPSTR0             0x00030a4a
#define EXBC_DPSTR1             0x00030a4c
#define EXBC_DPSTR2             0x00030a4e
#define EXBC_DPSTR3             0x00030a50
#define EXBC_DPSTR4             0x00030a52
#define EXBC_DPSTR5             0x00030a54
#define EXBC_TEST               0x00030a56

/* External Bus Initial Settings */

#define EXBC_CS0CTRL1_DATA      0x5877 /* OE=6w  WE=9w  CE=8w  CYC=8w */
#define EXBC_CS0CTRL2_DATA      0x3110 /* IDLE=3 OE=1   WE=1   CE=0 */
#define EXBC_CS1CTRL1A_DATA     0x1415 /* CE=21w CYC=22w */
#define EXBC_CS1CTRL1B_DATA     0x1109 /* OE=18w WE=10w */
#define EXBC_CS1CTRL2_DATA      0x1220 /* IDLE=1 OE=2   WE=2   CE=0 */
#define EXBC_CS2CTRL1_DATA      0x110a /* OE=2w  WE=2w  rsvd   CYC=11w */
#define EXBC_CS3CTRL1_DATA      0x1129 /* OE=2w  WE=2w  CE=3w  CYC=10w */
#define EXBC_CS2CTRL2_DATA      0x8899 /* IDLE=0 OE=8   WE=9   RE=12 */
#define EXBC_CS3CTRL2_DATA      0x1110 /* WD=8b  IDLE=1 OE=1   CE=0 */
#define EXBC_CS4CTRL1_DATA      0x889A /* OE=9w  WE=9w  CE=10w CYC=11w */
#define EXBC_CS4CTRL2_DATA      0x5220 /* LO BUS=16bit IDLE=3 OE=3 CE=0 */
#define EXBC_BUSCTRL_DATA       0x0000 /* Ext wait invalid */
#define EXBC_BUSRLS_DATA        0x0000 /* normal not-open not-open */
#define EXBC_CFCTRL1_DATA       0x0000 /* CF card selected */
#define EXBC_CFCTRL2_DATA       0x0000 /* CF 16bit low output */
#define EXBC_SMCTRL_DATA        0x0001 /* SCE low level */
#define EXBC_BUSINTEN_DATA      0x0000 /* CF low-high disabled */
#define EXBC_BUSSTS_DATA        0x0007 /* IOIS16+CFWAIT+CFRDY shown */
#define EXBC_BUSWAITMD_DATA     0x0000 /* CF RDY/-BSY connected */
#define EXBC_ECC1CP_DATA        0x00ff /* SM ECC CP */
#define EXBC_ECC1LP_DATA        0xffff /* SM ECC LP */
#define EXBC_ECC2CP_DATA        0x00ff /* SM ECC CP */
#define EXBC_ECC2LP_DATA        0xffff /* SM ECC LP */
#define EXBC_ECCCLR_DATA        0x0000 /* SM ECC clear */
#define EXBC_PAGESZ_DATA        0x0001 /* SM page size = 256+8byte/page */
#define EXBC_IMGDSPDEST_DATA    0x0000 /* IMG source=CS0 dest=CS0 */
#define EXBC_PRIORCTL_DATA      0x001b /* Priorities */
#define EXBC_SOURCEADD1_DATA    0x0000 /* DMA addr incr + upper address */
#define EXBC_SOURCEADD2_DATA    0x0000 /* DMA lower address */
#define EXBC_DESTADD1_DATA      0x0000 /* DMA addr incr + upper address */
#define EXBC_DESTADD2_DATA      0x0000 /* DMA lower address */
#define EXBC_DMASIZE_DATA       0x0000 /* DMA size of transfer */
#define EXBC_DMADEVSEL_DATA     0x0000 /* DMA src=CS0 dest=CS0 */
#define EXBC_DMACTL_DATA        0x0000 /* DMA no switching, not started */
#define EXBC_IMGDSPADD1_DATA    0x0000 /* IMG upper address */
#define EXBC_IMGDSPADD2_DATA    0x0000 /* IMG lower addres */

// Physical memory map

#if 1
#define EXBC_DPSTR0_DATA        0x0021 /* CS0 (FLASH) 10:0000-210:0000 */
#define EXBC_DPSTR1_DATA        0x0061 /* SDRAM      210:0000-610:0000 */
#define EXBC_DPSTR2_DATA        0x0066 /* CS1 (CF)   610:0000-660:0000 */
#define EXBC_DPSTR3_DATA        0x006a /* CS2 (SM)   660:0000-6a0:0000 */
#define EXBC_DPSTR4_DATA        0x006b /* CS3        6a0:0000-6b0:0000 */
#define EXBC_DPSTR5_DATA        0x006c /* CS4 ether  6b0:0000-6c0:0000 */
#else
#define EXBC_DPSTR0_DATA        0x0021 /* CS0 (FLASH) 10:0000-210:0000 */
#define EXBC_DPSTR1_DATA        0x0041 /* SDRAM      210:0000-410:0000 */
#define EXBC_DPSTR2_DATA        0x0081 /* CS1 (CF)   410:0000-810:0000 */
#define EXBC_DPSTR3_DATA        0x00a1 /* CS2 (SM)   810:0000-a10:0000 */
#define EXBC_DPSTR4_DATA        0x00c1 /* CS3        a10:0000-c10:0000 */
#define EXBC_DPSTR5_DATA        0x00ef /* CS4 ether  c10:0000-e10:0000 */
#endif

#define EXBC_TEST_DATA          0x0000 /* Normal mode */

/* UART Registers */

/* Register Offsets */

#define UART_DTRR              0     /* Data Transmit/Receive Register */
#define UART_BRSR              2     /* Bit Rate Set Register */
#define UART_MSR               4     /* Mode Set Register */
#define UART_RFCR              6     /* Reception FIFO Control Register */
#define UART_TFCR              8     /* Transmission FIFO Control Register */
#define UART_LCR              10     /* Line Control Register */
#define UART_SR               12     /* Status Register */

/* UART status */

#define UART_RX_FIFO_LEVEL    0x0800 /* RX FIFO >= trigger level */
#define UART_TX_FIFO_LEVEL    0x0400 /* TX FIFO < trigger level */
#define UART_ERROR            0x0200 /* Error RX FIFO */
#define UART_TIMEOUT          0x0100 /* Timeout */
#define UART_RX_FIFO_NOEMPTY  0x0004 /* RX FIFO is not empty */
#define UART_TX_FIFO_EMPTY    0x0002 /* TX FIFO is empty */
#define UART_TX_EMPTY         0x0001 /* both RX&TX FIFO shift-reg empty */

/* FIFO trigger level */

#define UART_TRIGGER_LEVEL_01 0x0000 /*  1-byte */
#define UART_TRIGGER_LEVEL_04 0x0100 /*  4-byte */
#define UART_TRIGGER_LEVEL_08 0x0200 /*  8-byte */
#define UART_TRIGGER_LEVEL_16 0x0300 /* 16-byte */
#define UART_TRIGGER_LEVEL_24 0x0400 /* 24-byte */
#define UART_TRIGGER_LEVEL_32 0x0500 /* 32-byte */

#define UART_TRIGGER_LEVEL_1_INIT UART_TRIGGER_LEVEL_01

/* UART mode settings */

#define UART_RXFIFO_INT       0x8000 /* Enable receiver trigger interrupt */
#define UART_TXFIFO_INT       0x4000 /* Enable transmitter trigger interrupt */
#define UART_RCVERR_INT       0x2000 /* Enable receiving error interrupt */
#define UART_NOTIMEOUT_INT    0x0000 /* Disable timeout interrupt */
#define UART_TIMEOUT_3WORD    0x0400 /* Enable timeout interrupt with 3-word */
#define UART_TIMEOUT_7WORD    0x0800 /* Enable timeout interrupt with 7-word */
#define UART_TIMEOUT_15WORD   0x0c00 /* Enable timeout interrupt with 15-word */
#define UART_NOPARITY         0x0000 /* No-parity */
#define UART_ODDPARITY        0x0018 /* Odd parity */
#define UART_EVENPARITY       0x0010 /* Even parity */
#define UART_STOPBIT_1        0x0000 /* Stop bit = 1bit */
#define UART_STOPBIT_2        0x0004 /* Stop bit = 2bit */
#define UART_DATABIT_7        0x0001 /* Data bit = 7bit */
#define UART_DATABIT_8        0x0000 /* Data bit = 8bit */

#define UART_MODE_1_INIT = UART_NOPARITY | UART_STOPBIT_1 | UART_DATABIT_8

/* UART error bits */

#define UART_DATA_VALID       0x10   /* Data is valid */
#define UART_BREAK_FLAG       0x08   /* Break detected */
#define UART_FRAME_ERROR      0x04   /* Frame error is detected */
#define UART_OVERRUN          0x02   /* Overrun detected */
#define UART_PARITY_ERROR     0x01   /* Parity error detected */

#if 1
/* UART BRSR baudrate based on DM270 at 27MHz
 * baud rate = ARM_CLK / 16 / (VALUE+1)
 * VALUE = (ARM_CLK / 16 / baud_rate ) - 1
 */

# define UART_BAUD_115200       13

#elif 0
/* UART BRSR baudrate based on DM270 at 94.5MHz
 * baud rate = ARM_CLK / 16 / (VALUE+1)
 * VALUE = (ARM_CLK / 16 / baud_rate ) - 1
 */

# define UART_BAUD_2400       2429
# define UART_BAUD_4800       1229
# define UART_BAUD_9600        614
# define UART_BAUD_14400       409
# define UART_BAUD_19200       307
# define UART_BAUD_28800       204
# define UART_BAUD_38400       153
# define UART_BAUD_57600       102
# define UART_BAUD_115200       50
# define UART_BAUD_230400       25
# define UART_BAUD_460800       12
# define UART_BAUD_921600        5
#else
/* UART BRSR baudrate based on DM270 at 67.5MHz
 * baud rate = ARM_CLK / 16 / (VALUE+1)
 * VALUE = (ARM_CLK / 16 / baud_rate ) - 1
 */

# define UART_BAUD_2400       0x06dc
# define UART_BAUD_4800       0x036d
# define UART_BAUD_9600       0x01b6
# define UART_BAUD_14400      0x0123
# define UART_BAUD_19200      0x00da
# define UART_BAUD_28800      0x0091
# define UART_BAUD_38400      0x006c
# define UART_BAUD_57600      0x0048
# define UART_BAUD_115200     0x0023
# define UART_BAUD_230400     0x0011
# define UART_BAUD_460800     0x0008
# define UART_BAUD_921600     0x0003
#endif

/* Synonyms */

#define UART_CONST_FIFO_CLEAR     0x8000 /* RX/TX FIFO clear bit (RFCR/TFCR) */
#define UART_CONST_WORDCOUNT_MASK 0x003f /* Word count mask (RFCR/TFCR) */
#define UART_CONST_BREAK          0x0100 /* Break mask (LCR) */
#define UART_CONST_TX_EMPTY       0x0001 /* Empty bit (TX) */
#define UART_CONST_RX_READ_VALID  0x1000

#endif /* __RRLOAD_DM270_REGISTER_H */
