/***************************************************************
*             Copyright (C) 2003 by SerComm Corp.
*                    All Rights Reserved.
*
*      Use of this software is restricted to the terms and
*      conditions of SerComm's software license agreement.
*
*                        www.sercomm.com
****************************************************************/
#include <stdio.h>
#include <string.h>
#include "types.h"
#include "flash.h"
#include "prototype.h"
#include "util.h"
#include "io.h"
#include "memconfig.h"

#define  KFS_START     0x00010000
#define  KFS_END       (FLASH_SIZE-(64<<10))

#define  WRITE_EALL             1
#define  WRITE_UPGRADE          2
#define  ERASE_EALL	        3
#define  ERASE_UPGRADE	        4
#define  ERASE_VERIFY_EALL      5
#define  ERASE_VERIFY_UPGRADE   6

#define VCI_LEN   56

typedef struct {
   unsigned char    DestAddress[6];
   unsigned char    SrcAddress[6];
   unsigned short   sap;
   unsigned short   wcmd;
   unsigned short   wsequence;
   unsigned short   woffset;
   unsigned short   wsegment;
   unsigned short   wLen;
   unsigned char    bData[600];
} DLC __attribute__((packed));

#define IDLE            0x00
#define PROG            0x01
#define PROGERR         0x02
#define X_PROGERR       0x0002
#define COM_SEQ_ERR     0x05
#define X_COM_SEQ_ERR   0x0005
#define SEQ_NUM_ERR     0x06
#define X_SEQ_NUM_ERR   0x0006
#define PROG_ERR        0x07
#define X_PROG_ERR      0x0007
#define VERIFY_ERR      0x09
#define X_VERIFY_ERR    0x0009
#define AS_ERR          0x08

#define GET_VERSION_INFO  0x0000
#define DOWN_REQUEST      0x0001
#define DOWN_DATA         0x0002
#define DOWN_RESET        0x0003
#define DOWN_VERIFY       0x0004
#define DOWN_EALL         0x0005
#define DOWN_REPLY        0x0006
#define AS_NORMAL         0x0008
#define AS_FORCE          0x0009

typedef struct DCB_TYPE {
   unsigned short  sequn;
   unsigned short  dest_seg;
   unsigned short dest_off;
   unsigned short src_seg;
   unsigned short src_off;
   unsigned short count;
   unsigned short erase;
   unsigned short no;
   unsigned short tme_of_day;
   unsigned short state;
} DCB_BUF;

static volatile DCB_BUF   dcb;
#define  PID_LEN   70
#define  PID_CONTENT_LEN	(PID_LEN-14)
static unsigned char PID[PID_LEN] = {
   // 0x73,0x45,0x72,0x43,0x6f,0x4d,0x6d, // Prifix[7], 'sErCoMm'
   0x00,0x00,0x00,0x00,0x00,0x00,0x00, // Prifix[7], 'sErCoMm'
   0x01,0x00,                          // VerControl, '1.0'
   0x00,0x01,                          // DownControl, 1
   0x04,0x70,0x56,0x00,0x00,0x00,0x00,0x00,
   0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
   0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
   0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,// Hid[32], '047056' for WVC54G
   0x00,0x00,                          // Hver
   0x00,0x00,                          // ProdID
   0x00,0x00,                          // ProIDmask
   0x00,0x00,                          // ProtID
   0x00,0x00,                          // ProtIDmask
   0x00,0x00,                          // FuncID
   0x00,0x00,                          // FuncIDmask
   0x10,0x00,                          // Firmware version
   0x00,0x00,                          // Cseg
   0x00,0x08,                          // Code size, '2048'
   //0x73,0x45,0x72,0x43,0x6f,0x4d,0x6d  // Postfix[7], 'sErCoMm'
   0x00,0x00,0x00,0x00,0x00,0x00,0x00  // Postfix[7], 'sErCoMm'
};

static u8 sendbuf[1024];
u8 mac[6] = {0x00,0xC0,0x02,0xFF,0xFF,0xFF};

static void getPIDfromFlash(void)
{
   char *p = (char *)(FLASH_BASE+FLASH_SIZE - 7); // "sErCoMm"
   int  fn = 0, i = 0;

   for (; (int)p > (FLASH_BASE+KFS_START); p--)
   {
      // search for 'sErCoMm' where the PID located
      if (p[0] != 's')
         continue;
      if (p[1] != 'E')
         continue;
      if (p[2] != 'r')
         continue;
      if (p[3] != 'C')
         continue;
      if (p[4] != 'o')
         continue;
      if (p[5] != 'M')
         continue;
      if (p[6] != 'm')
         continue;

	p -= (PID_CONTENT_LEN + 7);

	// search second "sErCoMm" in PID
	if(p[0]=='s'&&p[1]=='E'&&p[2]=='r'&&p[3]=='C'&&p[4]=='o'&&p[5]=='M'&&p[6]=='m'){
		fn = 1;
		util_printf("got PID in flash\n");
	      	break;
	}
	else{	
		continue;	
      	}	
   }
   if (fn)
   {
      p += 7;
      for (i=0; i< PID_CONTENT_LEN; i++)
      		PID[i+7] = *p++;
   }
}

static void blink_all(void)
{
   static int i = 0;

   if (i == 0)
      //GpIOBitSet1(LED_GPIO_PWR);
      GpIOBitClear1(LED_GPIO_PWR);
   i++;
   if (i == 40000)
      //GpIOBitClear1(LED_GPIO_PWR);
      GpIOBitSet1(LED_GPIO_PWR);
   if (i == 80000)
      i = 0;
}

static void reboot(void)
{
#if 1
   // ensure reboot bit is cleared
   //GpIOBitSet1(GPIO_REBOOT);
   // ensure directions for the bits we are using
   // DR0Set(GPIO_REBOOT);
   // GpIOBitClear(GPIO_REBOOT);
    GpIOBitClear1(GPIO_REBOOT);
#endif
#if 0
   typedef void(*ENTRY)(void);
   ENTRY entry_addr;

   entry_addr = (ENTRY) 0;

   (*(entry_addr))(); 
#endif
}

#if 0
static int flash_erase(void)
{
   int st = 0;

#define BSPCONF_PARAMS_FLASH_SIZE 0x00020000

   st = FWBErase(FLASH_BASE,BTLDR_FLASH_SIZE-1,0);
   if (st == 0)
      return 1;
   return !FWBErase(BSPCONF_PARAMS_FLASH_START,
      BSPCONF_PARAMS_FLASH_SIZE+BSPCONF_KERNEL_FLASH_SIZE+
      BSPCONF_FS_FLASH_SIZE-1,0);
}

static int flash_erase_range(u8* S, u8* E)
{
   return !FWBErase((u32)S+FLASH_BASE,FLASH_BASE+E-S,0);
}

int flash_write(u8* pDst, u8* pSrc, int cnt)
{
   int i;

   pDst += FLASH_BASE;
   i = FWBOpen((u32)pDst);
   while (cnt)
   {
     if (!FWBWriteByte((u32)pDst++,(char)*pSrc++))
     {
        util_printf("ERROR: Flash write aborted %x\n", (int)pDst);
        return 1;
     }
     cnt--;
   }
   i = FWBClose();

   return 0;
}

static int block_erase(u16 *adr)
{
   return !FWBErase((u32)adr,16,0);
}
#endif

static int writeMAC2Flash(char *new_mac)
{
   u8 *ptr, *offsetPtr = NULL;
   int retVal,i,len;
   u8 *bs;

   ptr = (u8 *) EMPTY_SDRAM_START;
   len = 0;
   bs = (u8 *) FLASH_MAC_ADDR;
   flash_read((u32)bs-FLASH_BASE, (u16*)ptr, (8<<10));

   /* Now Erase the block */ 
   retVal = block_erase((u16*)bs);
   if (retVal != 0)
   {
      util_printf("ASMode: Flash Erase block failed\n");
      return 1;
   }

   /* Now update the MAC address in the local copy at
    * Intel offset and Write it back into the Flash
    */
   offsetPtr = ptr;
   for (i=0; i<6; i++, offsetPtr++)
      *offsetPtr = new_mac[i];
   retVal = flash_write((u32)bs-FLASH_BASE, (u16*)ptr, (8<<10));
   if (retVal != 0)
   {
      util_printf("ASMode: Flash Write block failed\n");
      return 1;
   }
   return 0;
}

int ASMode(void)
{
   u8 rec_buf[1024];
   u16 lenrec;
   int i;
   int asn=0;
   DLC * psDLCHead;
   DLC * psDLCHead1;
   u8 src_mac[6];
   u8 new_mac[6];

   u_memset(src_mac,0x00,6);
   u_memset(new_mac,0x00,6);
util_printf("ASMode.\n");
   getPIDfromFlash();
   u_memset (rec_buf, 0x00, 1024);
   while(1)
   {
      blink_all();
      Lan_Receive((u16*)rec_buf,&lenrec);
      if (lenrec)
      {	
         psDLCHead= (DLC *) rec_buf;
         if (( rec_buf[24]=='P')  && (rec_buf[25]=='M') &&
             (rec_buf[26]=='T') && (rec_buf[27]=='I')  )
         {
            if (asn > 0) // we have processed once
               continue;
            u_memcpy (new_mac, (u8*)&rec_buf[14],6);

            if (writeMAC2Flash(new_mac))
               return 0;

            for (i=0; i<6; i++)
               mac[i] = new_mac[i];
            dmfe_SetMACAddr(mac);
            u_memcpy((u8*)src_mac,(u8*)&rec_buf[6],6); // From who
            asn = 1;
util_printf("ASMode: done assign\n");
         }
         else
         {
            if (rec_buf[0] == 0xFF && rec_buf[1] == 0xFF &&
                rec_buf[2] == 0xFF && rec_buf[3] == 0xFF &&
                rec_buf[4] == 0xFF && rec_buf[5] == 0xFF)
               continue; // it is a broadcast, ignore it

            if (rec_buf[0] != mac[0] || rec_buf[1] != mac[1] ||
                rec_buf[2] != mac[2] || rec_buf[3] != mac[3] ||
                rec_buf[4] != mac[4] || rec_buf[5] != mac[5])
               continue; // it is not the packet for me, ignore it

            if (rec_buf[6] != src_mac[0] || rec_buf[7] != src_mac[1] ||
                rec_buf[8] != src_mac[2] || rec_buf[9] != src_mac[3] ||
                rec_buf[10] != src_mac[4] || rec_buf[11] != src_mac[5])
               continue; // it is not the one who change my MAC, ignore it

util_printf("Got Packet for me\n");
            if (asn && psDLCHead->sap ==0x8888)
            {
util_printf("ASMode reply\n");
               u_memset(sendbuf,0x00,sizeof(DLC));
               psDLCHead1=(DLC *)sendbuf;
               u_memcpy((u8 *)psDLCHead1->DestAddress,
                  (u8 *)psDLCHead->SrcAddress,6);
               u_memcpy((u8 *)psDLCHead1->SrcAddress,mac,6);
               psDLCHead1->wcmd = psDLCHead->wcmd;
               psDLCHead1->sap = psDLCHead->sap;
               psDLCHead1->wsequence = psDLCHead->wsequence;
               psDLCHead1->woffset = psDLCHead->woffset;
               psDLCHead1->wsegment = psDLCHead->wsegment;
               psDLCHead1->wLen=0x0038;  // 56
               u_memcpy((u8 *)psDLCHead1->bData,
                  (u8 *)&PID[7],VCI_LEN);
               // give him a reply telling that I am done
               for (asn = 1; asn <= 12; asn++)
               {
                  Lan_Transmit((u16 *)psDLCHead1,(VCI_LEN+24));
                  Lan_Transmit((u16 *)psDLCHead1,(VCI_LEN+24));
                  for (i = 0; i < 7000; i++) ;
               }
util_printf("ASMode reboot\n");
               reboot();
               return 0;
            } // if (sap)
         } // else
      } // if (lenrec)
   } // while
   return 1;
}


int Download(void)
{
   DLC * psDLCHead;
   DLC * psDLCHead1;
   int as;
   unsigned long i;
   unsigned long ulAddress;
   unsigned short *p;
   unsigned long ulCount;
   unsigned short len_rec;
   unsigned short rec_buf[1024];
   int cnt = 0;

util_printf("Download.\n");
   getPIDfromFlash();
   u_memset ((char *)rec_buf,0x00,sizeof(DLC)); 
   while(1)
   {
      len_rec = 0;
      Lan_Receive((unsigned short*)rec_buf,&len_rec);
      if (len_rec)
      {
         psDLCHead= (DLC *) rec_buf;
         if (psDLCHead->sap ==0x8888)
         {
            u_memset(sendbuf,0x00,sizeof(DLC));
            psDLCHead1=(DLC *)sendbuf;
            u_memcpy((unsigned char*)psDLCHead1->DestAddress,
               (unsigned char*)psDLCHead->SrcAddress,6);
            u_memcpy((unsigned char*)psDLCHead1->SrcAddress, mac,6);
            psDLCHead1->wcmd = psDLCHead->wcmd;
            psDLCHead1->sap = psDLCHead->sap;
            psDLCHead1->wsequence = psDLCHead->wsequence;
            psDLCHead1->woffset = psDLCHead->woffset;
            psDLCHead1->wsegment = psDLCHead->wsegment;

            switch (psDLCHead->wcmd)
            {
            case GET_VERSION_INFO:
               psDLCHead1->wLen=0x0038;  // 56
               u_memcpy((unsigned char *)psDLCHead1->bData,
                  (unsigned char *)&PID[7],VCI_LEN);
               Lan_Transmit((unsigned short*)psDLCHead1,(VCI_LEN+24));
               break;

            case DOWN_EALL:
            case DOWN_REQUEST:
               dcb.state=PROG;
               dcb.sequn=((psDLCHead->wsequence+1) & 0xffff);
               if (psDLCHead->wcmd==DOWN_REQUEST)
                  dcb.erase=4;
               else
                  dcb.erase=3;
               dcb.no=0;
               psDLCHead1->wLen=0x0002;
               Lan_Transmit((unsigned short*)psDLCHead1,64);
               break;

            case DOWN_DATA:
               switch (dcb.state)
               {
               case PROG:
                  if (dcb.sequn==psDLCHead->wsequence)
                  {
                     ulAddress=((psDLCHead->wsegment)<<4)+psDLCHead->woffset;
                     if (ulAddress == 0)
                        dcb.no++;
                     ulAddress += (0x100000*(dcb.no-1));
                     ulCount = (unsigned long)(psDLCHead->wLen);
                     if ( dcb.erase == ERASE_EALL)  // erase all
                     {
                        util_printf("erase all...\n");
                        if (flash_erase() != 0)
                        {
                           dcb.state=PROGERR;
                           *((u16 *)&psDLCHead1->bData[0])=X_PROG_ERR;
                           util_printf("fail.\n");
                        } else
                        {
                           dcb.erase= WRITE_EALL;
                           util_printf("done.\n");
                        }
                     }
                     else
                     if (dcb.erase == ERASE_UPGRADE) //upgrade 
                     {
                        util_printf("erase range...\n");
                        if(flash_erase_range((u32)KFS_START, (u32)KFS_END) != 0)
                        {
                           dcb.state=PROGERR;
                           *((u16 *)&psDLCHead1->bData[0])=X_PROG_ERR;
                           util_printf("fail.\n");
                        } else
                        {
                           dcb.erase= WRITE_UPGRADE;
                           util_printf("done.\n");
                        }
                     }
                     if( dcb.erase == WRITE_EALL)
                     {
                        dcb.sequn++;
                        *((u16 *)&psDLCHead1->bData[0])=0;
                        // skip MAC
                        if (ulAddress < BTLDR_FLASH_SIZE ||
                            ulAddress >= KFS_START)
                        { // skip MAC
                           // util_printf("write %x\n",ulAddress);
                           if (flash_write((u32)ulAddress,
                                (u16*)(psDLCHead->bData),ulCount))
                           {
                              dcb.state=PROGERR;
                              *((u16 *)&psDLCHead1->bData[0])=X_PROG_ERR;
                           }
                        }
                     }
                     else
                     if (dcb.erase == WRITE_UPGRADE)
                     {
                        dcb.sequn++;
                        *((u16 *)&psDLCHead1->bData[0])=0; // set return to OK
                        if (ulAddress >= KFS_START) 
                        {  // only offset greater than KFS_START will write
                           // util_printf("write %x\n",ulAddress);
                           if (flash_write((u32)ulAddress,
                                (u16*)(psDLCHead->bData),ulCount) != 0)
                           {
                              dcb.state=PROGERR;
                              *((u16 *) &psDLCHead1->bData[0])=X_PROG_ERR;
                           }
                        }
                     }
                  }
                  else 
                  {
                     if (dcb.sequn >= psDLCHead->wsequence) 
                        *((u16 *)&psDLCHead1->bData[0])=0;
                     else 
                        *((u16 *)&psDLCHead1->bData[0])=X_SEQ_NUM_ERR;
                  } /* endif */
                  psDLCHead1->wLen=0x0002;                      
                  break;	                	

               case IDLE:
                  psDLCHead1->wLen=0x0002;
                  *((u16 *) &psDLCHead1->bData[0])=X_COM_SEQ_ERR;
                  break;
               } //end of switch	

               Lan_Transmit((unsigned short*)psDLCHead1,64);
               break;

            case DOWN_RESET:
               if (dcb.state == PROG && dcb.erase == ERASE_VERIFY_EALL)
                  writeMAC2Flash(mac);
               // Calculate new check sum 3
               // write new check sum 3
               psDLCHead1->wLen=0x0000;
               *((u16 *)&psDLCHead1->bData[0])=0;
               Lan_Transmit((unsigned short*)psDLCHead1,64);
               Lan_Transmit((unsigned short*)psDLCHead1,64);
               for (i = 0; i < 7000; i++) ;
               Lan_Transmit((unsigned short*)psDLCHead1,64);
               Lan_Transmit((unsigned short*)psDLCHead1,64);
               for (i = 0; i < 7000; i++) ;
               Lan_Transmit((unsigned short*)psDLCHead1,64);
               Lan_Transmit((unsigned short*)psDLCHead1,64);
               reboot();
               break;

            case DOWN_VERIFY:
               switch (dcb.state)
               {
               case PROG:
                  if (dcb.sequn==psDLCHead->wsequence)
                  {
                     if (dcb.erase==WRITE_EALL)
                     {
                        dcb.erase = ERASE_VERIFY_EALL; //just for reset flag
                        dcb.no = 0;
                     } /* endif */
                     if (dcb.erase==WRITE_UPGRADE)
                     {
                        dcb.erase = ERASE_VERIFY_UPGRADE; //just for reset flag
                        dcb.no = 0;
                     } /* endif */
                     ulAddress = (psDLCHead->wsegment << 4)+ psDLCHead->woffset;
                     if (ulAddress == 0)
                        dcb.no++;
                     ulAddress += (0x100000*(dcb.no-1));
                     ulCount = ((u32)(psDLCHead->wLen));
                     psDLCHead1->bData[0]=0;
                     psDLCHead1->bData[1]=0;
                     ulCount=(ulCount+1)>>1;
                     if (ulAddress>=KFS_START)
                     {
                        ulAddress += FLASH_BASE;
                        for(i=0;i<ulCount;i++)
                        {
                           p =(unsigned short *) (ulAddress + i*2);  
                           if (*p !=*(u16 *)&psDLCHead->bData[i*2])
                           {
                              dcb.state=PROGERR;
                              *((u16 *) &psDLCHead1->bData[0])=X_VERIFY_ERR;	
                              break;
                           }
                        }
                     }
                     ++dcb.sequn;
                  }
                  else 
                  {
                     if ((dcb.sequn-1)==psDLCHead->wsequence) 
                        *((u16 *) &psDLCHead1->bData[0])=0;
                     else 
                        *((u16*)&psDLCHead1->bData[0])=X_SEQ_NUM_ERR;
                  } /* endif */
                  psDLCHead1->wLen=0x0002;
                  Lan_Transmit((unsigned short*)psDLCHead1,64);
                  break;
               } // end of switch (dcb.status)
               break;

            case AS_NORMAL:
               as=ASMode();
               break;

            case AS_FORCE:
               as=ASMode();
               break;
            } // end of switch (dcb.state))
         }  // end of if (psDLCHead->sap == 0x0000 || psDLCHead->sap == 0x8888)
      } //end of if ((rec_len=Lan_Receive((u16*)psDLCHead,&len_rec))!=0)
      if (cnt == 0)
         //GpIOBitSet1(LED_GPIO_PWR);
         GpIOBitClear1(LED_GPIO_PWR);
      cnt++;
      if (cnt == 40000)
         //GpIOBitClear1(LED_GPIO_PWR);
         GpIOBitSet1(LED_GPIO_PWR);
      if (cnt == 80000)
         cnt = 0;
   } //end of while(1)
}

void to_download(int dnl)
{
   dmfe_GetMACAddr(mac);
//   dmfe_init();
//   dmfe_SetMACAddr(mac);
   // getPIDfromFlash();
   if (dnl)
      Download();
   else
      ASMode();
   return;
}
