// File: hwinit_dm270.S
//
// Common portions of setup, head (rrload) and head (linux)
//
// Contains the necessary h/w setup specific to the TI DM270 EVM board.
//
// Derived from setup_dsc21.S, by RidgeRun, Inc.
// 09/26/02 Todd Fischer
//
//  This program is free software; you can redistribute  it and/or modify it
//  under  the terms of  the GNU General  Public License as published by the
//  Free Software Foundation;  either version 2 of the  License, or (at your
//  option) any later version.
//
//  THIS  SOFTWARE  IS  PROVIDED  ``AS  IS''  AND   ANY  EXPRESS  OR IMPLIED
//  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
//  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT,  INDIRECT,
//  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
//  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
//  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
//  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//  You should have received a copy of the  GNU General Public License along
//  with this program; if not, write  to the Free Software Foundation, Inc.,
//  675 Mass Ave, Cambridge, MA 02139, USA.

//#include "memconfig.h"
//#include "dm270-registers.h"

/* External Bus */

#define EXBC_CS0CTRL1           0x00030a00 
#define EXBC_CS0CTRL2           0x00030a02
#define EXBC_CS1CTRL1A          0x00030a04
#define EXBC_CS1CTRL1B          0x00030a06
#define EXBC_CS1CTRL2           0x00030a08
#define EXBC_CS2CTRL1           0x00030a0a
#define EXBC_CS2CTRL2           0x00030a0c
#define EXBC_CS3CTRL1           0x00030a0e
#define EXBC_CS3CTRL2           0x00030a10
#define EXBC_CS4CTRL1           0x00030a12
#define EXBC_CS4CTRL2           0x00030a14
#define EXBC_BUSCTRL            0x00030a16
#define EXBC_BUSRLS             0x00030a18
#define EXBC_CFCTRL1            0x00030a1a
#define EXBC_CFCTRL2            0x00030a1c
#define EXBC_SMCTRL             0x00030a1e
#define EXBC_BUSINTEN           0x00030a20
#define EXBC_BUSSTS             0x00030a24
#define EXBC_BUSWAITMD          0x00030a26
#define EXBC_ECC1CP             0x00030a28
#define EXBC_ECC1LP             0x00030a2a
#define EXBC_ECC2CP             0x00030a2c
#define EXBC_ECC2LP             0x00030a2e
#define EXBC_ECCCLR             0x00030a30
#define EXBC_PAGESZ             0x00030a32
#define EXBC_IMGDSPDEST         0x00030a34
#define EXBC_PRIORCTL           0x00030a36
#define EXBC_SOURCEADD1         0x00030a38
#define EXBC_SOURCEADD2         0x00030a3a
#define EXBC_DESTADD1           0x00030a3c
#define EXBC_DESTADD2           0x00030a3e
#define EXBC_DMASIZE            0x00030a40
#define EXBC_DMADEVSEL          0x00030a42
#define EXBC_DMACTL             0x00030a44
#define EXBC_IMGDSPADD1         0x00030a46
#define EXBC_IMGDSPADD2         0x00030a48
#define EXBC_DPSTR0             0x00030a4a
#define EXBC_DPSTR1             0x00030a4c
#define EXBC_DPSTR2             0x00030a4e
#define EXBC_DPSTR3             0x00030a50
#define EXBC_DPSTR4             0x00030a52
#define EXBC_DPSTR5             0x00030a54
#define EXBC_TEST               0x00030a56

/* External Bus Initial Settings */

//#define EXBC_CS0CTRL1_DATA      0x889A
#define EXBC_CS0CTRL1_DATA      0x779A
#define EXBC_CS0CTRL2_DATA      0x1110 /* IDLE=1 OE=1   WE=1   CE=0 */
#define EXBC_CS1CTRL1A_DATA     0x1415 /* CE=21w CYC=22w */
#define EXBC_CS1CTRL1B_DATA     0x1109 /* OE=18w WE=10w */
#define EXBC_CS1CTRL2_DATA      0x1220 /* IDLE=1 OE=2   WE=2   CE=0 */
#define EXBC_CS2CTRL1_DATA      0x3207
#define EXBC_CS3CTRL1_DATA      0x889A
#define EXBC_CS2CTRL2_DATA      0x1129
#define EXBC_CS3CTRL2_DATA      0x5110 /* WD=16b  IDLE=1 OE=1   CE=0 */
#define EXBC_CS4CTRL1_DATA      0xCCCD
#define EXBC_CS4CTRL2_DATA      0x7110
#define EXBC_BUSCTRL_DATA       0x0000 /* Ext wait invalid */
#define EXBC_BUSRLS_DATA        0x0000 /* normal not-open not-open */
#define EXBC_CFCTRL1_DATA       0x0000 /* CF card selected */
#define EXBC_CFCTRL2_DATA       0x0000 /* CF 16bit low output */
#define EXBC_SMCTRL_DATA        0x0001 /* SCE low level */
#define EXBC_BUSINTEN_DATA      0x0000 /* CF low-high disabled */
#define EXBC_BUSSTS_DATA        0x0007 /* IOIS16+CFWAIT+CFRDY shown */
#define EXBC_BUSWAITMD_DATA     0x0000 /* CF RDY/-BSY connected */
#define EXBC_ECC1CP_DATA        0x00ff /* SM ECC CP */
#define EXBC_ECC1LP_DATA        0xffff /* SM ECC LP */
#define EXBC_ECC2CP_DATA        0x00ff /* SM ECC CP */
#define EXBC_ECC2LP_DATA        0xffff /* SM ECC LP */
#define EXBC_ECCCLR_DATA        0x0000 /* SM ECC clear */
#define EXBC_PAGESZ_DATA        0x0001 /* SM page size = 256+8byte/page */
#define EXBC_IMGDSPDEST_DATA    0x0000 /* IMG source=CS0 dest=CS0 */
#define EXBC_PRIORCTL_DATA      0x001b /* Priorities */
#define EXBC_SOURCEADD1_DATA    0x0000 /* DMA addr incr + upper address */
#define EXBC_SOURCEADD2_DATA    0x0000 /* DMA lower address */
#define EXBC_DESTADD1_DATA      0x0000 /* DMA addr incr + upper address */
#define EXBC_DESTADD2_DATA      0x0000 /* DMA lower address */
#define EXBC_DMASIZE_DATA       0x0000 /* DMA size of transfer */
#define EXBC_DMADEVSEL_DATA     0x0000 /* DMA src=CS0 dest=CS0 */
#define EXBC_DMACTL_DATA        0x0000 /* DMA no switching, not started */
#define EXBC_IMGDSPADD1_DATA    0x0000 /* IMG upper address */
#define EXBC_IMGDSPADD2_DATA    0x0000 /* IMG lower addres */
        
// Physical memory map

#define EXBC_DPSTR0_DATA        0x0009 /* CS0 (FLASH) 10:0000-090:0000 */
#define EXBC_DPSTR1_DATA        0x0049 /* SDRAM       90:0000-490:0000 */
#define EXBC_DPSTR2_DATA        0x0059 /* CS1 (CF)   490:0000-590:0000 */
#define EXBC_DPSTR3_DATA        0x0069 /* CS2 (SM)   590:0000-690:0000 */
#define EXBC_DPSTR4_DATA        0x0079 /* CS3        690:0000-790:0000 */
#define EXBC_DPSTR5_DATA        0x0081 /* CS4 ether  790:0000-810:0000 */

#define EXBC_TEST_DATA          0x0000 /* Normal mode */

#define GIO_DIR0                0x030580
#define GIO_DIR1		0x030582
#define GIO_BITSET0             0x03058C
#define GIO_BITSET1             0x03058E
#define GIO_BITCLR0             0x030592
#define GIO_BITCLR1             0x030594
#define GIO_FSEL0		0x0305A4
#define	GIO_FSEL1		0x0305A6
#define	GIO_INV0		0x030586
#define	GIO_INV1		0x030588

#define GIO_DIR0_DATA           0x7EDF
#define GIO_BITSET0_DATA        0x0120
#define GIO_BITCLR0_DATA        0x8000

        	
// **********************************************************
// Register and data definitions
// **********************************************************
#define ICACHE_ICAMODE    0x30E00
#define CLOCKC_PLLA	  0x30880
#define CLOCKC_PLLB	  0x30882
#define CLOCKC_CLKC	  0x30884
#define CLOCKC_SEL	  0x30886
#define CLOCKC_DIV	  0x30888
#define CLOCKC_BYP	  0x3088A
#define CLOCKC_MOD0	  0x30890
#define CLOCKC_MOD1	  0x30892
#define CLOCKC_MOD2	  0x30894


#define ICACHE_DATA_CLEAR 0x0002
#define ICACHE_DATA_ENABLE 0x0001

#if 1
        // Ingenient values to run at    94.500 MHz   
        // 23.39 BogoMIPS

#define PLLA_DATA        0x00E3
#define PLLB_DATA        0x00D1
#define SEL_DATA	 0x1011
#define DIV_DATA	 0x0011

#else
/* Values that allow networking to work
         8.26 BogoMIPS
   PLLA: Fin=27 Mhz, M=10, N=4,
   PLLA = ( Fin * M / N ) = 27 Mhz * 10 / 4 = 67.5 Mhz

   PLLB: Fin=27 Mhz, M=10, N=4,
   PLLB = ( Fin * M / N ) = 27 Mhz * 10 / 4 = 67.5 Mhz

    ARM: PLLA / 2 = 33.75 Mhz
    DSP: PLLA / 1 = 67.5 Mhz
    AXL: PLLA / 1 = 67.5 Mhz
  SDRAM: PLLA / 1 = 67.5 Mhz
*/ 

#  define PLLA_DATA   0x0093 /* PLLA config: M=10 N=4 */
#  define PLLB_DATA   0x0093 /* PLLB config: M=10 N=4 */
#  define SEL_DATA    0x0000 /* PLL selection: AXL:A, SDRAM:A, 
                                               DSP:A, ARM:A */
#  define DIV_DATA    0x0001 /* DIV ratency config: AXL=1, SDRAM=1/2 
                                                    DSP=1/2, ARM=1/2 */
#endif
        
#define CLKC_DATA	 0x07E0
        
#define BYP_DATA	 0x0000
#define MOD0_DATA	 0xFFFF
#define MOD1_DATA	 0xFFFF
#define MOD2_DATA	 0xFFFF
#define BYPON_DATA	 0xFFFF
#define BYPOFF_DATA	 0x0000
        
        		
	.global hw_ini
	// Save lr so we can return
hw_ini:	mov r9, lr
	    
        // Initialize the board's clocks.
        bl clock_ini
      
        // Pause to let PLLs lock
        mov  r1, #0xFF00
wloop:  subs r1, r1, #1
        bne  wloop

	// Initialize board's memory controller
        bl memcntl_ini
        
        // Initialize board's SDRAM controller
        bl sdram_ini

	// Initialize the instruction cache
	bl icache_ini

	bl kk_test
        // Enable CS8900 LAN chip
        //bl lan_ini
        		

done_hwinit:
	mov lr, r9   	
	mov pc, lr	
        
// **********************
// Macro regw(add,data)
// **********************
        .macro regw, reg_add, reg_data
        ldr r0, =\reg_add
        ldr r1, =\reg_data
        strh r1, [r0]       
        .endm

//lan_ini:
//        regw GIO_DIR0,  GIO_DIR0_DATA
//        regw GIO_BITSET0, GIO_BITSET0_DATA
//        regw GIO_BITCLR0, GIO_BITCLR0_DATA     
//     	mov	pc, lr

kk_test:
	regw GIO_FSEL0,  0x8000
	regw GIO_FSEL1,  0x0001
	regw GIO_INV0,  0x0000
	regw GIO_INV1,  0x0000
	regw GIO_BITSET1, 0x0200
        regw GIO_DIR0,  0xDFFF
        regw GIO_DIR1,  0x80BB
//        regw GIO_BITSET1,  0x1000 // reset dm9000
     	mov	pc, lr
           

// **************************************
// Instruction Cache Initialize
// **************************************
icache_ini:
	regw    ICACHE_ICAMODE, ICACHE_DATA_CLEAR
icache_wait:
	ldr     r0, =ICACHE_ICAMODE
	ldrh    r1, [r0]   		; r1 = ICACHE_ICAMODE
	bic     r1, r1, #0xFB
	tst     r1, r1
	bne     icache_wait

	regw    ICACHE_ICAMODE, ICACHE_DATA_ENABLE
	mov	pc, lr
	
// **************************************
// CLOCK controller initialize
// **************************************
clock_ini:

	// Select clock source
	regw    CLOCKC_CLKC, CLKC_DATA 

	// PLL bypass register
	regw	CLOCKC_BYP, BYPON_DATA

	// Enable all modules
	regw    CLOCKC_MOD0, MOD0_DATA
	regw    CLOCKC_MOD1, MOD1_DATA
	regw    CLOCKC_MOD2, MOD2_DATA
		
	// Initialize PLL settings
	regw	CLOCKC_PLLA, PLLA_DATA  @ ARM Clock
	regw	CLOCKC_PLLB, PLLB_DATA	@ DSP Clock
	regw	CLOCKC_SEL,  SEL_DATA
	regw    CLOCKC_DIV,  DIV_DATA

	// Wait for all PLL's to lock
	ldr	r0, =CLOCKC_PLLA
	ldr	r1, =CLOCKC_PLLB

pll_wait:
	ldrh	r2, [r0]        @ r2 = PLLA value
	bic	r2, r2, #0x7F   @ 32-bit mode
	tst	r2, r2
	beq	pll_wait	@ keep looping until non-zero

	ldrh	r2, [r1]        @ r2 = PLLB value 
	bic	r2, r2, #0x7F	
	tst	r2, r2
	beq	pll_wait	@ keep looping until non-zero

	regw	CLOCKC_BYP, BYPOFF_DATA
	mov	pc, lr	

// *******************************************************
// Memory Controller initialize
// *******************************************************

memcntl_ini:
	regw EXBC_CS1CTRL1A, EXBC_CS1CTRL1A_DATA
	regw EXBC_CS1CTRL1B, EXBC_CS1CTRL1B_DATA
	regw EXBC_CS1CTRL2, EXBC_CS1CTRL2_DATA
	regw EXBC_CS2CTRL1, EXBC_CS2CTRL1_DATA
	regw EXBC_CS3CTRL1, EXBC_CS3CTRL1_DATA
	regw EXBC_CS2CTRL2, EXBC_CS2CTRL2_DATA
	regw EXBC_CS3CTRL2, EXBC_CS3CTRL2_DATA
	regw EXBC_CS4CTRL1, EXBC_CS4CTRL1_DATA
	regw EXBC_CS4CTRL2, EXBC_CS4CTRL2_DATA
	regw EXBC_BUSCTRL, EXBC_BUSCTRL_DATA
	regw EXBC_BUSRLS, EXBC_BUSRLS_DATA
	regw EXBC_DPSTR0, EXBC_DPSTR0_DATA
	regw EXBC_DPSTR1, EXBC_DPSTR1_DATA
	regw EXBC_DPSTR2, EXBC_DPSTR2_DATA
	regw EXBC_DPSTR3, EXBC_DPSTR3_DATA
	regw EXBC_DPSTR4, EXBC_DPSTR4_DATA
	regw EXBC_DPSTR5, EXBC_DPSTR5_DATA
	regw EXBC_TEST,EXBC_TEST_DATA
        mov  pc, lr
 
