/*
 * File: io.h
 *
 * This is an interface to a utility which performs basic IO operations or
 * the parallel and serial ports of the device.
 *
 * See Also
 *   io.c -- An implementation exposing a io.h interface.
 *
 * Copyright (C) 2002 RidgeRun, Inc.
 * Author: RidgeRun, Inc  <skranz@ridgerun.com>
 *  - Support for DSC25 added, 9-6-02, Gregory Nutt
 *  - Support for DM270 added, 2-19-03, Gregory Nutt
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS  PROVIDED  ``AS  IS''  AND   ANY  EXPRESS  OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT,  INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Please report all bugs/problems to the author or <support@dsplinux.net>
 *
 * key: RRGPLCR (do not remove)
 *
 */

#ifndef IO_H
#define IO_H

#define SER 0
#define PAR 1
#define USB 2

#define __swap_16(x)       \
  ((((x) & 0xff00) >> 8) | \
   (((x) & 0x00ff) << 8))

#define __swap_32(x)            \
  ((((x) & 0xff000000) >> 24) | \
   (((x) & 0x00ff0000) >>  8) | \
   (((x) & 0x0000ff00) <<  8) | \
   (((x) & 0x000000ff) << 24))

extern void io_putchar(unsigned char c);

  // --io_putchar--
  // Writes a char to the console. For more
  // information see `man putchar`.

extern void io_delay(int milliseconds);

  // --io_delay--
  // Waits this many milliseconds and then returns to caller.
  // This provides an easy way for the caller to simply stall
  // for the requested number of milliseconds. The implementation
  // of this routine is h/w dependent and is why the io_xxx.c
  // files implement it.

extern int io_AddressIsInFlashSpace(unsigned int addr);

  // --io_AddressIsInFlashSpace--
  // If the passed in address is from the on-board
  // flash area of the board's memory map then returns
  // 1, otherwise returns 0.

extern void io_init(void);

  // --io_init--
  // Called once at bootloader boot to initialize the underlying
  // implementation.

extern void io_change_con(int new_con);

  // --io_change_con--
  // Change the console port.
  //
  //  new_con
  //          SER -- serial port
  //          PAR -- parallel port
  //          USB -- usb port
#endif
