/*
 * File: io_dm270.c
 *
 * An implementation of a h/w io utility specific to the TI DM270 EVM
 * platform. This implementation, while  vendor dependent, exposes the
 * vendor independent io.h interface allowing it to plug generically into
 * the bootloader. Please see io.h for more info.
 * 
 * See Also
 *   io.h
 *
 * Copyright (C) 2002 RidgeRun, Inc.
 * Author: RidgeRun, Inc  <skranz@ridgerun.com>
 *  - Derived from io_dsc21.c, 9/6/02, Gregory Nutt
 *  - Derived from io_dsc25.c, 2/19/03, Gregory Nutt
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS  PROVIDED  ``AS  IS''  AND   ANY  EXPRESS  OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT,  INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Please report all bugs/problems to the author or <support@dsplinux.net>
 *
 * key: RRGPLCR (do not remove)
 *
 */

#include "types.h"
#include "io.h"
#include "util.h"
#include "btldr_pi.h" /* for btldr_AddressIsInFlashSpace */
#include "dm270-registers.h"

#define LOOPS_PER_MSEC 400

/******************************
 Routine:
 Description:
 ******************************/
static unsigned short inreg_ser(unsigned int reg)
{
  unsigned short val;
  val = *((volatile unsigned short *)(UART1_REGISTER_BASE + reg));
  return val;
}

/******************************
 Routine:
 Description:
 ******************************/
static void outreg_ser(unsigned int reg, unsigned short val)
{
  *((volatile unsigned short *)(UART1_REGISTER_BASE + reg)) = val;
}

/******************************
 Routine:
 Description:
******************************/
static void uart_init()
{
    outreg_ser(UART_RFCR, UART_CONST_FIFO_CLEAR);
    outreg_ser(UART_TFCR, UART_CONST_FIFO_CLEAR);
    outreg_ser(UART_LCR, 0x0000);
    outreg_ser(UART_RFCR, UART_TRIGGER_LEVEL_01);
    outreg_ser(UART_TFCR, UART_TRIGGER_LEVEL_16);
    outreg_ser(UART_BRSR, UART_BAUD_115200);
    outreg_ser(UART_MSR, (UART_NOPARITY|UART_STOPBIT_1|UART_DATABIT_8));
}
/******************************
 Routine:
 Description:
 ******************************/
static void io_putchar_ser(unsigned char c)
{
  if ('\n' == c) {
    io_putchar_ser('\r');
  }
  while (0 == (inreg_ser(UART_SR) & UART_TX_FIFO_LEVEL)) {}
  outreg_ser(UART_DTRR, (unsigned short)c);
}

/******************************
 Routine:
 Description:
   Note: See io.h for description.
 ******************************/
void io_putchar(unsigned char c)
{
	io_putchar_ser(c);
}

/******************************
 Routine:
 Description:
   Note: See io.h for description.
 ******************************/
void io_delay(int milliseconds)
{
  volatile int i, j;
  for (i=0; i<milliseconds; i++) {
    for (j=0; j<LOOPS_PER_MSEC; j++) {
    }
  }
}
/******************************
 Routine:
 Description:
   Note: See io.h for description.

 TF: This routine should not be in io.h

 ******************************/

int io_AddressIsInFlashSpace(unsigned int addr)
{
  return( btldr_AddressIsInFlashSpace(addr) );
}
/******************************
 Routine:
 Description:
   Note: See io.h for description.
 ******************************/
void io_init(void)
{
  uart_init();
}
