#include "lcd.h"
#include "c_table.h"

/* Macro Declaration */
#define  inw(p)    (*(volatile unsigned short *)(p))
#define  outw(v,p) (*(volatile unsigned short *)(p) = (v))

#define REG_BITSET0	0x0003058c
#define REG_BITCLR0	0x00030592
#define REG_DIR0	0x00030580
#define LCD_DIR	outw(0x0AFF, REG_DIR0)

#define LCD_CS_HIGH		outw(0x1000,REG_BITSET0)
#define LCD_WR_HIGH		outw(0x2000,REG_BITSET0)
#define LCD_DATA_HIGH	outw(0x4000,REG_BITSET0)
#define LCD_BL_HIGH		outw(0x8000,REG_BITSET0)
#define LCD_CS_LOW		outw(0x1000,REG_BITCLR0)
#define LCD_WR_LOW		outw(0x2000,REG_BITCLR0)
#define LCD_DATA_LOW	outw(0x4000,REG_BITCLR0)
#define LCD_BL_LOW   	outw(0x8000,REG_BITCLR0)

#define DELAY		{ int delay=500; while(delay)  delay--; }

#define MAX_CHARS	12



/* 
 * local proto type 
 */

/* used for sending command , command format 100-code-X */

void
lcd_command (unsigned char code)
{

	int i;

	/* init */

	LCD_CS_HIGH;
	LCD_DATA_HIGH;
	LCD_WR_HIGH;

	/* first issue a CS LOW to indicate going to send command */
	LCD_CS_LOW;
	DELAY;

	/* send command ID 100 */

	LCD_WR_LOW;
	DELAY;
	LCD_DATA_HIGH;		/* 1 */
	DELAY;
	LCD_WR_HIGH;
	DELAY;

	LCD_WR_LOW;
	DELAY;
	LCD_DATA_LOW;		/* 0 */
	DELAY;
	LCD_WR_HIGH;
	DELAY;

	LCD_WR_LOW;
	DELAY;
	LCD_DATA_LOW;		/* 0 */
	DELAY;
	LCD_WR_HIGH;
	DELAY;


	/* send code here */
	for (i = 0; i < 8; i++)
	{
		unsigned char mask = 0x80;

		LCD_WR_LOW;
		DELAY;
		if (code & (mask >> i))
		{
			LCD_DATA_HIGH;
		}
		else
		{
			LCD_DATA_LOW;
		}
		DELAY;
		LCD_WR_HIGH;
		DELAY;
	}

	/* send last don't care bit */
	LCD_WR_LOW;
	DELAY;
	LCD_DATA_LOW;		/* 0 */
	DELAY;
	LCD_WR_HIGH;
	DELAY;

	/* restore to indicate command finish */

	LCD_DATA_HIGH;
	LCD_WR_HIGH;
	LCD_CS_HIGH;
}



/* used for writing data, data format 101-addr-data */

void
lcd_data (unsigned char addr, unsigned char data)
{

	int i;

	/* init */

	LCD_CS_HIGH;
	DELAY;
	LCD_DATA_HIGH;
	LCD_WR_HIGH;

	/* first issue a CS LOW to indicate going to write */
	LCD_CS_LOW;
	DELAY;

	/* write ID 101 */

	LCD_WR_LOW;
	DELAY;
	LCD_DATA_HIGH;		/* 1 */
	DELAY;
	LCD_WR_HIGH;
	DELAY;

	LCD_WR_LOW;
	DELAY;
	LCD_DATA_LOW;		/* 0 */
	DELAY;
	LCD_WR_HIGH;
	DELAY;

	LCD_WR_LOW;
	DELAY;
	LCD_DATA_HIGH;		/* 1 */
	DELAY;
	LCD_WR_HIGH;
	DELAY;


	/* write addr here */
	for (i = 0; i < 6; i++)
	{
		unsigned char mask = 0x20;

		LCD_WR_LOW;
		DELAY;
		if (addr & (mask >> i))
		{
			LCD_DATA_HIGH;
		}
		else
		{
			LCD_DATA_LOW;
		}
		DELAY;
		LCD_WR_HIGH;
		DELAY;
	}

	/* write data here */
	for (i = 0; i < 4; i++)
	{
		unsigned char mask = 0x08;
		LCD_WR_LOW;
		DELAY;
		if (data & (mask >> i))
		{
			LCD_DATA_HIGH;
		}
		else
		{
			LCD_DATA_LOW;
		}
		DELAY;
		LCD_WR_HIGH;
		DELAY;
	}


	/* restore to indicate command finish */

	LCD_DATA_HIGH;
	LCD_WR_HIGH;
	LCD_CS_HIGH;
}

void
lcd_poweron (void)
{
	/* initialization */
	lcd_command (CMD_LCDON);
	lcd_command (CMD_SYSEN);
	lcd_command (CMD_BIAS13);
}

void
lcd_showall (void)
{
	char c;

	for (c = 0x00; c < 0x19; c++)
		lcd_data (c, 0x0f);
}

void
lcd_poweroff (void)
{
	LCD_DIR;	
	lcd_clearall ();
	lcd_command (CMD_LCDOFF);
	lcd_command (CMD_SYSDIS);
}

void
lcd_clearall (void)
{
	char c;

	LCD_DIR;	
	for (c = 0x00; c < 0x19; c++)
		lcd_data (c, 0x00);
}



