/* asm/arch/memconfig.h */

#ifndef __ASM_ARCH_MEMCONFIG_H_
#define __ASM_ARCH_MEMCONFIG_H_

/*************************************************
 *                   FLASH
 *   0010:0000 +----------------+ (0000:0000)
 *             |     Loader     |     (  32K)
 *   0010:8000 +----------------+ (0000:8000)
 *             |  MAC Address   |     (   8K)
 *   0010:A000 +----------------+ (0000:A000)
 *             |  Config Data   |     (   8K)
 *   0010:C000 +----------------+ (0000:C000)
 *             |    Reserved    |     (  16K)
 *   0011:0000 +----------------+ (0001:0000)
 *             | Linux Kernel   |     ( 896K)
 *             |                |
 *   001F:0000 +----------------+ (000F:0000)
 *             | File System    |     (3136K)
 *             |                |
 *             |                |
 *   0050:0000 +----------------+ (0040:0000)
 *************************************************/

#define BSPCONF_FLASH_BASE 0x00100000
#define FLASH_BASE BSPCONF_FLASH_BASE
#define BSPCONF_FLASH_SIZE 0x00400000
#define FLASH_SIZE BSPCONF_FLASH_SIZE
#define BSPCONF_SDRAM_BASE 0x00900000

/* FLASH Memory Map */
#define BSPCONF_BTLDR_FLASH_OFFSET 0x00000000
#define BSPCONF_BTLDR_FLASH_SIZE 0x00008000

#define BSPCONF_KERNEL_FLASH_OFFSET 0x00010000
#define BSPCONF_KERNEL_FLASH_SIZE 0x000e0000

#define BSPCONF_FS_FLASH_OFFSET 0x000f0000
#define BSPCONF_FS_FLASH_SIZE 0x00310000

/* Download Program */
#define EMPTY_SDRAM_START   (BSPCONF_SDRAM_BASE+BSPCONF_FLASH_SIZE)
#define FLASH_MAC_OFFSET    0x00008000
#define FLASH_MAC_ADDR      (FLASH_BASE+FLASH_MAC_OFFSET)
#define BTLDR_FLASH_SIZE    0x00008000

/* Load Code to SDRAM (btldr_pi.c) */
#define KERNEL_LOAD_ADDR	0x01310000
#define KERNEL_ENTRY_ADDR	0x01319000
#define KERNEL_NUM_BYTES	BSPCONF_KERNEL_FLASH_SIZE	/* 0x000e0000 */

#define FILESYS_LOAD_ADDR	0x01000000
#define FILESYS_ENTRY_ADDR	0xffffffff
#define FILESYS_NUM_BYTES	BSPCONF_FS_FLASH_SIZE		/* 0x00310000 */

/*************************************************
 *                   SDRAM
 *   0090:0000 +----------------+ (0000:0000)
 *   	       | (video buffer) |	
 *             |     Loader     |
 *   0100:0000 +----------------+ (0070:0000)
 *             | File System    |
 *   0131:0000 +----------------+ (00A1:0000)
 *             | Linux Kernel   |
 *             |                |
 *             |                |
 *   0290:0000 +----------------+ (0200:0000)
 *************************************************/

#endif /* __ASM_ARCH_MEMCONFIG_H_ */
