//
// File: sdram_dsc24.S
//
// (Derived from head-arm-atmel.S and Boot.asm)
//
// Contains the necessary sdram h/w setup specific to the TI DSC25 EVM
// board. This is placed in its own file so that customers can create their
// own impl and relink with the shipped rrload *.o files forming a new
// rrload appropriate for any non-standard SDRAM components they are using.
// See rrload user manual for more info on customizing drivers.
//
// Copyright (C) 2002 RidgeRun, Inc.
// Author: RidgeRun, Inc  <skranz@ridgerun.com>
//
//  This program is free software; you can redistribute  it and/or modify it
//  under  the terms of  the GNU General  Public License as published by the
//  Free Software Foundation;  either version 2 of the  License, or (at your
//  option) any later version.
//
//  THIS  SOFTWARE  IS  PROVIDED  ``AS  IS''  AND   ANY  EXPRESS  OR IMPLIED
//  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
//  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT,  INDIRECT,
//  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
//  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
//  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
//  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//  You should have received a copy of the  GNU General Public License along
//  with this program; if not, write  to the Free Software Foundation, Inc.,
//  675 Mass Ave, Cambridge, MA 02139, USA.
//
// Please report all bugs/problems to the author or <support@dsplinux.net>
//
// key: RRGPLCR (do not remove)
//

        .text
        .align
        .global sdram_ini

// **********************
// Macro regw(add,data)
// **********************
        .macro regw, reg_add, reg_data
        ldr r0, \reg_add
        ldr r1, \reg_data
        strh r1, [r0]       
        .endm

// *******************************************************
// SDRAM initialize
// *******************************************************
sdram_ini:
	regw	EMIF_DPSTR0, DPSTR0_DATA
	regw	EMIF_DPSTR1, DPSTR1_DATA
	regw	EMIF_DPSTR2, DPSTR2_DATA
	regw	EMIF_DPSTR3, DPSTR3_DATA
	regw	EMIF_DPSTR4, DPSTR4_DATA
	regw	EMIF_DPSTR5, DPSTR5_DATA
	
	regw	SDRAMC_SDMODE,   SDMODE_DATA0
	regw	SDRAMC_REFCTL, REFCTL_DATA

	// SDRAM commands to initialize SDRAM
	// REG SDMODE = nMode | PREA   // precharge
	// for (i = 0; i < 8; i++) {
	//     REG SDMODE = nMode | REF;
        // }
        // REG SDMODE = nMode | REF; // auto refresh 8 times
           
           ldr     r0, SDRAMC_SDMODE
           ldr     r1, SDMODE_DATA1
           strh    r1, [r0]
           ldr     r1, SDMODE_DATA2
    // Repeat 8 times
           strh    r1, [r0]
           strh    r1, [r0]
           strh    r1, [r0]
           strh    r1, [r0]
           strh    r1, [r0]
           strh    r1, [r0]
           strh    r1, [r0]
           strh    r1, [r0]

           ldr     r1, SDMODE_DATA3
           strh    r1, [r0]

    // This fixes a rev1 SDRAM refresh problem
           regw    SDRAMC_SDPRTY3, SDPTRY3_DATA
           regw    SDRAMC_SDPRTY4, SDPTRY4_DATA
           regw    SDRAMC_SDPRTY5, SDPTRY5_DATA
           regw    SDRAMC_SDPRTY6, SDPTRY6_DATA
           regw    SDRAMC_SDPRTY9, SDPTRY9_DATA
           regw    SDRAMC_SDPRTY10, SDPTRY10_DATA
           regw    SDRAMC_PRTYON, PRTYON_DATA

           mov	pc, lr			@ return
		
// **********************************************************
// Register and data definitions
// **********************************************************
SDRAMC_SDMODE:	 .long  0x309A6
SDRAMC_REFCTL:	 .long  0x309A8

SDRAMC_SDPRTY3:	 .long  0x309AE		// HW 3A
SDRAMC_SDPRTY4:	 .long  0x309B0		// OSD
SDRAMC_SDPRTY5:	 .long  0x309B2		// Ext HOST
SDRAMC_SDPRTY6:	 .long  0x309B4		// ARM CPU
SDRAMC_SDPRTY9:	 .long  0x309BA		// DSP CPU
SDRAMC_SDPRTY10: .long  0x309BC		// SDRAM Refresh
SDRAMC_PRTYON:	 .long  0x309BE

SDPTRY3_DATA:	 .word  0x0040
SDPTRY4_DATA:	 .word  0x0020
SDPTRY5_DATA:	 .word  0x0010
SDPTRY6_DATA:	 .word  0x0080
SDPTRY9_DATA:	 .word  0x0001
SDPTRY10_DATA:	 .word  0x0002
PRTYON_DATA:     .word  0x0001
				
// NEC uPD4564163G5 setting
		.set	TRD_LAT,    1	// tRD latency 0:1 cycle, 1:2 cycle, 
		.set	CAS_LAT,    0	// cas latency 0:2 cycle, 1:3 cycle, 
		.set	BANK_NUM,   1	// back number 0:2 bank, 1:4 bank
		.set	MEM_TYPE,   2   // memory type 0:2k*256, 1:4k*256,
				        // 2:4K*512, 3:8k*512
		.set    TRCD,       1   // Fixes rev1 bug, but slows down SDRAM
//		.set    TRCD,       0   // use this eventually
		.set	SDR16,	    0	// memory width 0:32-bit 1:16-bit

		.set	SDMA_1,	    1	// DMA Channel(1/2): 0:ExtMem 1:Serial
		.set	SDMA_2,	    0	// 2:USB 3:MMC/SD 4: DSP(HPIB)
		.set	REF_EN,	    1	// refresh enable 0:disable, 1:enable
		.set	REF_CYCLE,  0x9D // 0x40  // refresh cycle (REFNUM)


		
// Command codes to SDRAM controller
		
		.set	NOSEL,	0x00	// no action
		.set	MRS,	0x01	// mode register set
		.set	PREA,	0x02	// precharge all
		.set	REF,	0x04	// auto refresh
		.set	SELF,	0x08	// self refresh on
		.set	SELFC,	0x10	// self refresh clear
		.set	PDN,	0x20	// power down
		
		.set	SDMODE_NMODE,	((TRCD << 13) | (CAS_LAT << 10) | (BANK_NUM << 12) | (MEM_TYPE << 8) | (SDR16 <<14) | (TRD_LAT << 15))


SDMODE_DATA0:		.word	SDMODE_NMODE
SDMODE_DATA1:		.word	(SDMODE_NMODE | PREA)
SDMODE_DATA2:		.word	(SDMODE_NMODE | REF)
SDMODE_DATA3:		.word	(SDMODE_NMODE | MRS)
REFCTL_DATA:		.word	((SDMA_1 <<13) | (SDMA_2 << 10) | (REF_EN << 8) | REF_CYCLE)

//*** For 32MB SDRAM, need to grow SDRAM region and move CF region ***
EMIF_DPSTR0:     .word   0x30a4a
EMIF_DPSTR1:     .word   0x30a4c
EMIF_DPSTR2:     .word   0x30a4e
EMIF_DPSTR3:     .word   0x30a50
EMIF_DPSTR4:     .word   0x30a52
EMIF_DPSTR5:     .word   0x30a54

// region 0 (Flash):  end address = 0x0090 0000 (default)
DPSTR0_DATA:    .word   0x09            // end address = 0x0090 0000
// region 1 (SDRAM):  start address = end address of region 0
DPSTR1_DATA:    .word   0x49            // end address = 0x0490 0000
// region 2 (CF):     start address = end address of region 1
DPSTR2_DATA:    .word   0x59            // end address = 0x0590 0000
// region 3 (Network):     start address = end address of region 2
DPSTR3_DATA:    .word   0x69            // end address = 0x0690 0000
// region 4 (CF):     start address = end address of region 3
DPSTR4_DATA:    .word   0x79            // end address = 0x0790 0000
DPSTR5_DATA:    .word   0x81            // end address = 0x0810 0000
