/*
 * File: util.c
 *
 * An implementation of a utility which exposes the util.h interface. Please
 * see util.h for more info.
 *
 * See Also
 *   util.h
 *
 * Copyright (C) 2002 RidgeRun, Inc.
 * Author: RidgeRun, Inc  <skranz@ridgerun.com>
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS  PROVIDED  ``AS  IS''  AND   ANY  EXPRESS  OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT,  INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Please report all bugs/problems to the author or <support@dsplinux.net>
 *
 * key: RRGPLCR (do not remove)
 *
 */

#include "memconfig.h"
#include "types.h"
#include "util.h"
#include "io.h"

static const unsigned char hextable[256] = {
  255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
  255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
  255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
    0,   1,   2,   3,   4,   5,   6,   7,   8,   9, 255, 255, 255, 255, 255, 255,
  255,  10,  11,  12,  13,  14,  15, 255, 255, 255, 255, 255, 255, 255, 255, 255,
  255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
  255,  10,  11,  12,  13,  14,  15, 255, 255, 255, 255, 255, 255, 255, 255, 255,
  255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
  255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
  255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
  255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
  255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
  255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
  255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
  255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
  255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
};

static char *do_sprintf(char *buf, const char *fmt, ...);

/******************************
 Routine:
 Description:
  Format codes
    %d - decimal
    %i - decimal
    %b - 1 byte val (2 ascii hex digits)
    %x - 2 byte val (4 ascii hex digits)
    %X - 4 byte val (8 ascii hex digits)
    %s - An ASCII string
    %c - An ASCII char
    %I - Internet address (x.x.x.x)
******************************/
static const char hex[]="0123456789ABCDEF";
static char *do_printf(char *buf, const char *fmt, const int *dp)
{
  register char *p;
  char tmp[16];
  while (*fmt) {
    if (*fmt == '%') {  // switch() uses more space
      fmt++;
      if (*fmt == 'X') {
        const long *lp = (const long *)dp;
        register long h = *lp++;
        dp = (const int *)lp;
        *(buf++) = hex[(h>>28)& 0x0F];
        *(buf++) = hex[(h>>24)& 0x0F];
        *(buf++) = hex[(h>>20)& 0x0F];
        *(buf++) = hex[(h>>16)& 0x0F];
        *(buf++) = hex[(h>>12)& 0x0F];
        *(buf++) = hex[(h>>8)& 0x0F];
        *(buf++) = hex[(h>>4)& 0x0F];
        *(buf++) = hex[h& 0x0F];
      }
      if (*fmt == 'x') {
              register int h = *(dp++);
        *(buf++) = hex[(h>>12)& 0x0F];
        *(buf++) = hex[(h>>8)& 0x0F];
        *(buf++) = hex[(h>>4)& 0x0F];
        *(buf++) = hex[h& 0x0F];
      }
      if (*fmt == 'b') {
        register int h = *(dp++);
        *(buf++) = hex[(h>>4)& 0x0F];
        *(buf++) = hex[h& 0x0F];
      }
      if ((*fmt == 'd') || (*fmt == 'i')) {
        register int dec = *(dp++);
        p = tmp;
        if (dec < 0) {
          *(buf++) = '-';
          dec = -dec;
        }
        do {
          *(p++) = '0' + (dec%10);
          dec = dec/10;
        } while(dec);
        while ((--p) >= tmp) *(buf++) = *p;
      }
      if (*fmt == 'I') {
        union {
          long l;
          unsigned char c[4];
        } u;
        const long *lp = (const long *)dp;
        u.l = *lp++;
        dp = (const int *)lp;
        buf = do_sprintf(buf,"%d.%d.%d.%d",
                         u.c[0], u.c[1], u.c[2], u.c[3]);
      }
      if (*fmt == 'c') {
        *(buf++) = *(dp++);
      }
      if (*fmt == 's') {
        p = (char *)*dp++;
        while (*p) *(buf++) = *p++;
      }
    } else {
      *(buf++) = *fmt;
    }
    fmt++;
  }
  *buf = 0;
  return(buf);
}

/******************************
 Routine:
 Description:
 ******************************/
static char *do_sprintf(char *buf, const char *fmt, ...)
{
  return do_printf(buf, fmt, ((const int *)&fmt)+1);
}

/******************************
 Routine:
 Description:
   Note: See util.h for description.
 ******************************/
void util_printf(const char *fmt, ...)
{
  char buf[256],*p;
  p = buf;
  do_printf(buf, fmt, ((const int *)&fmt)+1);
  while (*p) io_putchar(*p++);
}

void u_memcpy(char * dest, const char *src, int cnt)
{
  if (cnt <= 0)
    return;
  while (cnt--)
    *dest++ = *src++;
}

void u_memset(char * dest, char c, int cnt)
{
  if (cnt <= 0)
    return;
  while (cnt--)
    *dest++ = c;
}

