/***********************************************************************
 * File:  xflat-user.ld     
 *
 * Created 2002, Copyright (C) 2002 Cadenux, LLC.
 * Author: Gregory Nutt <greg.nutt@cadenux.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Please report all bugs/problems to the author or <info@cadenux.com>
 ***********************************************************************/

MEMORY 
{
  a5world : ORIGIN = 0x0, LENGTH = 2097152
}

/***********************************************************************
 * The XFLAT program image is divided into two segments:
 *
 * (1) ISpace (Instruction Space).  This is the segment that contains
 *     code (.text).  Everything in the segment should be fetch-able
 *     machine PC instructions (jump, branch, call, etc.).
 * (2) DSpace (Data Space).  This is the segment that contains both
 *     read-write data (.data, .bss) as well as read-only data (.rodata).
 *     Everything in this segment should be access-able with machine
 *     with machine load and store instructions.
 ***********************************************************************/

SECTIONS
{
  .text 0x00000000 :
    {
      /* ISpace is located at address 0.  Every (unrelocated) ISpace
       * address is an offset from the begining of this segment.
       */

      text_start = . ;
      *(.text)
      *(.glue_7)
      *(.glue_7t)
      *(.init)
      *(.fini)
      *(.gnu.linkonce.t.*)
      _etext = . ;

    } > a5world

    /* DSpace is also located at address 0.  Every (unrelocated) DSpace
     * address is an offset from the begining of this segment.
     */

  .data 0x00000000 :
    {
      __data_start = . ;
      *(.rodata)
      *(.data)
      *(__libc_subinit)
      *(__libc_subfreeres)
      *(__libc_atexit)
      *(.gnu.linkonce.d.*)
      *(.ctors)
      *(.dtors)
      *(.note.ABI-tag)
      _edata = . ;
      edata = ALIGN( 0x10 ) ;
    } > a5world+_etext
  .bss :
    {
      __bss_start = _edata ;
      *(.bss)
      *(COMMON)
      end = ALIGN( 0x10 ) ;
      _end = ALIGN( 0x10 ) ;
    } > a5world+_etext
}
