/************************************************************************
 * linux/asm-armnommu/arch-dm270/gio_ioctl.h
 *
 *   Copyright (C) 2003 Cadenux, LLC (http://www.cadenux.com)
 *   author:  Gregory Nutt <greg.nutt@cadenux.com>
 *
 * Derived from linux/asm-armnommu/arch-dsc25/gio_ioctl.h
 *
 *   Copyright (C) 2002 Cadenux, LLC (http://www.cadenux.com)
 *   author:  Gregory Nutt <greg.nutt@cadenux.com>
 ************************************************************************/

#ifndef __ASM_ARCH_GIO_IOCTL_H__
#define __ASM_ARCH_GIO_IOCTL_H__
 
/************************************************************************
 * Included Files
 ************************************************************************/

#include <asm/ioctl.h>
 
/************************************************************************
 * Definitions
 ************************************************************************/

#define GIO_MAGIC_NUMBER 'g'

/* Major numbers, (from Documentation/devices.txt local/experimental list)  */
 
#define GIO_MAJOR 125
 
/************************************************************************
 * GIO_DIRECTION: The GIO_DIRECTION can be either input (GIO_INPUT) or
 * output (GIO_OUTPUT).  If unspecified, the GIO will default to input.
 */

#define GIO_DIRECTION _IO(GIO_MAGIC_NUMBER, 0)

/* GIO_DIRECTION Arguments: */

#define GIO_OUTPUT 0
#define GIO_INPUT  1

/************************************************************************
 * GIO_INVERSION: The GIO polarity of the GIO pin can be inverted.  If
 * unspecified, the default behavior will be NOT inverted.
 */

#define GIO_INVERSION _IO(GIO_MAGIC_NUMBER, 1)

/* GIO_INVERSION Arguments: */

#define GIO_UNINVERTED 0
#define GIO_INVERTED   1

/************************************************************************
 * GIO_BITSET: If the direction is set to output, this ioctl will set the
 * output.  If the direction is input, this ioctl will return the value
 * of the GIO pin (0, 1, or -1 on error).
 */

#define GIO_BITSET _IO(GIO_MAGIC_NUMBER, 2)

/* GIO_BITSET arguments:  None */

/************************************************************************
 * GIO_BITCLR: If the direction is set to output, this ioctl will clear the
 * output.  If the direction is input, this ioctl will return the value
 * of the GIO pin (0, 1, or -1 on error).
 */

#define GIO_BITCLR _IO(GIO_MAGIC_NUMBER, 3)

/* GIO_BITCLR arguments:  None */

/************************************************************************
 * GIO_IRQPORT: For the DM270, GIO0-GIO15 can be configured as external
 * interrupt ports
 */

#define GIO_IRQPORT _IO(GIO_MAGIC_NUMBER, 4)

/* GIO_IRQPORT arguments: */

#define GIO_NOINTERRUPT 0 /* Turn off interrupts */
#define GIO_INTERRUPT   1 /* Enable interrupt with one-side (falling)
			   * edge detection (unless the input is inverted,
			   * then rising edge). */

/* These arguments are unique to the DM270 */

#define GIO_INTERRUPT1  GIO_INTERRUPT
#define GIO_INTERRUPT2  2 /* Enable interrupt with both-side edge
			   * detection. Interruption occurs in both
			   * rising and falling edges. */

/************************************************************************
 * GIO_FSEL: For the DM270 GIO8-GIO33 can be used as dedicated function
 * ports rather than general purpose IO ports.
 */

#define GIO_FSEL _IO(GIO_MAGIC_NUMBER, 5)

/* GIO_FSEL Arguments: */

#define GIO_NORMAL       0 /* 0=GIO8-GIO33 Normal */

/* McBSP 1 has its pins multiplexed tihe ARM GIO8 through GIO15 */

#define GIO_DSP_BFSR1    1 /* 0=GIO8  Normal, 1=GIO8 DSP_BFSR1 */
#define GIO_DSP_BCLKR1   1 /* 0=GIO9  Normal, 1=GIO9 DSP_BCLKR1 */
#define GIO_DSP_BDR1     1 /* 0=GIO10 Normal, 1=GIO10 DSP_BDR1 */
#define GIO_DSP_BFSX1    1 /* 0=GIO13 Normal, 1=GIO11 DSP_BFSX1 */
#define GIO_DSP_BCLKX1   1 /* 0=GIO12 Normal, 1=GIO12 DSP_BCLKX1 */
#define GIO_DSP_BDX1     1 /* 0=GIO13 Normal, 1=GIO13 DSP_BDX1 */
#define GIO_DSP_CLKS1    1 /* 0=GIO14 Normal, 1=GIO14 DSP_CLKS1 */

/* XF pins multiplexed with GIO 15 */

#define GIO_DSP_XF     1 /* 0=GIO15 Normal, 1=GIO15 DSP_XF */

/* Provisions for setting clock rates in GIO16-GIO18.
 * CLKOUT can output clock generated by OSEL and O*DIV registers in
 * general clock output.
 */

#define GIO_FSEL_CLKOUT0 1 /* 0=GIO16 Normal, 1=GIO16 CKLOUT0 */
#define GIO_FSEL_CLKOUT1 1 /* 0=GIO17 Normal, 1=GIO17 CKLOUT1 */
#define GIO_FSEL_CLKOUT2 1 /* 0=GIO18 Normal, 1=GIO18 CKLOUT2 */

#define GIO_SDO1         1 /* 0=GIO19 Normal, 1=GIO19 SDO1 */
#define GIO_SDI1         1 /* 0=GIO20 Normal, 1=GIO20 SDI1 */
#define GIO_SCLK1        1 /* 0=GIO21 Normal, 1=GIO21 SCLK1 */
#define GIO_SDEN1        1 /* 0=GIO22 Normal, 1=GIO22 SDEN1 */
#define GIO_RXD1         1 /* 0=GIO23 Normal, 1=GIO23 RXD1 */

#define GIO_TXD1         1 /* 0=GIO24 Normal, 1=GIO24 TXD1 */
#define GIO_MIRQ         1 /* 0=GIO25 Normal, 1=GIO25 MIRQ */
#define GIO_MRST         2 /* 0=GIO25 Normal, 2=GIO25 MRST */
#define GIO_C_WEN        1 /* 0=GIO26 Normal, 1=GIO26 C_WEN */

/* GIO27 is connected to the shutter */

#define GIO_CCDC_SHUT     1 /* 0=GIO27 Normal, 1=GIO27 CCDC_SHUT */
#define GIO_FIELD         1 /* 0=GIO28 Normal, 1=GIO28 FIELD */
#define GIO_PWM0          1 /* 0=GIO29 Normal, 1=GIO29 PWM0 */
#define GIO_PWM1          1 /* 0=GIO30 Normal, 1=GIO30 PWM1 */
#define GIO_SCL           1 /* 0=GIO31 Normal, 1=GIO31 SCL */
#define GIO_SDA           1 /* 0=GIO32 Normal, 1=GIO32 SDA */
#define GIO_EXTLINE_ID    1 /* 0=GIO33 Normal, 1=GIO33 EXTLINE_ID */

/************************************************************************
 * GIO_BITRATE: Forward/Transfer rate set bit.
 * The DM270 does not support bit rate registers.  This IOCTL command
 * is defined only for compatibility with the DSC25 IOCTL.  The driver
 * will always return -ENOSYS in response to this command.
 */

#define GIO_BITRATE _IO(GIO_MAGIC_NUMBER, 6)

#define GIO_IOC_MAXNR 6
 
#endif /* __ASM_ARCH_GIO_IOCTL_H__ */
