/*
 * linux/include/asm-nommu/arch-dm270/hardware.h
 * Definitions for the DM270.
 *
 *   Copyright (C) 2003 Cadenux, LLC (http://www.cadenux.com)
 *   author:  Gregory Nutt <greg.nutt@cadenux.com>
 *
 * Derived from linux/include/asm-nommu/arch-dsc25/hardware.h
 *
 *   Copyright (C) 2002 Cadenux, LLC (http://www.cadenux.com)
 *   author:  Gregory Nutt <greg.nutt@cadenux.com>
 *
 * Modified from linux/include/asm-nommu/arch-dsc21/hardware.h
 *    
 *  Copyright (C) 2001 RidgeRun, Inc. (http://www.ridgerun.com)
 *  author: Gordon McNutt <gmcnutt@ridgerun.com>
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __ASM_ARCH_HARDWARE_H
#define __ASM_ARCH_HARDWARE_H

#define KERNTOPHYS(a) ((unsigned long)(&a))

/* ARM clock rate (MHz) */

/* #define DM270_CLOCK_RATE     37125000      37.125 MHz */
#define DM270_CLOCK_RATE        67500000   /* 67.500 MHz */

/* Memory mapped registers */

#define TIMER0_REGISTER_BASE    0x00030000 /* Timer0 */
#define TIMER1_REGISTER_BASE	0x00030080 /* Timer1 */
#define TIMER2_REGISTER_BASE	0x00030100 /* Timer2 */
#define TIMER3_REGISTER_BASE	0x00030180 /* Timer3 */
#define SERIAL0_REGISTER_BASE	0x00030200 /* Serial0 */
#define SERIAL1_REGISTER_BASE	0x00030280 /* Serial1 */
#define UART0_REGISTER_BASE	0x00030300 /* UART0 */
#define UART1_REGISTER_BASE	0x00030380 /* UART1 */
#define WDT_REGISTER_BASE	0x00030400 /* Watch dog timer */
#define MMCSD_REGISTER_BASE     0x00030480 /* MMC/SD */
#define INTCTRL_REGISTER_BASE	0x00030500 /* INTC */
#define GIO_REGISTER_BASE	0x00030580 /* GIO */
#define HPIB_REGISTER_BASE      0x00030600 /* DSP Controller */
#define OSD_REGISTER_BASE	0x00030680 /* OSD */
#define CCDC_REGISTER_BASE	0x00030700 /* CCD controller */
#define PREVIEW_REGISTER_BASE	0x00030780 /* Preview engine */
#define VIDEOENC_REGISTER_BASE	0x00030800 /* NTSC/PAL video encoder */
#define CLOCKC_REGISTER_BASE	0x00030880 /* Clock controller */
#define BUSC_REGISTER_BASE	0x00030900 /* Bus controller */
#define SDRAMC_REGISTER_BASE	0x00030980 /* SDRAM controller */
#define EXBC_REGISTER_BASE	0x00030a00 /* External Memory Interface */
#define USB0_REGISTER_BASE      0x00030a80 /* USB 0 */
#define USB1_REGISTER_BASE      0x00030b00 /* USB 1 */
#define AF_REGISTER_BASE        0x00030b80 /* 3A(AF/AE/AWB) */
#define MSTICK1_REGISTER_BASE   0x00030c00 /* Memory Stick 1 */
#define MSTICK2_REGISTER_BASE   0x00030c80 /* Memory Stick 2 */
#define ATM_REGISTER_BASE       0x00030d00 /* ATM */
#define I2C_REGISTER_BASE       0x00030d80 /* I2C */
#define ICACHE_REGISTER_BASE    0x00030e00 /* Instruction cache */

/* Timers */

#define DM270_TIMER0_MODE    0x00030000
#define DM270_TIMER0_SEL     0x00030002
#define DM270_TIMER0_SCAL    0x00030004
#define DM270_TIMER0_DIV     0x00030006
#define DM270_TIMER0_TRG     0x00030008
#define DM270_TIMER0_CNT     0x0003000a
#define UCLINUX_TIMER0_CNT   DM270_TIMER0_CNT

#define DM270_TIMER1_MODE    0x00030080
#define DM270_TIMER1_SEL     0x00030082
#define DM270_TIMER1_SCAL    0x00030084
#define DM270_TIMER1_DIV     0x00030086
#define DM270_TIMER1_TRG     0x00030088
#define DM270_TIMER1_CNT     0x0003008a

#define DM270_TIMER2_MODE    0x00030100
#define DM270_TIMER2_SEL     0x00030102
#define DM270_TIMER2_SCAL    0x00030104
#define DM270_TIMER2_DIV     0x00030106
#define DM270_TIMER2_TRG     0x00030108
#define DM270_TIMER2_CNT     0x0003010a

#define DM270_TIMER3_MODE    0x00030180
#define DM270_TIMER3_SEL     0x00030182
#define DM270_TIMER3_SCAL    0x00030184
#define DM270_TIMER3_DIV     0x00030186
#define DM270_TIMER3_TRG     0x00030188
#define DM270_TIMER3_CNT     0x0003018a

/* Uarts (note that the register fields aren't needed by the standard linux
 * serial driver -- we only use them for our initial debugging console)
 */

#define DM270_UART0          0x00030300
#define DM270_UART0_DTRR     0x00030300
#define DM270_UART0_BRSR     0x00030302
#define DM270_UART0_MSR      0x00030304
#define DM270_UART0_RFCR     0x00030306
#define DM270_UART0_TFCR     0x00030308
#define DM270_UART0_LCR      0x0003030a
#define DM270_UART0_SR       0x0003030c

#define DM270_UART1          0x00030380
#define DM270_UART1_DTRR     0x00030380
#define DM270_UART1_BRSR     0x00030382
#define DM270_UART1_MSR      0x00030384
#define DM270_UART1_RFCR     0x00030386
#define DM270_UART1_TFCR     0x00030388
#define DM270_UART1_LCR	     0x0003038a
#define DM270_UART1_SR 	     0x0003038c

/* Watchdog */

#define DM270_WDTMD          0x00030400 /* WD timer mode */
#define DM270_WDTRST         0x00030402 /* WD timer reset */
#define DM270_WDTPRSCL       0x00030404 /* WD timer pre-scaler value */
#define DM270_WDTDIV         0x00030406 /* WD timer count value */
#define DM270_WDTEXRST       0x00030408 /* External reset control */

#define WDT_BLOCK_SIZE       0x0000000c

/* MMC/SD Card Interface */

#define DM270_MMCSD_CTL      0x00030480 /* MMC Control Register */
#define DM270_MMCSD_CLK      0x00030482 /* MMC Memory Clock Register */
#define DM270_MMCSD_ST0      0x00030484 /* MMC Status Register #0 */
#define DM270_MMCSD_ST1      0x00030486 /* MMC Status Register #1 */
#define DM270_MMCSD_IM       0x00030488 /* MMC Interrupt Enable Register */
#define DM270_MMCSD_TOR      0x0003048a /* MMC Response Time-Out Register */
#define DM270_MMCSD_TOD      0x0003048c /* MMC Data Read Time-Out Register */
#define DM270_MMCSD_BLEN     0x0003048e /* MMC Block Length Register */
#define DM270_MMCSD_NBLK     0x00030490 /* MMC Number of Blocks Register */
#define DM270_MMCSD_NBLC     0x00030492 /* MMC Number of Blocks Counter */
#define DM270_MMCSD_DRR      0x00030494 /* MMC Data Receive Register */
#define DM270_MMCSD_DXR      0x00030496 /* MMC Data Transmit Register */
#define DM270_MMCSD_CMD      0x00030498 /* MMC Command Register */
#define DM270_MMCSD_ARGL     0x0003049a /* MMC Argument - Low Register */
#define DM270_MMCSD_ARGH     0x0003049c /* MMC Argument - High Register */
#define DM270_MMCSD_RSP0     0x0003049e /* MMC Response Register #0 */
#define DM270_MMCSD_RSP1     0x000304a0 /* MMC Response Register #1 */
#define DM270_MMCSD_RSP2     0x000304a2 /* MMC Response Register #2 */
#define DM270_MMCSD_RSP3     0x000304a4 /* MMC Response Register #3 */
#define DM270_MMCSD_RSP4     0x000304a6 /* MMC Response Register #4 */
#define DM270_MMCSD_RSP5     0x000304a8 /* MMC Response Register #5 */
#define DM270_MMCSD_RSP6     0x000304aa /* MMC Response Register #6 */
#define DM270_MMCSD_RSP7     0x000304ac /* MMC Response Register #7 */
#define DM270_MMCSD_DRSP     0x000304ae /* MMC Data Response Register */
#define DM270_MMCSD_ETOK     0x000304b0 /* MMC SPI Error Token Register */
#define DM270_MMCSD_CIDX     0x000304b2 /* MMC Command Index Register */
#define DM270_MMCSD_CKC      0x000304b4 /* Test Register */
#define DM270_MMCSD_TORC     0x000304b6 /* Test Register */
#define DM270_MMCSD_TODC     0x000304b8 /* Test Register */
#define DM270_MMCSD_BLNC     0x000304ba /* Test Register */
#define DM270_MMCSD_DMATRG   0x000304bc /* DMA Trigger Register */
#define DM270_MMCSD_DMAMODE  0x000304be /* DMA Mode Register */
#define DM270_MMCSD_DMASTAD0 0x000304c0 /* DMA Address Low */
#define DM270_MMCSD_DMASTAD1 0x000304c2 /* DMA Address High */
#define DM270_MMCSD_DMASTAT0 0x000304c4 /* DMA Status Register #0 */
#define DM270_MMCSD_DMASTAT1 0x000304c6 /* DMA Status Register #1 */
#define DM270_MMCSD_DMATOR   0x000304c8 /* DMA Time-Out Register */

/* Registers specific to SDIO card */

#define DM270_MMCSD_SDIOCTL  0x000304ca
#define DM270_MMCSD_SDIOST0  0x000304cc
#define DM270_MMCSD_SDIOIEN  0x000304ce
#define DM270_MMCSD_SDIOIST  0x000304d0

#define MMCSD_BLOCK_SIZE     0x00000052

/* Interrupt */

#define DM270_FIQ0_STATUS    0x00030500 /* FIQs  0-15 */
#define DM270_FIQ1_STATUS    0x00030502 /* FIQs 16-31 */
#define DM270_FIQ2_STATUS    0x00030504 /* FIQs 32-39 */
#define DM270_IRQ0_STATUS    0x00030508 /* IRQs  0-15 */
#define DM270_IRQ1_STATUS    0x0003050a /* IRQs 16-31 */
#define DM270_IRQ2_STATUS    0x0003050c /* IRQs 32-39 */
#define DM270_FIQ0_ENTRY     0x00030510
#define DM270_FIQ1_ENTRY     0x00030512
#define DM270_IRQ0_ENTRY     0x00030518
#define DM270_IRQ1_ENTRY     0x0003051a
#define DM270_FIQ0_SELECT    0x00030520 /* FIQs  0-15 */
#define DM270_FIQ1_SELECT    0x00030522 /* FIQs 16-31 */
#define DM270_FIQ2_SELECT    0x00030524 /* FIQs 32-39 */
#define DM270_IRQ0_ENABLE    0x00030528 /* EINT  0-15 */
#define DM270_IRQ1_ENABLE    0x0003052a /* EINT 16-31 */
#define DM270_IRQ2_ENABLE    0x0003052c /* EINT 32-39 */
#define DM270_INT_RAW        0x00030530
#define DM270_EA0_BASE       0x00030538
#define DM270_EA1_BASE       0x0003053a

#define DM270_INT_PRI_ARRAY  0x00030540 /* Through 0x00030566:
					 * INTPR0 - INTPR19 */
/* GIO  */

#define DM270_GIO_DIR0       0x00030580
#define DM270_GIO_DIR1       0x00030582
#define DM270_GIO_DIR2       0x00030584
#define DM270_GIO_INV0       0x00030586
#define DM270_GIO_INV1       0x00030588
#define DM270_GIO_INV2       0x0003058a
#define DM270_GIO_BITSET0    0x0003058c
#define DM270_GIO_BITSET1    0x0003058e
#define DM270_GIO_BITSET2    0x00030590
#define DM270_GIO_BITCLR0    0x00030592
#define DM270_GIO_BITCLR1    0x00030594
#define DM270_GIO_BITCLR2    0x00030596
#define DM270_GIO_IRQPORT    0x00030598
#define DM270_GIO_IRQEDGE    0x0003059a
#define DM270_GIO_CHAT0      0x0003059c
#define DM270_GIO_CHAT1      0x0003059e
#define DM270_GIO_CHAT2      0x000305a0
#define DM270_GIO_NCHAT      0x000305aa
#define DM270_GIO_FSEL0      0x000305a4
#define DM270_GIO_FSEL1      0x000305a6

#define GIO_BLOCK_SIZE       0x00000028

#define GIO_0_BIT      (1 <<  0)
#define GIO_1_BIT      (1 <<  1)
#define GIO_2_BIT      (1 <<  2)
#define GIO_3_BIT      (1 <<  3)
#define GIO_4_BIT      (1 <<  4)
#define GIO_5_BIT      (1 <<  5)
#define GIO_6_BIT      (1 <<  6)
#define GIO_7_BIT      (1 <<  7)
#define GIO_8_BIT      (1 <<  8)
#define GIO_9_BIT      (1 <<  9)
#define GIO_10_BIT     (1 << 10)
#define GIO_11_BIT     (1 << 11)
#define GIO_12_BIT     (1 << 12)
#define GIO_13_BIT     (1 << 13)
#define GIO_14_BIT     (1 << 14)
#define GIO_15_BIT     (1 << 15)
#define GIO_16_BIT     (1 <<  0)
#define GIO_17_BIT     (1 <<  1)
#define GIO_18_BIT     (1 <<  2)
#define GIO_19_BIT     (1 <<  3)
#define GIO_20_BIT     (1 <<  4)
#define GIO_21_BIT     (1 <<  5)
#define GIO_22_BIT     (1 <<  6)
#define GIO_23_BIT     (1 <<  7)
#define GIO_24_BIT     (1 <<  8)
#define GIO_25_BIT     (1 <<  9)
#define GIO_26_BIT     (1 << 10)
#define GIO_27_BIT     (1 << 11)
#define GIO_28_BIT     (1 << 12)
#define GIO_29_BIT     (1 << 13)
#define GIO_30_BIT     (1 << 14)
#define GIO_31_BIT     (1 << 15)
#define GIO_32_BIT     (1 <<  0)
#define GIO_33_BIT     (1 <<  1)
#define GIO_ALL_BITS   0xff

#define MAX_GIO 33
#define MAX_I2C 31

/* HPIB */

#define DM270_HPIBCTL        0x00030600 /* HPIB Control Register */
#define DM270_HPIBSTAT       0x00030602 /* HPIB Status Register */

#define HPIB_BLOCK_SIZE      0x00000004

/* OSD */

#define DM270_OSDMODE        0x00030680 /* OSD Mode Setup */
#define DM270_VIDWINMD       0x00030682 /* Video Window Mode Setup */
#define DM270_OSDWIN0MD      0x00030684 /* OSD Win0 Mode Setup */
#define DM270_OSDWIN1MD      0x00030686 /* OSD Win1/Attrib Window Mode Setup */
#define DM270_RECTCUR        0x00030688 /* Rectangular Cursor Setup */
#define DM270_BMPCUR         0x0003068a /* Bitmap Cursor Setup */
#define DM270_VIDWIN0OFST    0x0003068c /* Video Win0 Offset */
#define DM270_VIDWIN1OFST    0x0003068e /* Video Win1 Offset */
#define DM270_OSDWIN0OFST    0x00030690 /* OSD Win0 Offset */
#define DM270_OSDWIN1OFST    0x00030692 /* OSD Win1 Offset */
#define DM270_VIDWINADH      0x00030694 /* Video Win0/1 AddressHigh */
#define DM270_VIDWIN0ADL     0x00030696 /* Video Win0 AddressLow */
#define DM270_VIDWIN1ADL     0x00030698 /* Video Win1 AddressLow */
#define DM270_OSDWINADH      0x0003069a /* OSD Win0/1 AddressHigh */
#define DM270_OSDWIN0ADL     0x0003069c /* OSD Win0 AddressLow */
#define DM270_OSDWIN1ADL     0x0003069e /* OSD Win1 AddressLow */
#define DM270_BASEPX         0x000306a0 /* Base Pixel X */
#define DM270_BASEPY         0x000306a2 /* Base Pixel Y */
#define DM270_VIDWIN0XP      0x000306a4 /* Video Win0 X-Pos  */
#define DM270_VIDWIN0YP      0x000306a6 /* Video Win0 Y-Pos  */
#define DM270_VIDWIN0XL      0x000306a8 /* Video Win0 X-Size  */
#define DM270_VIDWIN0YL      0x000306aa /* Video Win0 Y-Size  */
#define DM270_VIDWIN1XP      0x000306ac /* Video Win1 X-Pos  */
#define DM270_VIDWIN1YP      0x000306ae /* Video Win1 Y-Pos  */
#define DM270_VIDWIN1XL      0x000306b0 /* Video Win1 X-Size  */
#define DM270_VIDWIN1YL      0x000306b2 /* Video Win1 Y-Size  */
#define DM270_OSDWIN0XP      0x000306b4 /* OSD Bitmap Win0 X-Pos */
#define DM270_OSDWIN0YP      0x000306b6 /* OSD Bitmap Win0 Y-Pos */
#define DM270_OSDWIN0XL      0x000306b8 /* OSD Bitmap Win0 X-Size */
#define DM270_OSDWIN0YL      0x000306ba /* OSD Bitmap Win0 Y-Size */
#define DM270_OSDWIN1XP      0x000306bc /* OSD Bitmap Win1 X-Pos */
#define DM270_OSDWIN1YP      0x000306be /* OSD Bitmap Win1 Y-Pos */
#define DM270_OSDWIN1XL      0x000306c0 /* OSD Bitmap Win1 X-Size */
#define DM270_OSDWIN1YL      0x000306c2 /* OSD Bitmap Win1 Y-Size */
#define DM270_CURXP          0x000306c4 /* Rectangular Cursor Window X-Pos */ 
#define DM270_CURYP          0x000306c6 /* Rectangular Cursor Window Y-Pos */ 
#define DM270_CURXL          0x000306c8 /* Rectangular Cursor Window X-Size */
#define DM270_CURYL          0x000306ca /* Rectangular Cursor Window Y-Size */
#define DM270_BMPCURXP       0x000306cc /* Bitmap Cursor Window X-Pos */
#define DM270_BMPCURYP       0x000306ce /* Bitmap Cursor Window Y-Pos */
#define DM270_W0BMP01        0x000306d0 /* Win0 Bitmap to Palette Map 0/1 */
#define DM270_WOBMP23        0x000306d2 /* Win0 Bitmap to Palette Map 2/3 */
#define DM270_WOBMP45        0x000306d4 /* Win0 Bitmap to Palette Map 4/5 */
#define DM270_WOBMP67        0x000306d6 /* Win0 Bitmap to Palette Map 6/7 */
#define DM270_WOBMP89        0x000306d8 /* Win0 Bitmap to Palette Map 8/9 */
#define DM270_WOBMPAB        0x000306da /* Win0 Bitmap to Palette Map a/b */
#define DM270_WOBMPCD        0x000306dc /* Win0 Bitmap to Palette Map c/d */
#define DM270_WOBMPEF        0x000306de /* Win0 Bitmap to Palette Map e/f */
#define DM270_W1BMP01        0x000306e0 /* Win1 Bitmap to Palette Map 0/1 */
#define DM270_W1BMP23        0x000306e2 /* Win1 Bitmap to Palette Map 2/3 */
#define DM270_W1BMP45        0x000306e4 /* Win1 Bitmap to Palette Map 4/5 */
#define DM270_W1BMP67        0x000306e6 /* Win1 Bitmap to Palette Map 6/7 */
#define DM270_W1BMP89        0x000306e8 /* Win1 Bitmap to Palette Map 8/9 */
#define DM270_W1BMPAB        0x000306ea /* Win1 Bitmap to Palette Map a/b */
#define DM270_W1BMPCD        0x000306ec /* Win1 Bitmap to Palette Map c/d */
#define DM270_W1BMPEF        0x000306ee /* Win1 Bitmap to Palette Map e/f */
#define DM270_BMPCURDAT      0x000306f2 /* Bitmap Cursor Data Setup */
#define DM270_MISCCTL        0x000306f4 /* Miscellaneous Control */
#define DM270_CLUTRAMYCB     0x000306f6 /* CLUT RAM Y/Cb Setup */
#define DM270_CLUTRAMCR      0x000306f8 /* CLUT RAM Cr/Mapping Setup */
#define DM270_PPVWIN0ADH     0x000306fc /* Ping-Pong Video Win0 Address-High */
#define DM270_PPVWIN0ADL     0x000306fe /* Ping-Pong Video Win0 Address-Low */

#define OSD_BLOCK_SIZE       0x00000080

/* CCD Controller */

#define DM270_CCDC_SYNCEN    0x00030700
#define DM270_CCDC_MODESET   0x00030702
#define DM270_CCDC_HDW       0x00030704
#define DM270_CCDC_VDW       0x00030706
#define DM270_CCDC_PPLN      0x00030708
#define DM270_CCDC_LPFR      0x0003070a
#define DM270_CCDC_SPH       0x0003070c
#define DM270_CCDC_LPH       0x0003070e
#define DM270_CCDC_SLV0      0x00030710
#define DM270_CCDC_SLV1      0x00030712
#define DM270_CCDC_LNV       0x00030714
#define DM270_CCDC_CULH      0x00030716
#define DM270_CCDC_CULV      0x00030718
#define DM270_CCDC_HSIZE     0x0003071a
#define DM270_CCDC_SDOFST    0x0003071c
#define DM270_CCDC_CADU      0x0003071e
#define DM270_CCDC_CADL      0x00030720
#define DM270_CCDC_CCLAMP    0x00030722
#define DM270_CCDC_CDCSUB    0x00030724
#define DM270_CCDC_CCOLP     0x00030726
#define DM270_CCDC_COFST0    0x00030728
#define DM270_CCDC_COFST1    0x0003072a
#define DM270_CCDC_CMFIL     0x0003072c
#define DM270_CCDC_CGAIN     0x0003072e
#define DM270_CCDC_COFSTA    0x00030730
#define DM270_CCDC_OUTTH     0x00030732
#define DM270_CCDC_OUTCLIP   0x00030734
#define DM270_CCDC_VDINT0    0x00030736
#define DM270_CCDC_VDINT1    0x00030738
#define DM270_CCDC_SHUTCTL   0x0003073a
#define DM270_CCDC_CGAMMAWD  0x0003073c
#define DM270_CCDC_REC656IF  0x0003073e
#define DM270_CCDC_CCDCFG    0x00030740

/* Preview Engine */

#define DM2270_PVEN_PVEN      0x00030780
#define DM2270_PVEN_PVSET1    0x00030782
#define DM2270_PVEN_RADDU     0x00030784
#define DM2270_PVEN_RADDL     0x00030786
#define DM2270_PVEN_WADU      0x00030788
#define DM2270_PVEN_WADL      0x0003078a
#define DM2270_PVEN_HSRT      0x0003078c
#define DM2270_PVEN_HSIZE     0x0003078e
#define DM2270_PVEN_VSRT      0x00030790
#define DM2270_PVEN_VSIZE     0x00030792
#define DM2270_PVEN_PVSET2    0x00030794
#define DM2270_PVEN_N_FIL     0x00030796
#define DM2270_PVEN_DGAIN     0x00030798
#define DM2270_PVEN_WB_GAIN0  0x0003079a
#define DM2270_PVEN_WB_GAIN1  0x0003079c
#define DM2270_PVEN_SMTH      0x0003079e
#define DM2270_PVEN_H_RSZ     0x000307a0
#define DM2270_PVEN_V_RSZ     0x000307a2
#define DM2270_PVEN_BL_OFST0  0x000307a4
#define DM2270_PVEN_BL_OFST1  0x000307a6
#define DM2270_PVEN_MTX_GAIN0 0x000307a8
#define DM2270_PVEN_MTX_GAIN1 0x000307aa
#define DM2270_PVEN_MTX_GAIN2 0x000307ac
#define DM2270_PVEN_MTX_GAIN3 0x000307ae
#define DM2270_PVEN_MTX_GAIN4 0x000307b0
#define DM2270_PVEN_MTX_OFST0 0x000307b2
#define DM2270_PVEN_MTX_OFST1 0x000307b4
#define DM2270_PVEN_MTX_OFST2 0x000307b6
#define DM2270_PVEN_BYP_GAM   0x000307b8
#define DM2270_PVEN_CSC0      0x000307ba
#define DM2270_PVEN_CSC1      0x000307bc
#define DM2270_PVEN_CSC2      0x000307be
#define DM2270_PVEN_CSC3      0x000307c0
#define DM2270_PVEN_CSC4      0x000307c2
#define DM2270_PVEN_Y_OFST    0x000307c4
#define DM2270_PVEN_C_OFST    0x000307c6
#define DM2270_PVEN_CNT_BRT   0x000307c8
#define DM2270_PVEN_C_SUP0    0x000307ca
#define DM2270_PVEN_C_SUP1    0x000307cc
#define DM2270_PVEN_SETUP_Y   0x000307ce
#define DM2270_PVEN_SETUP_C   0x000307d0

/* NTSC/PAL Video Encoder Registers */

#define DM270_VID01          0x00030800 /* Video mode setup #1 */
#define DM270_VID02          0x00030802 /* Video mode setup #2 */
#define DM270_DLCD0          0x00030804 /* Digital LCD mode setup #0 */
#define DM270_DLCD1          0x00030806 /* Digital LCD mode setup #1 */
#define DM270_DCLK_PTN0E     0x00030808 /* DCLK even pattern #0 */
#define DM270_DCLK_PTN1E     0x0003080a /* DCLK even pattern #1 */
#define DM270_DCLK_PTN2E     0x0003080c /* DCLK even pattern #2 */
#define DM270_DCLK_PTN3E     0x0003080e /* DCLK even pattern #3 */
#define DM270_DCLK_PTN0O     0x00030810 /* DCLK odd pattern #0 */
#define DM270_DCLK_PTN1O     0x00030812 /* DCLK odd pattern #1 */
#define DM270_DCLK_PTN2O     0x00030814 /* DCLK odd pattern #2 */
#define DM270_DCLK_PTN3O     0x00030816 /* DCLK odd pattern #3 */
#define DM270_DCLK_STARTHE   0x00030818 /* DCLK H even start position enab */
#define DM270_DCLK_STARTHO   0x0003081a /* DCLK H odd start position enab */
#define DM270_DCLK_VLDH      0x0003081c /* DCLK H valid range */
#define DM270_DCLK_STARTV    0x0003081e /* DCLK V start position enab */
#define DM270_DCLK_VLDV      0x00030820 /* DCLK V valid range */
#define DM270_HVPWIDTH       0x00030822 /* H and V sync pulse width control */
#define DM270_HINTERVL       0x00030824 /* H sync interval control */
#define DM270_HSTART         0x00030826 /* H start position control */
#define DM270_HVALID         0x00030828 /* H valid area control */
#define DM270_VINTERVL       0x0003082a /* V sync interval control */
#define DM270_VSTART         0x0003082c /* V start position control */
#define DM270_VVALID         0x0003082e /* V valid area control */
#define DM270_HDELAY         0x00030830 /* H sync delay */
#define DM270_VDELAY         0x00030832 /* V sync delay */
#define DM270_CULLLINE       0x00030834 /* Line culling control */
#define DM270_PWMCTRL        0x00030836 /* PWM control */
#define DM270_PWMCNT         0x00030838 /* PWM count */
#define DM270_RGBLEVEL       0x0003083a /* RGB level control */
#define DM270_ATTRIBUTE0     0x0003083c /* Video attribute #0 */
#define DM270_ATTRIBUTE1     0x0003083e /* Video attribute #1 */
#define DM270_ATTRIBUTE2     0x00030840 /* Video attribute #2 */
#define DM270_REC656         0x00030842
#define DM270_EPSON_LCD      0x00030844
#define DM270_GCPDATA        0x00030846
#define DM270_CASIO          0x00030848
#define DM270_DOUTCTL        0x0003084a
#define DM270_SGHSYNP        0x0003084c

#define VIDEOENC_BLOCK_SIZE  0x0000004e

/* Clock controller */

#define DM270_CLOCKC_PLLA    0x00030880 /* PLLA config */
#define DM270_CLOCKC_PLLB    0x00030882 /* PLLB config */
#define DM270_CLOCKC_CLKC    0x00030884 /* CLK reverse, CLK source selection */
#define DM270_CLOCKC_SEL     0x00030886 /* PLL selection */
#define DM270_CLOCKC_DIV     0x00030888 /* DIV ratency config */
#define DM270_CLOCKC_BYP     0x0003088a /* PLL Bypass config */
#define DM270_CLOCKC_MMCLK   0x0003088c /* MMC CLK */
#define DM270_CLOCKC_CTST    0x0003088e /* CTST CLK */
#define DM270_CLOCKC_MOD0    0x00030890 /* CLK Enable/Disable */
#define DM270_CLOCKC_MOD1    0x00030892 /* CLK Enable/Disable */
#define DM270_CLOCKC_MOD2    0x00030894 /* CLK Enable/Disable */
#define DM270_CLOCKC_LPCTL0  0x00030896 /* SLEEP */
#define DM270_CLOCKC_LPCTL1  0x00030898 /* Power Down */
#define DM270_CLOCKC_OSEL    0x0003089a /* General CLK src selection */
#define DM270_CLOCKC_O0DIV   0x0003089c /* General CLK0 DIV config */
#define DM270_CLOCKC_O1DIV   0x0003089e /* General CLK1 DIV config */
#define DM270_CLOCKC_O2DIV   0x000308a0 /* General CLK2 DIV config */
#define DM270_CLOCKC_PWM0C   0x000308a2 /* PWM0 CLK config */
#define DM270_CLOCKC_PWM0H   0x000308a4 /* PWM0 H-Level duration config */
#define DM270_CLOCKC_PWM1C   0x000308a6 /* PWM1 CLK config */
#define DM270_CLOCKC_PWM1H   0x000308a8 /* PWM1 H-Level duration config */

/* Bus controller */

#define DM270_BUSC_ECR       0x00030900
#define DM270_BUSC_EBYTER    0x00030902
#define DM270_BUSC_EBITR     0x00030904
#define DM270_BUSC_REVR      0x00030906

/* SDRAM Controller */

#define DM270_SDRAMC_BUF0     0x00030980 /* D0L */
#define DM270_SDRAMC_BUF1     0x00030982 /* D0H */
#define DM270_SDRAMC_BUF2     0x00030984 /* D1L */
#define DM270_SDRAMC_BUF3     0x00030986 /* D1H */
#define DM270_SDRAMC_BUF4     0x00030988 /* D2L */
#define DM270_SDRAMC_BUF5     0x0003098a /* D2H */
#define DM270_SDRAMC_BUF6     0x0003098c /* D3L */
#define DM270_SDRAMC_BUF7     0x0003098e /* D3H */
#define DM270_SDRAMC_BUF8     0x00030990 /* D4L */
#define DM270_SDRAMC_BUF9     0x00030992 /* D4H */
#define DM270_SDRAMC_BUFA     0x00030994 /* D5L */
#define DM270_SDRAMC_BUFB     0x00030996 /* D5H */
#define DM270_SDRAMC_BUFC     0x00030998 /* D6L */
#define DM270_SDRAMC_BUFD     0x0003099a /* D6H */
#define DM270_SDRAMC_BUFE     0x0003099c /* D7L */
#define DM270_SDRAMC_BUFF     0x0003099e /* D7H */
#define DM270_SDRAMC_AD0      0x000309a0 /* Address Auto Incr, SDR Address1 */
#define DM270_SDRAMC_AD1      0x000309a2 /* SDR Address2 */
#define DM270_SDRAMC_BUFCTL   0x000309a4 /* SDR and BUFF control */
#define DM270_SDRAMC_MODE     0x000309a6 /* SDR Mode */
#define DM270_SDRAMC_REFCTL   0x000309a8 /* DMA selection, Refresh control */
#define DM270_SDRAMC_SDPRTY1  0x000309aa /* Priority1 */
#define DM270_SDRAMC_SDPRTY2  0x000309ac /* Priority2 */
#define DM270_SDRAMC_SDPRTY3  0x000309ae /* Priority3 */
#define DM270_SDRAMC_SDPRTY4  0x000309b0 /* Priority4 */
#define DM270_SDRAMC_SDPRTY5  0x000309b2 /* Priority5 */
#define DM270_SDRAMC_SDPRTY6  0x000309b4 /* Priority6 */
#define DM270_SDRAMC_SDPRTY7  0x000309b6 /* Priority7 */
#define DM270_SDRAMC_SDPRTY8  0x000309b8 /* Priority8 */
#define DM270_SDRAMC_SDPRTY9  0x000309ba /* Priority9 */
#define DM270_SDRAMC_SDPRTY10 0x000309bc /* Priority10 */
#define DM270_SDRAMC_PRTYON   0x000309be /* PRTY Enable */
#define DM270_SDRAMC_SDRCTEST 0x000309bE

/* External bus */

#define DM270_EXBC_CS0CTRL1   0x00030a00
#define DM270_EXBC_CS0CTRL2   0x00030a02
#define DM270_EXBC_CS1CTRL1A  0x00030a04
#define DM270_EXBC_CS1CTRL1B  0x00030a06
#define DM270_EXBC_CS1CTRL2   0x00030a08
#define DM270_EXBC_CS2CTRL1   0x00030a0a
#define DM270_EXBC_CS3CTRL1   0x00030a0e
#define DM270_EXBC_CS2CTRL2   0x00030a0c
#define DM270_EXBC_CS3CTRL2   0x00030a10
#define DM270_EXBC_CS4CTRL1   0x00030a12
#define DM270_EXBC_CS4CTRL2   0x00030a14
#define DM270_EXBC_BUSCTRL    0x00030a16
#define DM270_EXBC_BUSRLS     0x00030a18
#define DM270_EXBC_CFCTRL1    0x00030a1a
#define DM270_EXBC_CFCTRL2    0x00030a1c
#define DM270_EXBC_SMCTRL     0x00030a1e
#define DM270_EXBC_BUSINTEN   0x00030a20
#define DM270_EXBC_BUSSTS     0x00030a24
#define DM270_EXBC_BUSWAITMD  0x00030a26
#define DM270_EXBC_ECC1CP     0x00030a28
#define DM270_EXBC_ECC1LP     0x00030a2a
#define DM270_EXBC_ECC2CP     0x00030a2c
#define DM270_EXBC_ECC2LP     0x00030a2e
#define DM270_EXBC_ECCCLR     0x00030a30
#define DM270_EXBC_PAGESZ     0x00030a32
#define DM270_EXBC_IMGDSPDEST 0x00030a34
#define DM270_EXBC_PRIORCTL   0x00030a36
#define DM270_EXBC_SOURCEADD1 0x00030a38
#define DM270_EXBC_SOURCEADD2 0x00030a3a
#define DM270_EXBC_DESTADD1   0x00030a3c
#define DM270_EXBC_DESTADD2   0x00030a3e
#define DM270_EXBC_DMASIZE    0x00030a40
#define DM270_EXBC_DMADEVSEL  0x00030a42
#define DM270_EXBC_DMACTL     0x00030a44
#define DM270_EXBC_IMGDSPADD1 0x00030a46
#define DM270_EXBC_IMGDSPADD2 0x00030a48
#define DM270_EXBC_DPSTR0     0x00030a4a
#define DM270_EXBC_DPSTR1     0x00030a4c
#define DM270_EXBC_DPSTR2     0x00030a4e
#define DM270_EXBC_DPSTR3     0x00030a50
#define DM270_EXBC_DPSTR4     0x00030a52
#define DM270_EXBC_DPSTR5     0x00030a54
#define DM270_EXBC_TEST       0x00030a56

/* USB #0 */

#define DM270_USB_EP0OAD      0x00030a80
#define DM270_USB_EP0OSZ      0x00030a82
#define DM270_USB_EP0OCTL     0x00030a84
#define DM270_USB_EP0OIRQSZ   0x00030a86
#define DM270_USB_EP0ORDT     0x00030a8a
#define DM270_USB_EP0OREST    0x00030a8c
#define DM270_USB_EP0OST      0x00030a8e
#define DM270_USB_EP0IAD      0x00030a90
#define DM270_USB_EP0ISZ      0x00030a92
#define DM270_USB_EP0ICTL     0x00030a94
#define DM270_USB_EP0IIRQSZ   0x00030a96
#define DM270_USB_EP0IWDT     0x00030a98
#define DM270_USB_EP0IREST    0x00030a9c
#define DM270_USB_EP0IST      0x00030a9e
#define DM270_USB_EP1OAD      0x00030aa0
#define DM270_USB_EP1OSZ      0x00030aa2
#define DM270_USB_EP1OCTL     0x00030aa4
#define DM270_USB_EP1OIRQSZ   0x00030aa6
#define DM270_USB_EP1ORDT     0x00030aaa
#define DM270_USB_EP1OREST    0x00030aac
#define DM270_USB_EP1OST      0x00030aae
#define DM270_USB_EP1IAD      0x00030ab0
#define DM270_USB_EP1ISZ      0x00030ab2
#define DM270_USB_EP1ICTL     0x00030ab4
#define DM270_USB_EP1IIRQSZ   0x00030ab6
#define DM270_USB_EP1IWDT     0x00030ab8
#define DM270_USB_EP1IREST    0x00030abc
#define DM270_USB_EP1IST      0x00030abe
#define DM270_USB_EP2IAD      0x00030ac0
#define DM270_USB_EP2ISZ      0x00030ac2
#define DM270_USB_EP2ICTL     0x00030ac4
#define DM270_USB_EP2IIRQSZ   0x00030ac6
#define DM270_USB_EP2IWDT     0x00030ac8
#define DM270_USB_EP2IREST    0x00030acc
#define DM270_USB_EP2IST      0x00030ace
#define DM270_USB_EP3IAD      0x00030ad0
#define DM270_USB_EP3ISZ      0x00030ad2
#define DM270_USB_EP3ICTL     0x00030ad4
#define DM270_USB_EP3IIRQSZ   0x00030ad6
#define DM270_USB_EP3IWDT     0x00030ad8
#define DM270_USB_EP3IREST    0x00030adc
#define DM270_USB_EP3IST      0x00030ade
#define DM270_USB_EP4IAD      0x00030ae0
#define DM270_USB_EP4ISZ      0x00030ae2
#define DM270_USB_EP4ICTL     0x00030ae4
#define DM270_USB_EP4IIRQSZ   0x00030ae6
#define DM270_USB_EP4IWDT     0x00030ae8
#define DM270_USB_EP4IREST    0x00030aec
#define DM270_USB_EP4IST      0x00030aee

/* USB #1 */

#define DM270_USB_USBRST      0x00030b00
#define DM270_USB_USBRSM      0x00030b02
#define DM270_USB_USBINTMSK   0x00030b04
#define DM270_USB_USBCFC      0x00030b06
#define DM270_USB_USBST       0x00030b08
#define DM270_USB_USBINTST    0x00030b0a
#define DM270_USB_USBFRM      0x00030b0c
#define DM270_USB_USBALT      0x00030b0e
#define DM270_USB_USBCTL      0x00030b10
#define DM270_USB_DMAAD0      0x00030b12
#define DM270_USB_DMAAD1      0x00030b14
#define DM270_USB_DMASZ       0x00030b16
#define DM270_USB_DMACTL      0x00030b18
#define DM270_USB_DMATRG      0x00030b1a
#define DM270_USB_DMAREST     0x00030b1c
#define DM270_USB_DMAST       0x00030b1e

/* AF */

#define DM270_AF_H3ACTRL      0x00030b80
#define DM270_AF_AFCTRL       0x00030b82
#define DM270_AF_AFPAX1       0x00030b84
#define DM270_AF_AFPAX2       0x00030b86
#define DM270_AF_AFPAX3       0x00030b88
#define DM270_AF_AFPAX4       0x00030b8a
#define DM270_AF_AFIIRSH      0x00030b8c
#define DM270_AF_AFPAX5       0x00030b8e
#define DM270_AF_AFSDRA1      0x00030b90
#define DM270_AF_AFSDRA2      0x00030b92
#define DM270_AF_AFSDRFLG     0x00030b94
#define DM270_AF_AFCOEFF10    0x00030b96
#define DM270_AF_AFCOEFF11    0x00030b98
#define DM270_AF_AFCOEFF12    0x00030b9a
#define DM270_AF_AFCOEFF13    0x00030b9c
#define DM270_AF_AFCOEFF14    0x00030b9e
#define DM270_AF_AFCOEFF15    0x00030ba0
#define DM270_AF_AFCOEFF16    0x00030ba2
#define DM270_AF_AFCOEFF17    0x00030ba4
#define DM270_AF_AFCOEFF18    0x00030ba6
#define DM270_AF_AFCOEFF19    0x00030ba8
#define DM270_AF_AFCOEFF110   0x00030baa
#define DM270_AF_AFCOEFF20    0x00030bac
#define DM270_AF_AFCOEFF21    0x00030bae
#define DM270_AF_AFCOEFF22    0x00030bb0
#define DM270_AF_AFCOEFF23    0x00030bb2
#define DM270_AF_AFCOEFF24    0x00030bb4
#define DM270_AF_AFCOEFF25    0x00030bb6
#define DM270_AF_AFCOEFF26    0x00030bb8
#define DM270_AF_AFCOEFF27    0x00030bba
#define DM270_AF_AFCOEFF28    0x00030bbc
#define DM270_AF_AFCOEFF29    0x00030bbe
#define DM270_AF_AFCOEFF210   0x00030bc0
#define DM270_AF_AEWCTRL      0x00030bc2
#define DM270_AF_AEWWIN1      0x00030bc4
#define DM270_AF_AEWWIN2      0x00030bc6
#define DM270_AF_AEWWIN3      0x00030bc8
#define DM270_AF_AEWWIN4      0x00030bca
#define DM270_AF_AEWWIN5      0x00030bcc
#define DM270_AF_AEWSDRA1     0x00030bce
#define DM270_AF_AEWSDRA2     0x00030bd0
#define DM270_AF_AEWSDRFLG    0x00030bd2
#define DM270_AF_3A_TEST      0x00030bfe

/* Memory Stick #1 */
/* Memory Stick #2 */
/* ATM */

#define DM270_ATM_ATMCTRL    0x00030d00
#define DM270_ATM_ATM00      0x00030d04 /* First of 62 registers in
					 * range 0x00030d04-0003:0d7e */

/* I2C */

#define DM270_I2C_TXDATA    0x00030d80
#define DM270_I2C_RXDATA    0x00030d82
#define DM270_I2C_SCS       0x00030d84

/* Instruction Cache */

#define DM270_ICACHE_ICAMODE 0x00030e00

/* HARD_RESET_NOW -- used in blkmem.c. Should call arch_hard_reset(). */

#define HARD_RESET_NOW()

/* DSPRAM
 *
 * The DSP internal memory (32-Kwords of DARAM) can be read and written
 * by the ARM or an external host through the HPI-Bridge: Addresses
 * 0x0-0x7FFFh of DSP DARAM correspond to addresses 0004:0000-0004:FFFF
 * in the ARM memory map.  Addresses 0x0-0x7FFFh of DSP DARAM correspond
 * to addresses 000 1:0000-0001:FFFF in the external host memory map. 
 */

#define DSPRAM_ARM_BASE     0x00040000  /* DSPRAM ARM base address */
#define DSPRAM_ARM_END      0x0004ffff

#define DSPRAM_EXT_BASE     0x00100000  /* DSPRAM external host base address */
#define DSPRAM_EXT_END      0x0010ffff

#endif  /* __ASM_ARCH_HARDWARE_H */
