/*
 * linux/include/asm-armnommu/arch-dm270/serial.h
 *
 *   Copyright (C) 2003 Cadenux, LLC (http://www.cadenux.com)
 *   author:  Gregory Nutt <greg.nutt@cadenux.com>
 *
 * Derived from linux/include/asm-armnommu/arch-dsc25/serial.h
 *
 *   Copyright (C) 2002 Cadenux, LLC (http://www.cadenux.com)
 *   author:  Gregory Nutt <greg.nutt@cadenux.com>
 *
 *   Copyright (C) 1996 Russell King.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *  Changelog:
 *   15-10-1996	RMK	Created
 *   04-04-1998	PJB	Merged `arc' and `a5k' architectures
 *   02-28-2001 GJM     Modified for dsc21
 *   10-17-2002 GEN     Modified for dsc25
 *   01-19-2003 GEN     Modified for dm270
 */

#ifndef __ASM_ARCH_SERIAL_H
#define __ASM_ARCH_SERIAL_H

#include <asm/arch/hardware.h>
#include <asm/arch/irqs.h>

/* These are the default settings that will be used for the
 * DM270 serial console (if selected).
 */

#define DM270_DEFAULT_CONSOLE_BAUD   115200  /* 115,200 buad */
#define DM270_DEFAULT_CONSOLE_BITS   8       /* 8 data bits */
#define DM270_DEFAULT_CONSOLE_PARITY 'n'     /* No parity */

/* PORT_DSC21 is defined in linux/serial.h.  We can't be both a DM270
 * and an DSC21.  So we will pirate this value here.
 */

#define PORT_DM270     PORT_DSC21

/* Setup */

#define BASE_BAUD      (DM270_CLOCK_RATE / 16)
#define STD_COM_FLAGS  (ASYNC_BOOT_AUTOCONF | ASYNC_SKIP_TEST)
#define RS_TABLE_SIZE  2
#define STD_SERIAL_PORT_DEFNS \
	{  \
	magic: 0, \
        baud_base: BASE_BAUD, \
	port: 0, \
	irq: DM270_IRQ_UART0, \
	flags: STD_COM_FLAGS, \
	type: PORT_UNKNOWN, \
	iomem_base: (u8*)DM270_UART0, \
	iomem_reg_shift: 1, \
	io_type: SERIAL_IO_MEM \
	},	/* ttyS0 */ \
	{  \
	magic: 0, \
        baud_base: BASE_BAUD, \
	port: 0, \
	irq: DM270_IRQ_UART1, \
	flags: STD_COM_FLAGS, \
	type: PORT_UNKNOWN, \
	iomem_base: (u8*)DM270_UART1, \
	iomem_reg_shift: 1, \
	io_type: SERIAL_IO_MEM \
	}	/* ttyS1 */ 
#define EXTRA_SERIAL_PORT_DEFNS

/* Registers */

#define UART_DTRR  0  /* Data Transmission/Reception Register */
#define UART_BRSR  1  /* Bit Rate Set Register */
#define UART_MSR   2  /* Mode Set Register */
#define UART_RFCR  3  /* Reception FIFO Control Register */
#define UART_TFCR  4  /* Transmission FIFO Control Register */
#define UART_LCR   5  /* Line Control Register */
#define UART_SR    6  /* Status Register */

/* Bitmasks */

#define UART_DTRR_PEF   0x0100    /* Parity Error (0=None, 1=Err) */
#define UART_DTRR_ORF   0x0200    /* Overrun (0=None, 1=overrun) */
#define UART_DTRR_FEX   0x0400    /* Framing Error (0=None, 1=Err) */
#define UART_DTRR_BF    0x0800    /* Break detected (0=None, 1=Break) */
#define UART_DTRR_RVF   0x1000    /* Recv Wrd Valid (0=invalid, 1=valid) */

/* MXI 27MHz */
#define UART_BRSR9600   0x01B6	  /* DM270 12-26-2002	67.5MHz */
#define UART_BRSR57600  0x0024	  /* DM270 12-26-2002	67.5MHz */
#define UART_BRSR115200 0x0011	  /* DM270 12-26-2002	67.5MHz */

#define UART_MSR_CLS    0x0001    /* Char length (1=7bit, 0=8bit) */
#define UART_MSR_RSVD0  0x0002    /* Reserved */
#define UART_MSR_SBLD   0x0004    /* Stop bit length (1=2bits, 0=1bit) */
#define UART_MSR_PSB    0x0008    /* Parity select bit (1=odd, 0=even) */
#define UART_MSR_PEB    0x0010    /* Parity enable bit (1=enable, 0=disable) */
#define UART_MSR_TO3IC  0x0400	  /* Enable timeout interrupt with 3-word */
#define UART_MSR_TO7IC  0x0800	  /* Enable timeout interrupt with 7-word */
#define UART_MSR_TO15IC 0x0c00	  /* Enable timeout interrupt with 15-word */
#define UART_MSR_TOIC   0x0c00    /* Timeout interrupt control */
#define UART_MSR_RDEIE  0x2000    /* Received data error interrupt enable */
#define UART_MSR_TFTIE  0x4000    /* TX FIFO trigger interrupt enable */
#define UART_MSR_RFTIE  0x8000    /* RX FIFO trigger interrupt enable */
#define UART_MSR_INTS   0xfc00    /* All interrupt bits */
#define UART_MSR_MODE_BITS   0x001f /* Aata length, stop & parity */

#define UART_RFCR_FIFO_CLEAR 0x8000 /* Rx/Tx FIFO clear bit in RFCR/TFCR */
#define UART_RFCR_RWC_BITS   0x003f /* data length, stop & parity mask */

#define UART_SR_TREF    0x0001    /* TX register empty */
#define UART_SR_TFEF    0x0002    /* TX FIFO empty */
#define UART_SR_RFNEF   0x0004    /* RX FIFO not empty */
#define UART_SR_RSVD0   0x00f8    /* Reserved */
#define UART_SR_TOIF    0x0100    /* Time out interrupt */
#define UART_SR_RFER    0x0200    /* RX FIFO error */
#define UART_SR_TFTI    0x0400    /* TX FIFO is less than trigger level */
#define UART_SR_RFTI    0x0800    /* RXFIFO trigger is greater than trigger */
#define UART_SR_RSVD1   0xf000    /* Reserved */

#define UART_LCR_BOC    0x0100    /* Break output control (1=send break) */

#define UART_FCR_FTL1   0x0000    /* FIFO trigger level 1 byte */
#define UART_FCR_FTL4   0x0100    /* FIFO trigger level 4 bytes */
#define UART_FCR_FTL8   0x0200    /* FIFO trigger level 8 bytes */
#define UART_FCR_FTL16  0x0300    /* FIFO trigger level 16 bytes */
#define UART_FCR_FTL24  0x0400    /* FIFO trigger level 24 bytes */
#define UART_FCR_FTL32  0x0500    /* FIFO trigger level 32 bytes */
#define UART_FCR_RBRB   0x4000    /* Receiver block reset (rx FIFO only) */
#define UART_FCR_CB     0x8000    /* FIFO clear bit */

struct dm270_async_struct {
	int			magic;
	unsigned long		port;
	int			flags;
	int			xmit_fifo_size;
	struct serial_state	*state;
	struct tty_struct 	*tty;
	int			timeout;
	int			quot;
	int			x_char;	/* xon/xoff character */
	int			close_delay;
	unsigned short		closing_wait;
	unsigned short		closing_wait2;
	u16			MSR; 	/* Interrupt Enable Register */
	u16			LCR; 	/* Line control register */
	u16                     RFCR;
	u16                     TFCR;
	unsigned long		event;
	unsigned long		last_active;
	int			line;
	int			blocked_open; /* # of blocked opens */
	long			session; /* Session of opening process */
	long			pgrp; /* pgrp of opening process */
 	struct circ_buf		xmit;
 	spinlock_t		xmit_lock;
	u8			*iomem_base;
	u16			iomem_reg_shift;
	int			io_type;
	struct tq_struct	tqueue;
#ifdef DECLARE_WAITQUEUE
	wait_queue_head_t	open_wait;
	wait_queue_head_t	close_wait;
	wait_queue_head_t	delta_msr_wait;
#else	
	struct wait_queue	*open_wait;
	struct wait_queue	*close_wait;
	struct wait_queue	*delta_msr_wait;
#endif	
	struct dm270_async_struct *next_port;
	struct dm270_async_struct *prev_port;
};

#endif /* __ASM_ARCH_SERIAL_H */
