/************************************************************************
 * linux/include/asm-armnommu/arch-dsc24/dsp.h
 * DSC24 DSP Interface Definitions
 *
 *   Copyright(c) 2003, Cadenux, LLC (http://www.cadenux.com)
 *   Author:  Gregory Nutt <greg.nutt@cadenux.com>
 *
 * Derived from: linux/include/asm-armnommu/arch-dsc21/dsp.h
 * DSC21 DSP Interface Definitions
 *
 *   Copyright(c) 2002, Cadenux, LLC
 *   Author:  Gregory Nutt <greg.nutt@cadenux.com>
 *
 * Derived loosely from RidgeRun, Inc.'s, dspi.h
 *
 *   Copyright (C) 2001 RidgeRun, Inc.
 *   Author: Clint Medley <cmedley@ridgerun.com> 10/30/01
 *     Based on the original hpi.h by Gordon McNutt <gmcnutt@ridgerun.com>
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * THIS  SOFTWARE  IS  PROVIDED  ``AS  IS''  AND   ANY  EXPRESS  OR IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT,  INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ************************************************************************/

#ifndef __ASM_ARCH_DSP_H_
#define __ASM_ARCH_DSP_H_

/************************************************************************
 * Included Files
 ************************************************************************/

#include <linux/types.h>
#include <asm/arch/hardware.h>

/************************************************************************
 * Definitions
 ************************************************************************/

/* This is the HPI address range in the DSP address space. */

#define DSPMEM_DSP_START   0 
#define DSPMEM_DSP_END     0x7fff

/* This is the HPI address range in the ARM address space. */

#define DSPMEM_ARM_START  DSPRAM_BASE /* Defined in hardware.h */
#define DSPMEM_ARM_END    DSPRAM_END

/* DSPMEM_IN_RANGE is a generic macro to test is a value is within
 * a range of values.
 */

#define DSPMEM_IN_RANGE(addr, start, end) \
  ((((__u32)(addr)) >= (start)) && (((__u32)(addr)) <= (end)))

/* DSPMEM_ADDR_ALIGNED verifies that a potential DSP address is
 * properly word aligned.
 */

#define DSPMEM_ADDR_ALIGNED(addr, cpu) ((((__u32)(addr)) & 1) == 0)

/* DSPMEM_DSP_ADDR checks if a DSP address lies in within the
 * DSP's HPI address range.
 */

#define DSPMEM_DSP_ADDR(addr, cpu) \
  DSPMEM_IN_RANGE(addr, DSPMEM_DSP_START, DSPMEM_DSP_END)

/* DSPMEM_ARM_ADDR checks if a ARM address lies in within the
 * ARM's HPI address range.
 */

#define DSPMEM_ARM_ADDR(addr) \
  DSPMEM_IN_RANGE(addr, DSPMEM_ARM_START, DSPMEM_ARM_END)

/* DSPMEM_DSP_TO_ARM maps a DSP HPI address into an ARM HPI address */

#define DSPMEM_DSP_TO_ARM(addr, cpu) \
  (((__u32)(addr) << 1) + DSPMEM_ARM_START)

/* DSPMEM_ARM_TO_DSP maps an ARM HPI address into a DSP HPI address */

#define DSPMEM_ARM_TO_DSP(addr) \
  (((__u32)(addr) - DSPMEM_ARM_START) >> 1)

#endif /* __ASM_ARCH_DSP_H_ */
