/************************************************************************
 * asm-armnommu/arch-dsc24/gio_ioctl.h
 ************************************************************************/

#ifndef __ASM_ARCH_GIO_IOCTL_H__
#define __ASM_ARCH_GIO_IOCTL_H__
 
/************************************************************************
 * Included Files
 ************************************************************************/

#include <asm/ioctl.h>
 
/************************************************************************
 * Definitions
 ************************************************************************/

#define GIO_MAGIC_NUMBER 'g'

/* Major numbers, (from Documentation/devices.txt local/experimental list)  */
 
#define GIO_MAJOR 125
 
/************************************************************************
 * GIO_DIRECTION: The GIO_DIRECTION can be either input (GIO_INPUT) or
 * output (GIO_OUTPUT).  If unspecified, the GIO will default to input.
 */

#define GIO_DIRECTION _IO(GIO_MAGIC_NUMBER, 0)

/* GIO_DIRECTION Arguments: */

#define GIO_OUTPUT 0
#define GIO_INPUT  1

/************************************************************************
 * GIO_INVERSION: The GIO polarity of the GIO pin can be inverted.  If
 * unspecified, the default behavior will be NOT inverted.
 */

#define GIO_INVERSION _IO(GIO_MAGIC_NUMBER, 1)

/* GIO_INVERSION Arguments: */

#define GIO_UNINVERTED 0
#define GIO_INVERTED   1

/************************************************************************
 * GIO_BITSET: If the direction is set to output, this ioctl will set the
 * output.  If the direction is input, this ioctl will return the value
 * of the GIO pin (0, 1, or -1 on error).
 */

#define GIO_BITSET _IO(GIO_MAGIC_NUMBER, 2)

/* GIO_BITSET arguments:  None */

/************************************************************************
 * GIO_BITCLR: If the direction is set to output, this ioctl will clear the
 * output.  If the direction is input, this ioctl will return the value
 * of the GIO pin (0, 1, or -1 on error).
 */

#define GIO_BITCLR _IO(GIO_MAGIC_NUMBER, 3)

/* GIO_BITCLR arguments:  None */

/************************************************************************
 * GIO_IRQPORT: For the DSC24, GIO0-GIO7 can be configured as external
 * interrupt ports
 */

#define GIO_IRQPORT _IO(GIO_MAGIC_NUMBER, 4)

/* GIO_IRQPORT arguments:*/

#define GIO_NOINTERRUPT 0
#define GIO_INTERRUPT   1

/************************************************************************
 * GIO_FSEL: Pins GIO8 through GIO20 can be set as functions.
 * Default is ordinary GIO (GIO_NORMAL).
 *
 * In addition, pins GIO16 and GIO17 can take different values to
 * set clock output on them.
 */

#define GIO_FSEL _IO(GIO_MAGIC_NUMBER, 5)

/* GIO_FSEL Arguments: */

#define GIO_NORMAL     0 /* 0=GIO8-GIO20 Normal */
#define GIO_FUNCTION   1

#define GIO_DIVIDED_CLOCK     1
#define GIO_MXI_CLOCK         2
#define GIO_MXI_CLOCK_DIVIDED 3

/************************************************************************
 * GIO_BITRATE: Forward/Transfer rate set bit.  GIO_FSEL has provisions
 * for setting clock rates in GIO16 and GIO17.  This ioctl will set
 * the division ratio of clock output of GIO16 or GIO17 when the pin is
 * is set to GIO divided clock output.
 *
 * This ioctl will set the BITRATE register without regard to the current
 * function setting of GIO16 or GIO17.
 */

#define GIO_BITRATE _IO(GIO_MAGIC_NUMBER, 6)

/* GIO_BITRATE Arguments: division ratio */

#define GIO_IOC_MAXNR 6
 
#endif /* __ASM_ARCH_GIO_IOCTL_H__ */
