/*
 * linux/include/asm-armnommu/arch-dsc24/io.h
 *
 *   Copyright (C) 2003, Cadenux, LLC <http://www.cadenux.com>
 *   Gregory Nutt <greg.nutt@cadenux.com>
 *
 * Derived from: linux/include/asm-armnommu/arch-dsc21/io.h
 *
 *   Copyright (C) 1997-1999 Russell King
 *
 * Modifications:
 *  06-12-1997	RMK	Created.
 *  07-04-1999	RMK	Major cleanup
 *  02-19-2001  gjm     Leveraged for armnommu/dsc21
 *  01-07-2003  GEN     Leveraged for armnommu/dsc24
 */
#ifndef __ASM_ARM_ARCH_IO_H
#define __ASM_ARM_ARCH_IO_H

/* kernel/resource.c uses this to initialize the global ioport_resource struct
 * which is used in all calls to request_resource(), allocate_resource(), etc.
 */

#define IO_SPACE_LIMIT 0xffffffff

/* If we define __io then asm/io.h will take care of most of the inb & friends
 * macros. It still leaves us some 16bit macros to deal with ourselves, though.
 * We don't have PCI or ISA on the dsc24.
 */

#define PCIO_BASE 0
#define __io(a) (PCIO_BASE + (a))

/* Some compiler options will convert short loads and stores into byte loads
 * and stores.  We don't want this to happen for IO reads and writes!
 */

#if 0
# define __arch_getw(a) (*(volatile unsigned short *)(a))
# define __arch_putw(v,a) (*(volatile unsigned short *)(a) = (v))
#else
static inline unsigned short __arch_getw(unsigned int addr)
{
  unsigned short retval;
  __asm__ __volatile__("\tldrh %0, [%1]\n\t" : "=r"(retval) : "r"(addr));
  return retval;
}
static inline void __arch_putw(unsigned short val, unsigned int addr)
{
  __asm__ __volatile__("\tstrh %0, [%1]\n\t": : "r"(val), "r"(addr));
}
#endif

/*
 * Defining these two gives us ioremap for free. See asm/io.h.
 * --gmcnutt
 */
#define iomem_valid_addr(iomem,sz) (1)
#define iomem_to_phys(iomem) (iomem)

#endif
