/*
 * linux/include/asm-armnommu/arch-dsc24/memory.h
 *
 *   Copyright (C) 2003 Cadenux, LLC (http://www.cadenux.com)
 *   author:  Gregory Nutt <greg.nutt@cadenux.com>
 *
 * Derived from:linux/include/asm-armnommu/arch-dsc21/memory.h
 *
 *   Copyright (c) 1999 Nicolas Pitre <nico@cam.org>
 *   Copyright (c) 2001 RidgeRun <www.ridgerun.com>
 *
 * 02/19/2001  Gordon McNutt  Leveraged for the dsc21.
 * 01/07/2003  Gregory Nutt   Leveraged for the dsc24.
 */

#ifndef __ASM_ARCH_MEMORY_H
#define __ASM_ARCH_MEMORY_H

/* Configured DRAM memory is specified during configuration and is found
 * in asm/arch/memconfig.h as CADENUX_DRAM_BASE and CADENUX_DRAM_SIZE.
 */

#include "linux/autoconf.h"
#include "asm/arch/memconfig.h"
#define END_MEM		(CADENUX_DRAM_BASE + CADENUX_DRAM_SIZE)

/* These are used in the fault handling code (even for nommu) and various other
 * macros. The current value comes from Steve Johnson's 2.0.38 uClinux port for
 * the dsc24.
 */

#define TASK_SIZE	(0x01a00000UL)
#define TASK_SIZE_26	TASK_SIZE

/* The alignment requirement of the memory management system
 * this is ~(1 << (PAGE_OFFSET+9))-1.
 */

#define PHYS_OFFSET_MASK 0x001fffff

/* The start of physical memory.  This is the first SDRAM address.
 * This value is used in the arch-specific bootup code like setup_arch &
 * bootmem_init.
 *
 * NOTE the alignment requirement of the memory management system
 * this is given by PHYS_OFFSET_MASK.  There are two approaches for
 * meeting this alignment requirement.  (1) place PHYS_OFFSET below
 * the kernel/start of SDRAM.  This could be made to work becuase
 * the region between the start of usable RAM (CADENUX_DRAM_BASE) and
 * PHYS_OFFSET will never be released to the memory manager.
 * This would be:
 * 
 * #define PHYS_OFFSET (CADENUX_DRAM_BASE & ~PHYS_OFFSET_MASK)
 *
 * However, the memory management system will attempt to locate the
 * mem_map tables at PAGE_OFFSET.  So these second, less conservative
 * solutions are probably safer:
 */

#if defined(CADENUX_BOOT_FROM_SDRAM)

  /* In this case, CADENUX_DRAM_BASE will be include memory for
   * the kernel.  Since the mem_map tables will begin at PAGE_OFFSET,
   * we will have to start the memory pool after the kernel.  This
   * could cause a substantial loss of memory due to the alignment
   * requirements!
   */

  extern unsigned long _end_kernel;
# define PHYS_OFFSET \
  ((((unsigned long)&_end_kernel) + PHYS_OFFSET_MASK) & ~PHYS_OFFSET_MASK)

#else

  /* Otherwise, the kernel is in rom/flash and all of this memory is
   * available.  So the mem_map tables can begin near the beginning
   * of physical SDRAM.  There can still be a significant loss, however,
   * if the beginning of SDRAM does not meet this alignment requirement
   */

# define PHYS_OFFSET \
  ((CADENUX_DRAM_BASE + PHYS_OFFSET_MASK) & ~PHYS_OFFSET_MASK)

#endif

/* PAGE_OFFSET -- the first address of the first page of memory. For archs with
 * no MMU this corresponds to the first free page in physical memory (aligned
 * on a page boundary).
 */

#define PAGE_OFFSET PHYS_OFFSET

#endif
