/*************************************************************************
 * asm/arch/fb_ioctl.h
 *
 * The DSC25 framebuffer driver header.  This file extends the standard
 * framebuffer definition of linux/fb.h by adding DSC25 specifics.
 *
 * Copyright (C) 2002 Cadenux, LLC. (http://www.cadenux.com)
 * Author: Gregory Nutt <greg.nutt@cadenux.com>
 *
 *************************************************************************/

#ifndef __ASM_ARCH_FB_IOCTL_H_
#define __ASM_ARCH_FB_IOCTL_H_

/*************************************************************************
 * Included Files
 *************************************************************************/

/* Standard frame buffer definitions */

#include <linux/fb.h>
#include <asm/types.h>

/*************************************************************************
 * Definitions
 *************************************************************************/

#define MAX_XRES  640
#define MAX_YRES  240

/* OSD Window 0 (fb0) */

#define DEFAULT_WIN0_XRES MAX_XRES
#define DEFAULT_WIN0_YRES MAX_YRES

/* OSD Window 1 (fb1) */

#define DEFAULT_WIN1_XRES MAX_XRES
#define DEFAULT_WIN1_YRES MAX_YRES

/* Rectangular or Bitmap Cursor */

#define DEFAULT_CURSOR_XRES 32
#define DEFAULT_CURSOR_YRES 32

/*************************************************************************
 * Public Types
 *************************************************************************/

struct dsc25_vid01_s
{
  __u16 venc_enable    :  1; /* 0: vs. video encoder disable */
  __u16 extern_sync    :  1; /* 1: vs. normal Y/C output */
  __u16 output_enable  :  1; /* 2: vs. video output disable */
  __u16 reserved       :  1; /* 3: (reserved) */
  __u16 by_enable      :  1; /* 4: vs. blue/Y signal output disable */
  __u16 gy_enable      :  1; /* 5: vs. green/Y signal output disable */
  __u16 ry_enable      :  1; /* 6: vs. red/Y signal output disable */
  __u16 y_lfp_tap      :  1; /* 7: vs. Y filter off */
  __u16 rgb_lpf_tap    :  2; /* 8:9: 0=off, 1=3tap, 2=5tap, 3=off */
  __u16 setup_75pct    :  1; /* 10: vs 0% setup selection */
  __u16 cbar_100pct    :  1; /* 11: vs 75% color bar */
  __u16 cbar_builtin   :  1; /* 12: vs. normal color bar */
  __u16 cutoff_13mhz   :  1; /* 13: vs. 1.5 MHz cutoff */
  __u16 nointerlace    :  1; /* 14: vs. interlace mode  */
  __u16 pal_select     :  1; /* 15: vs. NTSC video */
};

struct dsc25_vid02_s
{
  __u16 bright_width   :  8; /* 0:7: bright signal width (unit=2/clk) */
  __u16 bright_reverse :  1; /* 8: vs. normal bright polarity */
  __u16 bright_enable  :  1; /* 9: vs. bright disable */
  __u16 sync_enable    :  1; /* 10: vs. sync disable */
  __u16 composite_sync :  1; /* 11: vs. vertical sync output */
  __u16 digital_sync   :  1; /* 12: vs. analog sync output */
  __u16 range0_255     :  1; /* 13: vs. input signal range 16-253 */
  __u16 video_mode     :  2; /* 14:15: 0=Y/C, 1=analog RGB, 2=digital RGB */
};

struct dsc25_dlcd1_s
{
  __u16 odd_lines      :  3; /* 0:2: 0=RGB 1=RBG 2=GRB 3=GBR 4=BRG 5=BGR */
  __u16 even_lines     :  3; /* 3:5: 0=RGB 1=RBG 2=GRB 3=GBR 4=BRG 5=BGR */
  __u16 line_id_diff   :  1; /* 6: vs. same as odd/even field sel */
  __u16 rgb_line_cull  :  1; /* 7: vs. no RGB line cull */
  __u16 iron_man       :  1; /* 8: vs. RGB565 Iron Man disable */
  __u16 extern_pol     :  1; /* 9: vs. ext. line ID polarity not used */
  __u16 extern_id      :  1; /* 10: vs. external line ID not used */
  __u16 line_reverse   :  1; /* 11: vs. line reverse disable */
  __u16 pal_odd        :  1; /* 12: vs. PAL output timing even */
  __u16 hsync_high     :  1; /* 13: vs. hsync polarity active low */
  __u16 vsync_high     :  1; /* 14: vs. vsync polarity active low */
  __u16 drgb_enable    :  1; /* 15: vs. digital RGB disable */
};

struct dsc25_video_mode_s
{
  struct dsc25_vid01_s vid01; /* Video mode setup 1 values */
  struct dsc25_vid02_s vid02; /* Video mode setup 2 values */
  struct dsc25_dlcd1_s dlcd1; /* Digital LCD mode setup 1 values */
};

struct dsc25_osdmode_s
{
  __u16 cabg  :  8; /* 0:7: background color clut */
  __u16 bclut :  1; /*  8: background clut 0=ROM 1=RAM */
  __u16 rsvd  :  1; /*  9: (reserved) */
  __u16 vhrsz :  1; /* 10: video window horizontal expansion: 0=x1 1=x9/8 */
  __u16 vvrsz :  1; /* 11: video window vertical expansion: 0=x1 1=x6/5 */
  __u16 ef    :  1; /* 12: expansion filter 0=off 1=on */
  __u16 fsinv :  1; /* 13: field signal 0=normal 1=inverted */
  __u16 orsz  :  1; /* 14: OSD window vertical expansion: 0=x1 1=x6/5 */
  __u16 cs    :  1; /* 15: 0=offset binary, 1=complement of 2 */
};

/* Starting x and y coords for OSD/video win0, win1, or for rectangular
 * cursor.
 */

struct dsc25_pos_s
{
  __u16 xpos;
  __u16 ypos;
};

/* Size for bitmap cursor.  In 32x32x4 mode, only field 1 is used.
 * In 16x16x1 and 16x8x1 modes both field1 and field2 are used.
 */

struct dsc25_bmpcursor_pos_s
{
  struct dsc25_pos_s field1; /* All modes */
  struct dsc25_pos_s field2; /* 16x16x1 and 16x8x1 modes only */
};

/* Size rectangular cursor . */

struct dsc25_size_s
{
  __u16 xres;
  __u16 yres;
};

/* Register settings for mode */

struct dsc25_vidwinmode_s
{
  __u16 active :  1; /* 0:   0=Off, 1=On */
  __u16 ffmode :  1; /* 1:   0=field mode, 1=frame mode */
  __u16 hzoom  :  2; /* 2:3: 0=x1 1=x2 2=x4 3=x1 */
  __u16 vzoom  :  2; /* 4:5  0=x1 1=x2 2=x4 3=x1 */
  __u16 rsvd   : 10; /* 6:15 (reserved) */
};

#define INITIAL_VIDWINMODE 0x0000

struct dsc25_osdwinmode_s
{
  __u16 active  : 1; /* 0: 0=window inactive 1=on */
  __u16 ffmode  : 1; /* 1: 0=field display mode 1=frame display mode */
  __u16 trans   : 1; /* 2: 0=transparency mode disabled 1=enabled */
  __u16 blend   : 3; /* 3:5: Blending ratio: 0=0:1 1=1/8:7/8 2=3/8:5/8... 7=1:0*/
  __u16 bmwidth : 2; /* 6:7: Bitmap bit width: 0=1 1=2 2=4 3=8 */
  __u16 vzoom   : 2; /* 8:9: Vertical zoom: 0=x1 1=x2 2=x4 3=x1 */
  __u16 hzoom   : 2; /* 10:11: Horizontal zoom: 0=x1 1=x2 2=x4 3=x1 */
  __u16 clut    : 1; /* 12: 0=ROM lookup table, 1=RAM lookup table */
  __u16 rsvd    : 2; /* 13:14: (reserved) */
  __u16 attr    : 1; /* 15: 0=OSD Window1 1=Attribute Window (OSD win1 only) */
};

/* Initial OSDWINxMD settings: inactive, transparent, 8-bit */

#define INITIAL_OSDWINMODE 0x00c4

/* Register settings for the rectangular cursor */

struct dsc25_rectcur_s
{
  __u16 active  : 1; /* 0: 0=rectangular cursor inactive 1=on */
  __u16 vheight : 3; /* 1:3: Vertical line height: {1,2,4,6,8,10,12,14} */
  __u16 hwidth  : 3; /* 4:6: Horizontal line width: {1,4,8,16,20,24,28} */
  __u16 clut    : 1; /* 7: 0=ROM lookup table, 1=RAM lookup table */
  __u16 rcad    : 8; /* 8:15: Rectangular cursor color pallette address */
};

/* Register settings for the bitmap cursor */

struct dsc25_bmpcur_s
{
  __u16 active  : 1; /* 0: 0=bitmap cursor inactive 1=on */
  __u16 ffmode  : 1; /* 1: 0=field display mode 1=frame display mode */
  __u16 clut    : 1; /* 2: 0=ROM lookup table, 1=RAM lookup table */
  __u16 lookup  : 1; /* 3: 0=OSD win0 lookup table 1=OSD win1 lookup table */
  __u16 blend   : 3; /* 4:6: Blending ratio: 0=0:1 1=1/8:7/8 2=3/8:5/8... 7=1:0*/
  __u16 hlength : 1; /* 7: 0=16x16x1 1=16x8x1 */
  __u16 vzoom   : 2; /* 8:9: Vertical zoom: 0=x1 1=x2 2=x3 */
  __u16 hzoom   : 2; /* 10:11: Horizontal zoom: 0=x1 1=x2 2=x3 */
  __u16 bdelay  : 2; /* 12:13: blink interval 0=1 1=2 2=3 3=4 units */
  __u16 blink   : 1; /* 14: 0=disable blinking, 1=enable blinking */
  __u16 btype   : 1; /* 15: BMP cursor type: 0=32x32x4 1=16x16/8x1 */
};

/* Structure used to specify data for bitmap cursor:
 *
 * data:  Bitmap Cursor Data
 *        32(H)x32(V)x4(BIT): 
 *          BCDAT[15]-BCDAT[12]      
 *          BCDAT[11]-BCDAT[08]    
 *          BCDAT[07]-BCDAT[04]   
 *          BCDAT[03]-BCDAT[00] 
 *        16(H)x16(V)x1(BIT): 
 *          BCDAT[15]-BCDAT[00] 
 *        16(H)x8(V)x1(BIT):   
 *          BCDAT[15]-BCDAT[00]
 *
 * addr:  This is the bitmap cursor data address in RAM.  Bitmap cursor
 *        data is written to this address. The addressing scheme depends
 *        on the selected cusor type in BMPCUR.  Address Range:
 *        [0-63]: For 32(H)x32(V)x4(BIT) cusor type
 *        [0-31]: For 16(H)x16(V)x1(BIT) cursor type
 *        [0-15]: For 16(H)x08(V)x1(BIT) cursor type
 */

/* The general form is: */

struct dsc25_bmpdata_s
{
  __u16 addr;        /* MISCCTL start address (See above) */
  __u16 nelem;       /* Number of elements in data[] array */
  __u16 data[1];     /* Start of BMPCURDAT data (See above) */
};

/* But any of the specific forms may be used to set aside memory */

struct dsc25_32x32x4_bmpdata_s
{
  __u16 addr;        /* Probably  0 */
  __u16 nelem;       /* Probably 64 */
  __u16 data[64];    /* (See above) */
};

struct dsc25_16x16x1_bmpdata_s
{
  __u16 addr;        /* Probably  0 (or 16 for field 2) */
  __u16 nelem;       /* Probably 32 (or 16 for one field) */
  __u16 data[32];    /* (See above) */
};

struct dsc25_16x8x1_bmpdata_s
{
  __u16 addr;        /* Probably  0 (or 8 for field 2) */
  __u16 nelem;       /* Probably 16 (or 8 for one field) */
  __u16 data[16];    /* (See above) */
};

/*************************************************************************
 * IOCTL Commands
 *************************************************************************/

/* FBIOPUT_POS ***********************************************************
 * DESCRIPTION: This command will set window start position to the
 *   coordinates specified by the argument
 * ARGUMENT: A reference to an instance of struct dsc25_pos_s containing
 *   the new position.
 */

#define FBIOPUT_POS (FBIOPAN_DISPLAY + 1)

/* FBIOGET_POS ***********************************************************
 * DESCRIPTION: This command will get the currentwindow start position.
 * ARGUMENT: A reference to an instance of struct dsc25_pos_s in 
 *   which to return the position
 */

#define FBIOGET_POS (FBIOPUT_POS + 1)

/* FBIOPUT_RECTCURSOR_POS ************************************************
 * DESCRIPTION: This command will set rectangular cursor start
 *   position to the coordinates specified by the argument
 * ARGUMENT: A reference to an instance of struct dsc25_pos_s containing
 *   the new position.
 */

#define FBIOPUT_RECTCURSOR_POS (FBIOGET_POS + 1)

/* FBIOGET_RECTCURSOR_POS ************************************************
 * DESCRIPTION: This command will get the current rectangular cursor
 *   start position.
 * ARGUMENT: A reference to an instance of struct dsc25_pos_s in 
 *   which to return the position
 */

#define FBIOGET_RECTCURSOR_POS (FBIOPUT_RECTCURSOR_POS + 1)

/* FBIOPUT_BMPCURSOR_POS ************************************************
 * DESCRIPTION: This command will set bitmap cursor start
 *   position to the coordinates specified by the argument
 * ARGUMENT: A reference to an instance of struct dsc25_bmpcursor_pos_s
 *   containing the new position. In 32x32x4 mode, only field 1 of the
 *   position is used; in 16x16x1 and 16x8x1 modes both field1 and field2
 *   must be supplied.
 */

#define FBIOPUT_BMPCURSOR_POS (FBIOGET_RECTCURSOR_POS + 1)

/* FBIOGET_BMPCURSOR_POS *************************************************
 * DESCRIPTION: This command will get the current bitmap cursor start
 *   position.
 * ARGUMENT: A reference to an instance of struct dsc25_bmpcursor_pos_s in 
 *   which to return the position. In 32x32x4 mode, only field 1 of the
 *   position is returned; in 16x16x1 and 16x8x1 modes both field1 and field2
 *   are returned.
 */

#define FBIOGET_BMPCURSOR_POS (FBIOPUT_BMPCURSOR_POS + 1)

/* FBIOPUT_RECTCURSOR_SIZE ***********************************************
 * DESCRIPTION: This command will set rectangular cursor size
 *  to the value specified by the argument.
 * ARGUMENT: A reference to an instance of struct dsc25_size_s containing
 *   the new position.
 */

#define FBIOPUT_RECTCURSOR_SIZE (FBIOGET_BMPCURSOR_POS + 1)

/* FBIOGET_RECTCURSOR_SIZE ***********************************************
 * DESCRIPTION: This command will get the current rectangular cursor
 *   start position.
 * ARGUMENT: A reference to an instance of struct dsc25_size_s in 
 *   which to return the position
 */

#define FBIOGET_RECTCURSOR_SIZE (FBIOPUT_RECTCURSOR_SIZE + 1)

/* FBIOPUT_MODE **********************************************************
 * DESCRIPTION: This command will set the mode of an OSD window.
 * ARGUMENT:  A reference to an instance of struct dsc25_vidwinmode_s if
 *   the framebuffer corresponds to a video window or to struct
 *   dsc25_osdwinmode_s if the framebuffer corresponds to an OSD window.
 */

#define FBIOPUT_MODE (FBIOGET_RECTCURSOR_SIZE + 1)

/* FBIOGET_MODE **********************************************************
 * DESCRIPTION: This command will get the current mode of an OSD window
 * ARGUMENT:  A reference to an instance of structure in which to
 *   receive the mode settings.  This structure will be struct
 *   dsc25_vidwinmode_s if the framebuffer corresponds to a video window
 *   or a struct dsc25_osdwinmode_s if the framebuffer corresponds to an
 *   OSD window.
 */

#define FBIOGET_MODE (FBIOPUT_MODE + 1)

/* FBIOPUT_RECTCURSOR_MODE ***********************************************
 * DESCRIPTION: This command will set the mode of the rectangular cursor.
 * ARGUMENT: A reference to an instance of struct dsc25_rectcur_s
 *   containing the new window mode.
 */

#define FBIOPUT_RECTCURSOR_MODE (FBIOGET_MODE + 1)

/* FBIOGET_RECTCURSOR_MODE ***********************************************
 * DESCRIPTION: This command will get the current mode of the rectangular
 *   cursor.
 * ARGUMENT: A reference to an instance of struct dsc25_rectcur_s
 *   in which to receive the mode settings.
 */

#define FBIOGET_RECTCURSOR_MODE (FBIOPUT_RECTCURSOR_MODE + 1)

/* FBIOPUT_BMPCURSOR_MODE ************************************************
 * DESCRIPTION: This command will set the mode of the bitmap cursor.
 * ARGUMENT: A reference to an instance of struct dsc25_bmpcur_s
 *   containing the new window mode.
 */

#define FBIOPUT_BMPCURSOR_MODE (FBIOGET_RECTCURSOR_MODE + 1)

/* FBIOGET_BMPCURSOR_MODE ************************************************
 * DESCRIPTION: This command will get the current mode of the bitmap
 *   cursor.
 * ARGUMENT: A reference to an instance of struct dsc25_bmpcur_s
 *   in which to receive the mode settings.
 */

#define FBIOGET_BMPCURSOR_MODE (FBIOPUT_BMPCURSOR_MODE + 1)

/* FBIOPUT_BMPCURSOR_DATA ************************************************
 * DESCRIPTION: This command will set the bitmap of the bitmap cursor.
 * ARGUMENT: A reference to an instance of struct dsc25_bmpdata_s
 *   containing the new bitmap.
 */

#define FBIOPUT_BMPCURSOR_DATA (FBIOGET_BMPCURSOR_MODE + 1)

/* FBIOPUT_BKGRD *********************************************************
 * DESCRIPTION: This command will set the background to the specified
 *   clut value. (This is the same background clut that would be set via
 *   FBIOPUT_OSDMODE).
 * ARGUMENT:  The background clut value (type int).
 */

#define FBIOPUT_BKGRD (FBIOPUT_BMPCURSOR_DATA + 1)

/* FBIOGET_BKGRD *********************************************************
 * DESCRIPTION: This command will get the current background clut value
 *   (This is the same background clut that would be obtained via
 *   FBIOGET_OSDMODE).
 * ARGUMENT:  A pointer to the location to return the background clut
 *   value (type int*).
 */

#define FBIOGET_BKGRD (FBIOPUT_BKGRD + 1)

/* FBIOPUT_OSDMODE *****************************************************
 * DESCRIPTION: This command will set the osd mode as specified by the
 *   argument.
 * ARGUMENT: A reference to an instance of struct dsc25_osdmode_s in
 *   containing the new mode settings.
 */

#define FBIOPUT_OSDMODE (FBIOGET_BKGRD + 1)

/* FBIOGET_OSDMODE *****************************************************
 * DESCRIPTION: This command will return the current osd mode settings
 *   to the location specified by the argument.
 * ARGUMENT:  A reference to an instance of struct dsc25_osdmode_s in
 *   which to receive the mode settings.
 */

#define FBIOGET_OSDMODE (FBIOPUT_OSDMODE + 1)

/* FBIOPUT_VIDEOMODE *****************************************************
 * DESCRIPTION: This command will set the video mode as specified by the
 *   argument.
 * ARGUMENT:  A reference to an instance of struct dsc25_video_mode_s
 */

#define FBIOPUT_VIDEOMODE (FBIOGET_OSDMODE + 1)

/* FBIOGET_VIDEOMODE *****************************************************
 * DESCRIPTION: This command will return the current video mode settings
 *   to the location specified by the argument.
 * ARGUMENT:  A reference to an instance of struct dsc25_video_mode_s
 *   containing the new mode settings.
 */

#define FBIOGET_VIDEOMODE (FBIOPUT_VIDEOMODE + 1)

/* FBIOPUT_PINGPONG ******************************************************
 * DESCRIPTION: This command will select either the "normal" video win0
 *   buffer or the video win0 "ping pong" buffer.
 * ARGUMENT:  An integer value, 0=normal buffer, 1=ping ping buffer
 */

#define FBIOPUT_PINGPONG (FBIOGET_VIDEOMODE + 1)

#endif /* __ASM_ARCH_FB_IOCTL_H_ */
