/************************************************************************
 * asm-armnommu/arch-dsc25/gio_ioctl.h
 ************************************************************************/

#ifndef __ASM_ARCH_GIO_IOCTL_H__
#define __ASM_ARCH_GIO_IOCTL_H__
 
/************************************************************************
 * Included Files
 ************************************************************************/

#include <asm/ioctl.h>
 
/************************************************************************
 * Definitions
 ************************************************************************/

#define GIO_MAGIC_NUMBER 'g'

/* Major numbers, (from Documentation/devices.txt local/experimental list)  */
 
#define GIO_MAJOR 125
 
/************************************************************************
 * GIO_DIRECTION: The GIO_DIRECTION can be either input (GIO_INPUT) or
 * output (GIO_OUTPUT).  If unspecified, the GIO will default to input.
 */

#define GIO_DIRECTION _IO(GIO_MAGIC_NUMBER, 0)

/* GIO_DIRECTION Arguments: */

#define GIO_OUTPUT 0
#define GIO_INPUT  1

/************************************************************************
 * GIO_INVERSION: The GIO polarity of the GIO pin can be inverted.  If
 * unspecified, the default behavior will be NOT inverted.
 */

#define GIO_INVERSION _IO(GIO_MAGIC_NUMBER, 1)

/* GIO_INVERSION Arguments: */

#define GIO_UNINVERTED 0
#define GIO_INVERTED   1

/************************************************************************
 * GIO_BITSET: If the direction is set to output, this ioctl will set the
 * output.  If the direction is input, this ioctl will return the value
 * of the GIO pin (0, 1, or -1 on error).
 */

#define GIO_BITSET _IO(GIO_MAGIC_NUMBER, 2)

/* GIO_BITSET arguments:  None */

/************************************************************************
 * GIO_BITCLR: If the direction is set to output, this ioctl will clear the
 * output.  If the direction is input, this ioctl will return the value
 * of the GIO pin (0, 1, or -1 on error).
 */

#define GIO_BITCLR _IO(GIO_MAGIC_NUMBER, 3)

/* GIO_BITCLR arguments:  None */

/************************************************************************
 * GIO_IRQPORT: For the DSC25, GIO0-GIO7 can be configured as external
 * interrupt ports
 */

#define GIO_IRQPORT _IO(GIO_MAGIC_NUMBER, 4)

/* GIO_IRQPORT arguments:*/

#define GIO_NOINTERRUPT 0
#define GIO_INTERRUPT   1

/************************************************************************
 * GIO_FSEL: For the DSC25 GIO8-GIO28 can be used as dedicated function
 * ports rather than general purpose IO ports.
 */

#define GIO_FSEL _IO(GIO_MAGIC_NUMBER, 5)

/* GIO_FSEL Arguments: */

#define GIO_NORMAL     0 /* 0=GIO8-GIO28 Normal */

/* McBSP 1 has its pins multiplexed tihe ARM GIO8 through GIO14 */

#define GIO_DSP_BFSR1  1 /* 0=GIO8  Normal, 1=GIO8 DSP_BFSR1 */
#define GIO_DSP_BCLKR1 1 /* 0=GIO9  Normal, 1=GIO9 DSP_BCLKR1 */
#define GIO_DSP_BDR1   1 /* 0=GIO10 Normal, 1=GIO10 DSP_BDR1 */
#define GIO_DSP_FSX1   1 /* 0=GIO13 Normal, 1=GIO11 DSP_FSX1 */
#define GIO_DSP_BCLKX1 1 /* 0=GIO12 Normal, 1=GIO12 DSP_BCLKX1 */
#define GIO_DSP_BDX1   1 /* 0=GIO13 Normal, 1=GIO13 DSP_BDX1 */
#define GIO_DSP_CLKS1  1 /* 0=GIO14 Normal, 1=GIO14 DSP_CLKS1 */

/* XF pins multiplexed with GIO 15 */

#define GIO_DSP_XF     1 /* 0=GIO15 Normal, 1=GIO15 DSP_XF */

/* Provisions for setting clock rates in GIO16 and GIO17 */

#define GIO_MXI1       1 /* 0=GIO16 Normal, 1=GIO16 MXI clock output */
#define GIO_MXI2       2 /*   2=GIO16 MXI clock output of div freq 2 */
#define GIO_MXI3       3 /*   3=GIO16 MXI clock output of div freq 3 */

#define GIO_M48XI1     1 /* 0=GIO17 Normal, 1=GIO17 MX48I clock output */
#define GIO_M48XI2     2 /*   2=GIO17 MX48I clock output of div freq 2 */
#define GIO_M48XI3     3 /*   3=GIO17 MX48I clock output of div freq 3 */

#define GIO_SDEN0      1 /* 0=GIO18 Normal, 1=GIO18 SDEN0 */
#define GIO_SDEN1      1 /* 0=GIO19 Normal, 1=GIO19 SDEN1 */
#define GIO_MIRQ       1 /* 0=GIO20 Normal, 1=GIO20 MIRQ */

/* GIO21 is connected to the shutter */

#define GIO_CCDC_SHUT  1 /* 0=GIO21 Normal, 1=GIO21 CCDC_SHUT */

/* GIO22-24 can be associated with the Digital LCD Interface */

#define GIO_EXTLINE_ID 1 /* 0=GIO22 Normal, 1=GIO22 EXTLINE_ID */
#define GIO_PWM_OUT    1 /* 0=GIO23 Normal, 1=GIO23 PWM_OUT */
#define GIO_LCD_AC     1 /* 0=GIO24 Normal, 1=GIO24 LCD_ACI/F) */

#define GIO_SOE        1 /* 0=GIO25 Normal, 1=GIO25 SOE (SSFDC_OE) */
#define GIO_SWE        1 /* 0=GIO26 Normal, 1=GIO26 SWE (SSFDC_WE) */
#define GIO_CLKOUT2    1 /* 0=GIO27 Normal, 1=GIO27 CLKOUT2 output */
#define GIO_CLKOUT3    1 /* 0=GIO28 Normal, 1=GIO28 CLKOUT3 output */

/************************************************************************
 * GIO_BITRATE: Forward/Transfer rate set bit.  GIO_FSEL has provisions
 * for setting clock rates in GIO16 and GIO17.  This ioctl will set
 * the division ratio of clock output of GIO16 or GIO17 when the pin is
 * is set to GIO divided clock output.  This will control the clock output
 * from CLKOUT2 (GIO16 only and only if GIO_MXI1 is selected) or from
 * CLKOUT3 (GIO17 only and only if GIO_M48XI1 is selected).  The following
 * clock rate is output:
 *
 *   Clock Rate = ARM clock / set value + 1x2
 *
 * This ioctl will set the BITRATE register without regard to the current
 * function setting of GIO16 or GIO17.
 */

#define GIO_BITRATE _IO(GIO_MAGIC_NUMBER, 6)

/* GIO_BITRATE Arguments: division ratio; "set value" in the above formula */

#define GIO_IOC_MAXNR 6
 
#endif /* __ASM_ARCH_GIO_IOCTL_H__ */
