/*
 * linux/include/asm-nommu/arch-dsc25/hardware.h
 * Definitions for the DSC25.
 *
 * Copyright (C) 2002 Cadenux, LLC (http://www.cadenux.com)
 * author:  Gregory Nutt <greg.nutt@cadenux.com>
 *
 * Modified from linux/include/asm-nommu/arch-dsc21/hardware.h
 *    
 *  Copyright (C) 2001 RidgeRun, Inc. (http://www.ridgerun.com)
 *  author: Gordon McNutt <gmcnutt@ridgerun.com>
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __ASM_ARCH_HARDWARE_H
#define __ASM_ARCH_HARDWARE_H

#define KERNTOPHYS(a) ((unsigned long)(&a))

/* ARM clock rate (MHz) */

#define DSC25_CLOCK_RATE     37125000

/* Memory mapped registers */

#define TIMER0_REGISTER_BASE    0x00030000 /* Timer0 */
#define TIMER1_REGISTER_BASE	0x00030080 /* Timer1 */
#define TIMER2_REGISTER_BASE	0x00030100 /* Timer2 */
#define TIMER3_REGISTER_BASE	0x00030180 /* Timer3 */
#define SERIAL0_REGISTER_BASE	0x00030200 /* Serial0 */
#define SERIAL1_REGISTER_BASE	0x00030280 /* Serial1 */
#define UART0_REGISTER_BASE	0x00030300 /* UART0 */
#define UART1_REGISTER_BASE	0x00030380 /* UART1 */
#define WDT_REGISTER_BASE	0x00030400 /* Watch dog timer */
#define USB_REGISTER_BASE	0x00030480 /* USB */
#define INTCTRL_REGISTER_BASE	0x00030500 /* INTC */
#define GIO_REGISTER_BASE	0x00030580 /* GIO */
#define HPIB_REGISTER_BASE      0x00030600 /* DSP Controller (HPIB) */
#define OSD_REGISTER_BASE	0x00030680 /* OSD */
#define CCDC_REGISTER_BASE	0x00030700 /* CCD controller */
#define PREVIEW_REGISTER_BASE	0x00030780 /* Preview engine */
#define VIDEOENC_REGISTER_BASE	0x00030800 /* NTSC/PAL video encoder */
#define CLOCKC_REGISTER_BASE	0x00030880 /* Clock controller */
#define EXTCPU_REGISTER_BASE	0x00030900 /* External Host IF */
#define SDRAMC_REGISTER_BASE	0x00030980 /* SDRAM controller */
#define EXBC_REGISTER_BASE	0x00030a00 /* External Memory Interface */
#define MMCSD_REGISTER_BASE     0x00030a80 /* MMC/SD */
#define MEMSTICK_REGISTER_BASE  0x00030b00 /* Memory Stick */

/* Uarts (note that the register fields aren't needed by the standard linux
 * serial driver -- we only use them for our initial debugging console)
 */

#define DSC25_UART0          0x00030300
#define DSC25_UART0_DTRR     0x00030300
#define DSC25_UART0_BRSR     0x00030302
#define DSC25_UART0_MSR      0x00030304
#define DSC25_UART0_RFCR     0x00030306
#define DSC25_UART0_TFCR     0x00030308
#define DSC25_UART0_LCR      0x0003030a
#define DSC25_UART0_SR       0x0003030c

#define DSC25_UART1          0x00030380
#define DSC25_UART1_DTRR     0x00030380
#define DSC25_UART1_BRSR     0x00030382
#define DSC25_UART1_MSR      0x00030384
#define DSC25_UART1_RFCR     0x00030386
#define DSC25_UART1_TFCR     0x00030388
#define DSC25_UART1_LCR	     0x0003038a
#define DSC25_UART1_SR 	     0x0003038c

/* Timers */

#define DSC25_TIMER0_MODE    0x00030000
#define DSC25_TIMER0_SEL     0x00030002
#define DSC25_TIMER0_SCAL    0x00030004
#define DSC25_TIMER0_DIV     0x00030006
#define DSC25_TIMER0_TRG     0x00030008
#define DSC25_TIMER0_CNT     0x0003000a
#define UCLINUX_TIMER0_CNT   DSC25_TIMER0_CNT

#define DSC25_TIMER1_MODE    0x00030080
#define DSC25_TIMER1_SEL     0x00030082
#define DSC25_TIMER1_SCAL    0x00030084
#define DSC25_TIMER1_DIV     0x00030086
#define DSC25_TIMER1_TRG     0x00030088
#define DSC25_TIMER1_CNT     0x0003008a

#define DSC25_TIMER2_MODE    0x00030100
#define DSC25_TIMER2_SEL     0x00030102
#define DSC25_TIMER2_SCAL    0x00030104
#define DSC25_TIMER2_DIV     0x00030106
#define DSC25_TIMER2_TRG     0x00030108
#define DSC25_TIMER2_CNT     0x0003010a

#define DSC25_TIMER3_MODE    0x00030180
#define DSC25_TIMER3_SEL     0x00030182
#define DSC25_TIMER3_SCAL    0x00030184
#define DSC25_TIMER3_DIV     0x00030186
#define DSC25_TIMER3_TRG     0x00030188
#define DSC25_TIMER3_CNT     0x0003018a

/* Watchdog */

#define DSC25_WDTMD          0x00030400
#define DSC25_WDTRST         0x00030402
#define DSC25_WDTPRSCL       0x00030404
#define DSC25_WDTDIV         0x00030406

#define WDT_BLOCK_SIZE       0x00000008

/* Interrupt */

#define DSC25_FIQ0_STATUS    0x00030500
#define DSC25_FIQ1_STATUS    0x00030502
#define DSC25_IRQ0_STATUS    0x00030504
#define DSC25_IRQ1_STATUS    0x00030506
#define DSC25_FIQ0_ENTRY     0x00030508
#define DSC25_FIQ1_ENTRY     0x0003050A
#define DSC25_IRQ0_ENTRY     0x0003050C
#define DSC25_IRQ1_ENTRY     0x0003050E
#define DSC25_FIQ0_SELECT    0x00030520
#define DSC25_FIQ1_SELECT    0x00030522
#define DSC25_IRQ0_ENABLE    0x00030524
#define DSC25_IRQ1_ENABLE    0x00030526
#define DSC25_INT_RAW        0x00030528
#define DSC25_EA0_BASE       0x00030530
#define DSC25_EA1_BASE       0x00030532
#define DSC25_INT_PRI_ARRAY  0x00030540 /* Through  0x0003055A */

/* GIO  */

#define DSC25_GIO_DIR0       0x00030580
#define DSC25_GIO_DIR1       0x00030582
#define DSC25_GIO_INV0       0x00030584
#define DSC25_GIO_INV1       0x00030586
#define DSC25_GIO_BITSET0    0x00030588
#define DSC25_GIO_BITSET1    0x0003058a
#define DSC25_GIO_BITCLR0    0x0003058c
#define DSC25_GIO_BITCLR1    0x0003058e
#define DSC25_GIO_IRQPORT    0x00030590
#define DSC25_GIO_FSEL0      0x00030592
#define DSC25_GIO_FSEL1      0x00030594
#define DSC25_GIO_BITRATE0   0x00030596
#define DSC25_GIO_BITRATE1   0x00030598

#define GIO_BLOCK_SIZE       0x0000001c

#define GIO_0_BIT      (1 << 0)
#define GIO_1_BIT      (1 << 1)
#define GIO_2_BIT      (1 << 2)
#define GIO_3_BIT      (1 << 3)
#define GIO_4_BIT      (1 << 4)
#define GIO_5_BIT      (1 << 5)
#define GIO_6_BIT      (1 << 6)
#define GIO_7_BIT      (1 << 7)
#define GIO_ALL_BITS   0xff

#define MAX_GIO 31
#define MAX_I2C 31

/* HPIB */

#define DSC25_HPIBCTL        0x00030600 /* HPIB Control Register */
#define DSC25_HPIBSTAT       0x00030602 /* HPIB Status Register */

#define HPIB_BLOCK_SIZE      0x00000004

/* OSD */

#define DSC25_OSDMODE        0x00030680 /* OSD Mode Setup */
#define DSC25_VIDWINMD       0x00030682 /* Video Window Mode Setup */
#define DSC25_OSDWIN0MD      0x00030684 /* OSD Win0 Mode Setup */
#define DSC25_OSDWIN1MD      0x00030686 /* OSD Win1/Attrib Window Mode Setup */
#define DSC25_RECTCUR        0x00030688 /* Rectangular Cursor Setup */
#define DSC25_BMPCUR         0x0003068a /* Bitmap Cursor Setup */
#define DSC25_VIDWIN0OFST    0x0003068c /* Video Win0 Offset */
#define DSC25_VIDWIN1OFST    0x0003068e /* Video Win1 Offset */
#define DSC25_OSDWIN0OFST    0x00030690 /* OSD Win0 Offset */
#define DSC25_OSDWIN1OFST    0x00030692 /* OSD Win1 Offset */
#define DSC25_VIDWINADH      0x00030694 /* Video Win0/1 AddressHigh */
#define DSC25_VIDWIN0ADL     0x00030696 /* Video Win0 AddressLow */
#define DSC25_VIDWIN1ADL     0x00030698 /* Video Win1 AddressLow */
#define DSC25_OSDWINADH      0x0003069a /* OSD Win0/1 AddressHigh */
#define DSC25_OSDWIN0ADL     0x0003069c /* OSD Win0 AddressLow */
#define DSC25_OSDWIN1ADL     0x0003069e /* OSD Win1 AddressLow */
#define DSC25_BASEPX         0x000306a0 /* Base Pixel X */
#define DSC25_BASEPY         0x000306a2 /* Base Pixel Y */
#define DSC25_VIDWIN0XP      0x000306a4 /* Video Win0 X-Pos  */
#define DSC25_VIDWIN0YP      0x000306a6 /* Video Win0 Y-Pos  */
#define DSC25_VIDWIN0XL      0x000306a8 /* Video Win0 X-Size  */
#define DSC25_VIDWIN0YL      0x000306aa /* Video Win0 Y-Size  */
#define DSC25_VIDWIN1XP      0x000306ac /* Video Win1 X-Pos  */
#define DSC25_VIDWIN1YP      0x000306ae /* Video Win1 Y-Pos  */
#define DSC25_VIDWIN1XL      0x000306b0 /* Video Win1 X-Size  */
#define DSC25_VIDWIN1YL      0x000306b2 /* Video Win1 Y-Size  */
#define DSC25_OSDWIN0XP      0x000306b4 /* OSD Bitmap Win0 X-Pos */
#define DSC25_OSDWIN0YP      0x000306b6 /* OSD Bitmap Win0 Y-Pos */
#define DSC25_OSDWIN0XL      0x000306b8 /* OSD Bitmap Win0 X-Size */
#define DSC25_OSDWIN0YL      0x000306ba /* OSD Bitmap Win0 Y-Size */
#define DSC25_OSDWIN1XP      0x000306bc /* OSD Bitmap Win1 X-Pos */
#define DSC25_OSDWIN1YP      0x000306be /* OSD Bitmap Win1 Y-Pos */
#define DSC25_OSDWIN1XL      0x000306c0 /* OSD Bitmap Win1 X-Size */
#define DSC25_OSDWIN1YL      0x000306c2 /* OSD Bitmap Win1 Y-Size */
#define DSC25_CURXP          0x000306c4 /* Rectangular Cursor Window X-Pos */ 
#define DSC25_CURYP          0x000306c6 /* Rectangular Cursor Window Y-Pos */ 
#define DSC25_CURXL          0x000306c8 /* Rectangular Cursor Window X-Size */
#define DSC25_CURYL          0x000306ca /* Rectangular Cursor Window Y-Size */
#define DSC25_BMPCURXP       0x000306cc /* Bitmap Cursor Window X-Pos */
#define DSC25_BMPCURYP       0x000306ce /* Bitmap Cursor Window Y-Pos */
#define DSC25_W0BMP01        0x000306d0 /* Win0 Bitmap to Palette Map 0/1 */
#define DSC25_WOBMP23        0x000306d2 /* Win0 Bitmap to Palette Map 2/3 */
#define DSC25_WOBMP45        0x000306d4 /* Win0 Bitmap to Palette Map 4/5 */
#define DSC25_WOBMP67        0x000306d6 /* Win0 Bitmap to Palette Map 6/7 */
#define DSC25_WOBMP89        0x000306d8 /* Win0 Bitmap to Palette Map 8/9 */
#define DSC25_WOBMPAB        0x000306da /* Win0 Bitmap to Palette Map a/b */
#define DSC25_WOBMPCD        0x000306dc /* Win0 Bitmap to Palette Map c/d */
#define DSC25_WOBMPEF        0x000306de /* Win0 Bitmap to Palette Map e/f */
#define DSC25_W1BMP01        0x000306e0 /* Win1 Bitmap to Palette Map 0/1 */
#define DSC25_W1BMP23        0x000306e2 /* Win1 Bitmap to Palette Map 2/3 */
#define DSC25_W1BMP45        0x000306e4 /* Win1 Bitmap to Palette Map 4/5 */
#define DSC25_W1BMP67        0x000306e6 /* Win1 Bitmap to Palette Map 6/7 */
#define DSC25_W1BMP89        0x000306e8 /* Win1 Bitmap to Palette Map 8/9 */
#define DSC25_W1BMPAB        0x000306ea /* Win1 Bitmap to Palette Map a/b */
#define DSC25_W1BMPCD        0x000306ec /* Win1 Bitmap to Palette Map c/d */
#define DSC25_W1BMPEF        0x000306ee /* Win1 Bitmap to Palette Map e/f */
#define DSC25_BMPCURDAT      0x000306f2 /* Bitmap Cursor Data Setup */
#define DSC25_MISCCTL        0x000306f4 /* Miscellaneous Control */
#define DSC25_CLUTRAMYCB     0x000306f6 /* CLUT RAM Y/Cb Setup */
#define DSC25_CLUTRAMCR      0x000306f8 /* CLUT RAM Cr/Mapping Setup */
#define DSC25_PPVWIN0ADH     0x000306fc /* Ping-Pong Video Win0 Address-High */
#define DSC25_PPVWIN0ADL     0x000306fe /* Ping-Pong Video Win0 Address-Low */

#define OSD_BLOCK_SIZE       0x00000080

/* NTSC/PAL Video Encoder Registers */

#define DSC25_VID01          0x00030800 /* Video Mode Setup #1 */
#define DSC25_VID02          0x00030802 /* Video Mode Setup #2 */
#define DSC25_DLCD1          0x00030804 /* Digital LCD Mode Setup #1 */
#define DSC25_DLCD2          0x00030806 /* Digital LCD Mode Setup #2 */
#define DSC25_DCLKPTN0       0x00030808 /* DCLK Pattern Register #0 */
#define DSC25_DCLKPTN1       0x0003080a /* DCLK Pattern Register #1 */
#define DSC25_DCLKPTN2       0x0003080c /* DCLK Pattern Register #2 */
#define DSC25_DCLKPTN3       0x0003080e /* DCLK Pattern Register #3 */
#define DSC25_DCLKSTPH       0x00030810 /* DCLK Enable Start Position in H */
#define DSC25_DCLKVLDH       0x00030812 /* DCLK Valid Area in H */
#define DSC25_DCLKSTPV       0x00030814 /* DCLK Enable Start Position in V */
#define DSC25_DCLKVLDV       0x00030816 /* DCLK Valid Area in V */
#define DSC25_HVPWIDTH       0x00030818 /* H and V Sync Pulse Width Control */
#define DSC25_HINTERVL       0x0003081a /* H Sync Interval Control */
#define DSC25_HSTART         0x0003081c /* H Start Position Control */
#define DSC25_HVALID         0x0003081e /* H Valid Area Control */
#define DSC25_VINTERVL       0x00030820 /* V Sync Interval Control */
#define DSC25_VSTART         0x00030822 /* V Start Position Control */
#define DSC25_VVALID         0x00030824 /* V Valid Area Control */
#define DSC25_HDELAY         0x00030826 /* H Sync Delay */
#define DSC25_VDELAY         0x00030828 /* V Sync Delay */
#define DSC25_CULLLINE       0x0003082a /* Line Culling Control */
#define DSC25_PWMCTRL        0x0003082c /* PWM Control */
#define DSC25_PWMCNT         0x0003082e /* PWM Count */
#define DSC25_RGBLEVEL       0x00030830 /* RGB Level Control */

#define VIDEOENC_BLOCK_SIZE  0x00000032

/* Clock Controller */

#define DSC25_CLKC_PLLARM    0x00030880 /* ARM PLL Config & Lock Monitor */
#define DSC25_CLKC_PLLDSP    0x00030882 /* DSP PLL Config & Lock Monitor */
#define DSC25_CLCK_PLLSDR    0x00030884 /* SDRAM PLL Config & Lock Monitor */
#define DSC25_CLKC_CLKC      0x00030886 /* Input Clock Source Selection */
#define DSC25_CLKC_DIV       0x00030888 /* PLL Clock Divisor Settings */
#define DSC25_CLKC_MMCCLK    0x0003088a /* MMC Clock Divisor Settings */
#define DSC25_CLKC_MSCLK     0x0003088c /* MemStick Clock Divisor Settings */
#define DSC25_CLKC_BYP       0x0003088e /* PLL Bypass Configuration */
#define DSC25_CLKC_LPCTL1    0x00030890 /* Low Power Control #1 */
#define DSC25_CLKC_LPCTL2    0x00030892 /* Low Power Control #2 */
#define DSC25_CLKC_MOD1      0x00030894 /* Module Clock Enables #1 */
#define DSC25_CLKC_MOD2      0x00030896 /* Module Clock Enables #2 */
#define DSC25_CLKC_PLLSETA   0x00030898 /* ARM PLL Capacity Control */
#define DSC25_CLKC_PLLSETD   0x0003089a /* DSP PLL Capacity Control */
#define DSC25_CLKC_PLLSETS   0x0003089c /* SDRAM PLL Capacity Control */

/* External bus */

#define DSC25_EXTBUS_RESET   0x00030a00
#define DSC25_EXTBUS_MODE    0x00030a04
#define DSC25_EXTBUS_INTEN   0x00030a08

/* MMC/SD */

#define DSC25_MMCCTL         0x00030a80 /* MMC Control Register */
#define DSC25_MMCCLK         0x00030a82 /* MMC Memory Clock Register */
#define DSC25_MMCST0         0x00030a84 /* MMC Status Register #0 */
#define DSC25_MMCST1         0x00030a86 /* MMC Status Register #1 */
#define DSC25_MMCIE          0x00030a88 /* MMC Interrupt Enable Register */
#define DSC25_MMCTOR         0x00030a8a /* MMC Response Time-Out Register */
#define DSC25_MMCTOD         0x00030a8c /* MMC Data Read Time-Out Register */
#define DSC25_MMCBLEN        0x00030a8e /* MMC Block Length Register */
#define DSC25_MMCNBLK        0x00030a90 /* MMC Number of Blocks Register */
#define DSC25_MMCNBLC        0x00030a92 /* MMC Number of Blocks Counter */
#define DSC25_MMCDRR         0x00030a94 /* MMC Data Receive Register */
#define DSC25_MMCDXR         0x00030a96 /* MMC Data Transmit Register */
#define DSC25_MMCCMD         0x00030a98 /* MMC Command Register */
#define DSC25_MMCARGL        0x00030a9a /* MMC Argument - Low Register */
#define DSC25_MMCARGH        0x00030a9c /* MMC Argument - High Register */
#define DSC25_MMCRSP0        0x00030a9e /* MMC Response Register #0 */
#define DSC25_MMCRSP1        0x00030aa0 /* MMC Response Register #1 */
#define DSC25_MMSRPS2        0x00030aa2 /* MMC Response Register #2 */
#define DSC25_MMCRSP3        0x00030aa4 /* MMC Response Register #3 */
#define DSC25_MMCRSP4        0x00030aa6 /* MMC Response Register #4 */
#define DSC25_MMCRSP5        0x00030aa8 /* MMC Response Register #5 */
#define DSC25_MMCRSP6        0x00030aaa /* MMC Response Register #6 */
#define DSC25_MMCRSP7        0x00030aac /* MMC Response Register #7 */
#define DSC25_MMCDRSP        0x00030aae /* MMC Data Response Register */
#define DSC25_MMCETOK        0x00030ab0 /* MMC SPI Error Token Register */
#define DSC25_MMCCIDX        0x00030ab2 /* MMC Command Index Register */
#define DSC25_MMCCKC         0x00030ab4 /* Test Register */
#define DSC25_MMCTORC        0x00030ab6 /* Test Register */
#define DSC25_MMCTODC        0x00030ab8 /* Test Register */
#define DSC25_MMCBLNC        0x00030aba /* Test Register */
#define DSC25_DMATRG         0x00030abc /* DMA Trigger Register */
#define DSC25_DMAMODE        0x00030abe /* DMA Mode Register */
#define DSC25_DMAAD0         0x00030ac0 /* DMA Address Low */
#define DSC25_DMAAD1         0x00030ac2 /* DMA Address High */
#define DSC25_DMASTAT0       0x00030ac4 /* DMA Status Register #0 */
#define DSC25_DMASTAT1       0x00030ac6 /* DMA Status Register #1 */
#define DSC25_DMATOR         0x00030ac8 /* DMA Time-Out Register */

#define MMCSD_BLOCK_SIZE     0x0000004a

/* HARD_RESET_NOW -- used in blkmem.c. Should call arch_hard_reset(). */

#define HARD_RESET_NOW()

/* DSPRAM
 *
 * The DSP internal memory (32-Kwords of DARAM) can be read and written
 * by the ARM or an external host through the HPI-Bridge: Addresses
 * 0x0-0x7FFFh of DSP DARAM correspond to addresses 0004:0000-0004:FFFF
 * in the ARM memory map.  Addresses 0x0-0x7FFFh of DSP DARAM correspond
 * to addresses 000 1:0000-0001:FFFF in the external host memory map. 
 */

#define DSPRAM_ARM_BASE     0x00040000  /* DSPRAM ARM base address */
#define DSPRAM_ARM_END      0x0004ffff

#define DSPRAM_EXT_BASE     0x00100000  /* DSPRAM external host base address */
#define DSPRAM_EXT_END      0x0010ffff

/* SDRAM address (real byte address) */

#define SDRAM_BASE          0x04400000  /* sdram base address */
#define SDRAM_SIZE          0x00800000  /* hard-code 8MB for now */
#define SDRAM_PREVIEW       0x00000000  /* output from preview engine */
#define SDRAM_RAWDATA       0x00100000  /* output from CCDC */
#define SDRAM_WORK1         0x00100000  /* WORK area #1 for burst-capture & decoded data */
#define SDRAM_WORK2         0x00500000  /* WORK area #2 for burst-capture & decoded data */
#define SDRAM_ENCODED1      0x00900000  /* work area #1 for encoded data */
#define SDRAM_ENCODED2      0x00B00000  /* work area #2 for encoded data */
#define SDRAM_BURSTDATA     0x00B00000  /* BURST-compressed data start address */
#define SDRAM_BURSTDATA_END 0x00EFFFFF  /* BURST-compressed data end address */
#define SDRAM_OSD           0x00F00000  /* OSD bitmap start address */

/* CARD map */

#define CFC_CARD_BASE       0x06100000  /* CFC card base address */
#define CFC_CARD_SLOT_SIZE  0x02000000  /* CFC image slot size */
#define ETH_CARD_BASE       0x06b00000  /* Ethernet card base address */
#define ETH_CARD_SLOT_SIZE  0x00004000  /* Ethernet image slot size */

/* External Memory Start Address registers */

#define DSC25_EMST0         0x00030922 /* EM0 Region Start - Flash */
#define DSC25_EMST1         0x00030924 /* EM1 Region Start - CompactFlash */
#define DSC25_EMST2         0x00030926 /* EM2 Region Start - SmartMedia */
#define DSC25_EMST3         0x00030928 /* EM3 Region Start - Unused */
#define DSC25_EMST4         0x0003092a /* EM4 Region Start - Ethernet */

/* External Memory Size registers */

#define DSC25_EMSZ0         0x00030942 /* EM0 Region Size - Flash */
#define DSC25_EMSZ1         0x00030944 /* EM1 Region Size - CompactFlash */
#define DSC25_EMSZ2         0x00030946 /* EM2 Region Size - SmartMedia */
#define DSC25_EMSZ3         0x00030948 /* EM3 Region Size - Unused */
#define DSC25_EMSZ4         0x0003094a /* EM3 Region Size - Ethernet */

/* External memory start address values.  Starting addresses for
 * external memory regions is 0010:0000 + VALUE * 0010:0000
 */

#define DSC25_EMST0_DATA    0x00       /* 0010:0000 - Flash */
#define DSC25_EMST1_DATA    0x60       /* 0610:0000 - Compact Flash */
#define DSC25_EMST2_DATA    0x68       /* 0690:0000 - SmartMedia */
#define DSC25_EMST3_DATA    0x69       /* 06a0:0000 - Unused */
#define DSC25_EMST4_DATA    0x6a       /* 06b0:0000 - Ethernet */

/* External memory size values.  Size of an external memory region
 * is VALUE * 256Kbytes.
 */

#define DSC25_EMSZ0_DATA    16         /*   4 Mbytes */
#define DSC25_EMSZ1_DATA    32         /*   8 Mbyte */
#define DSC25_EMSZ2_DATA    4          /*   1 Mbyte */
#define DSC25_EMSZ3_DATA    4          /*   1 Mbyte */
#define DSC25_EMSZ4_DATA    1          /* 256 Kbytes */

/* External memory control registers */

#define DSC25_CS0CTRL1      0x00030a00 /* EM0 Control register #1 */
#define DSC25_CS0CTRL2      0x00030a02 /* EM0 Control register #2 */
#define DSC25_CS4CTRL1      0x00030a12 /* EM4 Control register #1 */
#define DSC25_CS4CTRL2      0x00030a14 /* EM4 Control register #2 */

/* External memory control register values */

#define DSC25_CS0CTRL1_DATA 0x2222     /* 3 clks for 90 ns at 27 mhz */
#define DSC25_CS0CTRL2_DATA 0x1000     /* 1 clk idle */
#define DSC25_CS4CTRL1_DATA 0x889a     /* REVISIT: not tuned */
#define DSC25_CS4CTRL2_DATA 0x5220     /* REVISIT: not tuned */

/* EM1/CompactFlash external memory control registers */

#define DSC25_BUSCTRL       0x00030a16  /* CFC bus control register */
#define DSC25_CFCTRL1       0x00030a1a  /* CFC control register #1 */
#define DSC25_CFCTRL2       0x00030a1c  /* CFC control register #2 */
#define DSC25_BUSINTEN      0x00030a20  /* CFC bus interrupt enable register */
#define DSC25_BUSINTSTS     0x00030a22  /* CFC bus interrupt status register */
#define DSC25_BUSWAITMD     0x00030a26  /* CFC bus wait mode register */

/* EM1/CompactFlash external memory control register values */

#define DSC25_BUSINTEN_ALL  0x000f      /* All interrupt enable bits */
#define DSC25_BUSINTSTS_ALL 0x000f      /* All interrupt status bits */

#endif  /* __ASM_ARCH_HARDWARE_H */
