/*
 * linux/include/asm-nommu/arch-dsc24/hardware.h
 * Definitions for the DSC24.
 *
 *   Copyright (C) 2003 Cadenux, LLC (http://www.cadenux.com)
 *   author:  Gregory Nutt <greg.nutt@cadenux.com>
 *
 * Derived from: linux/include/asm-nommu/arch-dsc21/hardware.h
 *
 *   Copyright (C) 2001 RidgeRun, Inc. (http://www.ridgerun.com)
 *   author: Gordon McNutt <gmcnutt@ridgerun.com>
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 * WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 * USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __ASM_ARCH_HARDWARE_H
#define __ASM_ARCH_HARDWARE_H

#define KERNTOPHYS(a) ((unsigned long)(&a))

/* ARM clock rate (MHz) */


#define DSC24_CLOCK_RATE       30412800

/* External bus */

#define DSC24_EXTBUS_RESET    0x00030a00
#define DSC24_EXTBUS_MODE     0x00030a04
#define DSC24_EXTBUS_INTEN    0x00030a08

/* Uarts (note that the register fields aren't needed by the standard linux
 * serial driver -- we only use them for our initial debugging console)
 */

#define DSC24_UART0       0x00030380
#define DSC24_UART0_DTRR  0x00030380
#define DSC24_UART0_BRSR  0x00030382
#define DSC24_UART0_MSR   0x00030384
#define DSC24_UART0_RFCR  0x00030386
#define DSC24_UART0_TFCR  0x00030388
#define DSC24_UART0_LCR	  0x0003038a
#define DSC24_UART0_SR 	  0x0003038c

#define DSC24_UART1       0x00030400
#define DSC24_UART1_DTRR  0x00030400
#define DSC24_UART1_BRSR  0x00030402
#define DSC24_UART1_MSR   0x00030404
#define DSC24_UART1_RFCR  0x00030406
#define DSC24_UART1_TFCR  0x00030408
#define DSC24_UART1_LCR	  0x0003040a
#define DSC24_UART1_SR 	  0x0003040c

/* Timers */

#define DSC24_TIMER0_MODE    0x00030000
#define DSC24_TIMER0_SEL     0x00030002
#define DSC24_TIMER0_SCAL    0x00030004
#define DSC24_TIMER0_DIV     0x00030006
#define DSC24_TIMER0_TRG     0x00030008
#define DSC24_TIMER0_CNT     0x0003000a
#define UCLINUX_TIMER0_CNT   DSC24_TIMER0_CNT

#define DSC24_TIMER1_MODE    0x00030080
#define DSC24_TIMER1_SEL     0x00030082
#define DSC24_TIMER1_SCAL    0x00030084
#define DSC24_TIMER1_DIV     0x00030086
#define DSC24_TIMER1_TRG     0x00030088
#define DSC24_TIMER1_CNT     0x0003008a

#define DSC24_TIMER2_MODE    0x00030100
#define DSC24_TIMER2_SEL     0x00030102
#define DSC24_TIMER2_SCAL    0x00030104
#define DSC24_TIMER2_DIV     0x00030106
#define DSC24_TIMER2_TRG     0x00030108
#define DSC24_TIMER2_CNT     0x0003010a

#define DSC24_TIMER3_MODE    0x00030180
#define DSC24_TIMER3_SEL     0x00030182
#define DSC24_TIMER3_SCAL    0x00030184
#define DSC24_TIMER3_DIV     0x00030186
#define DSC24_TIMER3_TRG     0x00030188
#define DSC24_TIMER3_CNT     0x0003018a

/* Interrupt */

#define DSC24_FIQ_STATUS     0x00030580
#define DSC24_IRQ0_STATUS    0x00030582
#define DSC24_IRQ1_STATUS    0x00030584
#define DSC24_FIQ_ENABLE     0x000305a0
#define DSC24_IRQ0_ENABLE    0x000305a2
#define DSC24_IRQ1_ENABLE    0x000305a4
#define DSC24_INTIDR         0x000305c0

/* GIO  */

#define DSC24_GIO_DIR0       0x00030600
#define DSC24_GIO_DIR1       0x00030602
#define DSC24_GIO_INV0       0x00030604
#define DSC24_GIO_INV1       0x00030606
#define DSC24_GIO_BITSET0    0x00030608
#define DSC24_GIO_BITSET1    0x0003060a
#define DSC24_GIO_BITCLR0    0x0003060c
#define DSC24_GIO_BITCLR1    0x0003060e
#define DSC24_GIO_IRQPORT    0x00030610
#define DSC24_GIO_FSEL       0x00030612
#define DSC24_GIO_BITRATE    0x00030614

#define GIO_BLOCK_SIZE       0x00000016

#define GIO_0_BIT      (1 << 0)
#define GIO_1_BIT      (1 << 1)
#define GIO_2_BIT      (1 << 2)
#define GIO_3_BIT      (1 << 3)
#define GIO_4_BIT      (1 << 4)
#define GIO_5_BIT      (1 << 5)
#define GIO_6_BIT      (1 << 6)
#define GIO_7_BIT      (1 << 7)
#define GIO_ALL_BITS   0xff

#define MAX_GIO 20
#define MAX_I2C 31

/* Watchdog */

#define DSC24_WDTMD          0x00030300
#define DSC24_WDTRST         0x00030302
#define DSC24_WDTPRSCL       0x00030304
#define DSC24_WDTDIV         0x00030306

#define WDT_BLOCK_SIZE       0x00000008

/* HARD_RESET_NOW -- used in blkmem.c. Should call arch_hard_reset(). */

#define HARD_RESET_NOW()

/* Memory mapped registers */

#define TIMER0_REGISTER_BASE    0x00030000 /* Timer0 */
#define TIMER1_REGISTER_BASE	0x00030080 /* Timer1 */
#define TIMER2_REGISTER_BASE	0x00030100 /* Timer2 */
#define TIMER3_REGISTER_BASE	0x00030180 /* Timer3 */
#define SERIAL0_REGISTER_BASE	0x00030200 /* Serial0 */
#define SERIAL1_REGISTER_BASE	0x00030280 /* Serial1 */
#define UART0_REGISTER_BASE	0x00030380 /* UART0 */
#define UART1_REGISTER_BASE	0x00030400 /* UART0 */
#define USB_REGISTER_BASE	0x00030480 /* USB */
#define WDT_REGISTER_BASE	0x00030500 /* Watch dog timer */
#define INTCTRL_REGISTER_BASE	0x00030580 /* INTC */
#define GIO_REGISTER_BASE	0x00030600 /* GIO */
#define OSD_REGISTER_BASE	0x00030680 /* OSD */
#define DSPC_REGISTER_BASE	0x00030700 /* DSP controller */
#define VIDEO_IF_REGISTER_BASE	0x00030780 /* Video interface controller */
#define BURSTC_REGISTER_BASE	0x00030800 /* Burst mode compression */
#define CLOCKC_REGISTER_BASE	0x00030880 /* Clock controller */
#define BUSC_REGISTER_BASE      0x00030900 /* Bus controller */
#define SDRAMC_REGISTER_BASE	0x00030980 /* SDRAM controller */
#define EXBC_REGISTER_BASE	0x00030A00 /* External bus controller */

/* DSPRAM */

#define DSPRAM_BASE		0x00040000 /* DSPRAM base address */
#define DSPRAM_END		0x0004ffff

/* SDRAM address (real byte address) */

#define SDRAM_BASE		0x00900000
#define SDRAM_SIZE		0x01000000 /* Hard-code 16MB for now */

#endif  /* __ASM_ARCH_HARDWARE_H */
